<?php
/**
 * Plugin Name: Cooked Pro
 * Plugin URI: https://cooked.pro
 * Description: The premium upgrade for Cooked.
 * Author: Boxy Studio
 * Author URI: https://boxystudio.com
 * Version: 1.7.5.4
 * Text Domain: cooked-pro
 * Domain Path: languages
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;
define( 'COOKED_PRO_VERSION', '1.7.5.4' );

if ( ! class_exists( 'Cooked_Pro_Plugin' ) ) :

// Include license file for updates
define( 'COOKED_PRO_FILE', __FILE__ );
require_once trailingslashit( plugin_dir_path( __FILE__ ) ) . 'license/license.php';

/**
 * Cooked_Pro_Plugin Class.
 *
 * @since 1.0.0
 */
final class Cooked_Pro_Plugin {

	/**
	 * @var Cooked_Pro_Plugin
	 * @since 1.0.0
	 */
	private static $instance;

	/**
	 * Main Cooked_Pro_Plugin Instance.
	 *
	 * Insures that only one instance of Cooked_Pro_Plugin exists in memory at any one
	 * time. Also prevents needing to define globals everywhere.
	 *
	 * @since 1.0.0
	 * @static
	 * @staticvar array $instance
	 * @uses Cooked_Pro_Plugin::setup_constants() Setup the constants needed.
	 * @uses Cooked_Pro_Plugin::includes() Include the required files.
	 * @uses Cooked_Pro_Plugin::load_textdomain() load the language files.
	 * @see Cooked()
	 * @return object|Cooked_Pro_Plugin
	 */

	public static function instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Cooked_Pro_Plugin ) ) {

			self::$instance = new Cooked_Pro_Plugin;
			self::$instance->setup_constants();
			self::$instance->includes();

			add_action( 'plugins_loaded', array( self::$instance, 'init' ) );
			add_action( 'plugins_loaded', array( self::$instance, 'load_textdomain' ) );

		}

		return self::$instance;

	}

	public function init(){

		// Make sure Cooked is active
		$cooked_pro_plugin = plugin_basename( __FILE__ );
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

		if ( !class_exists( 'Cooked_Plugin' ) ):
			add_action( 'admin_notices', function(){
				$class = 'notice notice-error';
				$message = sprintf( __( 'The %s core plugin needs to be installed and activated before %s can be enabled. %s', 'cooked-pro' ), '<strong><a href="https://wordpress.org/plugins/cooked/" target="_blank">Cooked</a></strong>', '<strong>Cooked Pro</strong>', '<strong><a href="' . wp_nonce_url( add_query_arg( array( 'action' => 'install-plugin', 'plugin' => 'cooked' ), admin_url( 'update.php' ) ), 'install-plugin_cooked' ) . '">' . __( 'Install it now', 'cooked-pro' ) . '</a></strong>' );
				printf( '<div class="%1$s" style="padding:10px 20px"><p style="font-size:1.2em">%2$s</p></div>', esc_attr( $class ), $message );
			});
			unset($_GET['activate']);
			deactivate_plugins( $cooked_pro_plugin );
			return;
		endif;

		self::$instance->module_setup();
		if ( !is_admin() ):
			self::$instance->enqueues = new Cooked_Pro_Enqueues();
		else:
			self::$instance->enqueues = new Cooked_Pro_Admin_Enqueues();
		endif;

		self::$instance->session = new Cooked_Pro_Session();

	}

	private function module_setup(){

		// Look for Cooked Modules
		$modules = file_exists( COOKED_PRO_DIR . 'modules' ) ? scandir( COOKED_PRO_DIR . 'modules' ) : false;
		if ($modules):
			foreach($modules as $module_name):
				if ($module_name !== '.' && $module_name !== '..'):
					if ( file_exists( COOKED_PRO_DIR . 'modules/'.$module_name.'/'.$module_name.'.php' ) ):
						require_once COOKED_PRO_DIR . 'modules/'.$module_name.'/'.$module_name.'.php';
					endif;
				endif;
			endforeach;
		endif;

	}

	/**
	 * Throw error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @since 1.0.0
	 * @access protected
	 * @return void
	 */
	public function __clone() {
		// Nope, can't do that.
		return false;
	}

	/**
	 * Disable unserializing of the class.
	 *
	 * @since 1.0.0
	 * @access protected
	 * @return void
	 */
	public function __wakeup() {
		// Nope, can't do that either.
		return false;
	}

	/**
	 * Setup plugin constants.
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function setup_constants() {

		// PHP Sessions.
		if ( ! defined( 'COOKED_USE_PHP_SESSIONS' ) ) {
			define( 'COOKED_USE_PHP_SESSIONS', false );
		}

		// Plugin Folder Path.
		if ( ! defined( 'COOKED_PRO_DIR' ) ) {
			define( 'COOKED_PRO_DIR', plugin_dir_path( __FILE__ ) );
		}

		// Plugin Folder Name.
		if ( ! defined( 'COOKED_PRO_FOLDER' ) ) {
			$foldername = untrailingslashit( str_replace( 'cooked-pro.php', '', plugin_basename( __FILE__ ) ) );
			define( 'COOKED_PRO_FOLDER', $foldername );
		}

		// Plugin Folder URL.
		if ( ! defined( 'COOKED_PRO_URL' ) ) {
			define( 'COOKED_PRO_URL', plugin_dir_url( __FILE__ ) );
		}

	}

	public function activation(){
		flush_rewrite_rules();
	}

	public function deactivation(){
		flush_rewrite_rules();
	}

	/**
	 * Include required files.
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function includes() {

		require_once COOKED_PRO_DIR . 'includes/class.cooked-enqueues.php';
		require_once COOKED_PRO_DIR . 'includes/session-functions.php';
		require_once COOKED_PRO_DIR . 'includes/core-tweaks.php';

	}

	/**
	 * Loads the plugin language files.
	 *
	 * @access public
	 * @since 1.0.0
	 * @return void
	 */
	public function load_textdomain() {

		/*
		 * When translating Cooked, be sure to move your language file into the proper location:
		 *
		 * - wp-content/languages/plugins/cooked
		 *
		 * If you do not move custom language files here, they will be lost when updating Cooked. Boxy Studio
		 * recommends Loco Translate for easy translations: https://boxystudio.ticksy.com/article/3235/
		 */

		// Set filter for plugin's languages directory.
		$cooked_lang_dir = COOKED_PRO_DIR . 'languages/';
		$cooked_lang_dir = apply_filters( 'cooked_pro_languages_directory', $cooked_lang_dir );

		// Traditional WordPress plugin locale filter.
		$locale = apply_filters( 'plugin_locale',  get_locale(), 'cooked-pro' );
		$mofile = sprintf( '%1$s-%2$s.mo', 'cooked-pro', $locale );

		// Look in wp-content/languages/plugins/cooked
		$lang_file_ext = WP_LANG_DIR . '/plugins/cooked-pro/' . $mofile;

		if ( file_exists( $lang_file_ext ) ) {

			// Load the externally located language files.
			load_textdomain( 'cooked-pro', $lang_file_ext );

		} else {

			// Load the default language files.
			load_plugin_textdomain( 'cooked-pro', false, COOKED_PRO_FOLDER . '/languages' );

		}

	}

}

endif; // End if class_exists check.

/**
 * The main function for that returns Cooked_Plugin
 *
 * The main function responsible for returning the Cooked_Plugin
 * Instance to functions everywhere.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $_Cooked_Pro = Cooked_Pro(); ?>
 *
 * @since 1.0.0
 * @return object|Cooked_Plugin
 */
function Cooked_Pro() {
	return Cooked_Pro_Plugin::instance();
}

// Let's Get Cooking!
global $_Cooked_Pro;
$_Cooked_Pro = Cooked_Pro();
