<?php

/**
 * Cooked Pro License Management / Updater
 *
 * @package     Cooked
 * @subpackage  User Interactions
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

define( 'COOKED_PRO_STORE_URL', 'https://cooked.pro' );
define( 'COOKED_PRO_ITEM_NAME', 'Cooked Pro' );
define( 'COOKED_PRO_PLUGIN_LICENSE_PAGE', 'cookedpro-license' );

if( !class_exists( 'EDD_SL_Plugin_Updater' ) ):
	require_once 'class.updater.php';
endif;

class Cooked_License {

	function __construct(){

		// Filters
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'license_settings'), 99, 3 );

		// Actions
		add_action( 'admin_init', array(&$this, 'cooked_pro_plugin_updater'), 0 );
		add_action( 'admin_init', array(&$this, 'cooked_pro_activate_license') );
		add_action( 'admin_init', array(&$this, 'cooked_pro_deactivate_license') );

	}

	public static function cooked_pro_plugin_updater() {

		global $_cooked_settings;
		$license_key = ( isset($_cooked_settings['cooked_pro_license_key']) && $_cooked_settings['cooked_pro_license_key'] ? trim( $_cooked_settings['cooked_pro_license_key'] ) : false );

		if ( $license_key ):
			$cooked_pro_updater = new EDD_SL_Plugin_Updater( COOKED_PRO_STORE_URL, COOKED_PRO_FILE, array(
					'version' 	=> COOKED_PRO_VERSION,
					'license' 	=> $license_key,
					'item_name' => COOKED_PRO_ITEM_NAME,
					'author' 	=> 'Boxy Studio',
					'beta'		=> false
				)
			);
		else:
			add_action( 'admin_notices', function(){
				$class = 'notice notice-error';
				$message = sprintf( '<strong>' . __( 'Welcome to %s!', 'cooked-pro' ) . '</strong>', 'Cooked Pro' ) . ' ' . sprintf( esc_html__( 'Please %s to enable automatic updates.', 'cooked-pro' ), '<a href="' . trailingslashit( admin_url() ) . 'admin.php?page=cooked_settings&' . rand(100000,999999) . '#license">' . esc_html__( 'activate your license key', 'cooked-pro' ) . '</a>' ) . ' <a href="https://cooked.pro/get-cooked/" target="_blank">' . esc_html__( 'Need to purchase a license?', 'cooked-pro' ) . '</a>';
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message );
			});
		endif;

	}

	public static function license_settings( $settings, $pages, $categories ){

		global $_cooked_settings;
		$license_key = ( isset( $_cooked_settings['cooked_pro_license_key'] ) && $_cooked_settings['cooked_pro_license_key'] ? $_cooked_settings['cooked_pro_license_key'] : false );
		$license_status = get_option( 'cooked_pro_license_status', false );
		$license_data = get_option( 'cooked_pro_license_data', false );

		$settings['license'] = array(
			'name' => esc_html__('License','cooked-pro'),
			'icon' => 'private',
			'fields' => array(
				'cooked_pro_license_key' => array(
					'title' => esc_html__('License Key', 'cooked-pro'),
					'desc' => ( $license_key ? '' : sprintf( esc_html__('Enter your %s license key to activate updates &amp; support.','cooked-pro'), 'Cooked Pro' ) ),
					'type' => 'text',
					'default' => ''
				)
			)
		);

		$settings['license']['fields']['cooked_pro_nonce'] = array(
			'type' => 'nonce'
		);

		if( current_user_can('edit_cooked_settings') && $license_key ):

			if ( isset( $license_data->expires ) && $license_data->expires && $license_data->expires != 'lifetime' && $license_data->expires != 'unlimited' ):
				$expires = ( isset( $license_data->error ) && $license_data->error && $license_data->error == 'expired' ? '<p class="cooked-license-expired">' . sprintf( esc_html__( 'Expired on %s', 'cooked-pro' ), date_i18n( COOKED_DATE_FORMAT, strtotime( $license_data->expires ) ) ) : '<p>' . sprintf( esc_html__( 'Expires: %s', 'cooked-pro' ), date_i18n( COOKED_DATE_FORMAT, strtotime( $license_data->expires ) ) ) );
				$expires .= '<br><a href="https://cooked.pro/account/">' . esc_html__('Renew License', 'cooked-pro') . '</a>';
				$expires .= '</p>';
			elseif ( isset( $license_data->expires ) && $license_data->expires == 'lifetime' || isset( $license_data->expires ) && $license_data->expires == 'unlimited' ):
				$expires = '<p>' . esc_html__( 'Lifetime License', 'cooked-pro' ) . '</p>';
			else:
				$expires = false;
			endif;

			if ( !$expires ):

				delete_option( 'cooked_pro_license_status' );
				delete_option( 'cooked_pro_license_data' );
				$_cooked_settings['cooked_pro_license_key'] = '';
				update_option( 'cooked_settings', $_cooked_settings );
				wp_redirect( admin_url( 'admin.php?page=cooked_settings#license' ) );
				exit();

			endif;

			$license_class = ( isset( $license_data->error ) && $license_data->error && $license_data->error == 'expired' ? ' expired' : ' valid' );
			$license_html = ( $expires ? $expires : '' );

			if ( $license_html ):

				$settings['license']['fields']['cooked_pro_license_info'] = array(
					'title' => '',
					'placeholder' => '<div class="cooked-html-block' . $license_class . '"><h3 class="cooked-settings-title">' . esc_html__('Your License', 'cooked-pro') . '</h3>' . $license_html . '</div>',
					'type' => 'html',
					'default' => ''
				);

			endif;

		endif;

		return $settings;

	}

	public static function cooked_pro_activate_license() {

		global $_cooked_settings;

		// listen for the license field to be entered.
		if( isset( $_POST['cooked_settings']['cooked_pro_license_key'] ) && $_POST['cooked_settings']['cooked_pro_license_key'] ) {

			$license_status = get_option( 'cooked_pro_license_status', false );
			$license_data = get_option( 'cooked_pro_license_data', false );

			if ( !$license_status || !$license_data ){

				// run a quick security check
			 	if( ! check_admin_referer( 'cooked_pro_nonce', 'cooked_pro_nonce' ) )
					return; // get out if we didn't click the Activate button

				// retrieve the license from the database
				$license = trim( esc_html( $_POST['cooked_settings']['cooked_pro_license_key'] ) );

				// data to send in our API request
				$api_params = array(
					'edd_action' => 'activate_license',
					'license'    => $license,
					'item_name'  => urlencode( COOKED_PRO_ITEM_NAME ), // the name of our product in EDD
					'url'        => home_url()
				);

				// Call the custom API.
				$response = wp_remote_post( COOKED_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

				// make sure the response came back okay
				if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

					if ( is_wp_error( $response ) ) {
						$message = $response->get_error_message();
					} else {
						$message = __( 'An error occurred, please try again.' );
					}

				} else {

					$license_data = json_decode( wp_remote_retrieve_body( $response ) );

					if ( false === $license_data->success ) {

						switch( $license_data->error ) {

							case 'expired' :

								$message = sprintf(
									__( 'Your license key expired on %s.' ),
									date_i18n( COOKED_DATE_FORMAT, strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
								);
								break;

							case 'revoked' :

								$message = __( 'Your license key has been disabled.' );
								break;

							case 'missing' :

								$message = __( 'Invalid license.' );
								break;

							case 'invalid' :
							case 'site_inactive' :

								$message = __( 'Your license is not active for this URL.' );
								break;

							case 'item_name_mismatch' :

								$message = sprintf( __( 'This appears to be an invalid license key for %s.' ), COOKED_PRO_ITEM_NAME );
								break;

							case 'no_activations_left':

								$message = __( 'Your license key has reached its activation limit.' );
								break;

							default :

								$message = __( 'An error occurred, please try again.' );
								break;
						}

					}

				}

				// Check if anything passed on a message constituting a failure
				if ( ! empty( $message ) ) {
					$base_url = admin_url( 'admin.php?page=cooked_settings#license' );
					$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );
				} else {
					$redirect = admin_url( 'admin.php?page=cooked_settings#license' );
				}

				$_cooked_settings['cooked_pro_license_key'] = $license;
				update_option( 'cooked_settings', $_cooked_settings );
				update_option( 'cooked_pro_license_data', $license_data );
				update_option( 'cooked_pro_license_status', $license_data->license );

				// $license_data->license will be either "valid" or "invalid"
				wp_redirect( $redirect );
				exit();
			}
		}
	}

	public static function cooked_pro_deactivate_license() {

		global $_cooked_settings;
		$license = trim( $_cooked_settings['cooked_pro_license_key'] );

		// listen for our activate button to be clicked
		if( $license && isset( $_POST['cooked_settings']['cooked_pro_license_key'] ) && !$_POST['cooked_settings']['cooked_pro_license_key'] ) {

			// run a quick security check
		 	if( ! check_admin_referer( 'cooked_pro_nonce', 'cooked_pro_nonce' ) )
				return; // get out if we didn't click the Activate button

			// data to send in our API request
			$api_params = array(
				'edd_action' => 'deactivate_license',
				'license'    => $license,
				'item_name'  => urlencode( COOKED_PRO_ITEM_NAME ),
				'url'        => home_url()
			);

			// Call the custom API.
			$response = wp_remote_post( COOKED_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

			// make sure the response came back okay
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

				if ( is_wp_error( $response ) ) {
					$message = $response->get_error_message();
				} else {
					$message = __( 'An error occurred, please try again.' );
				}

				$base_url = admin_url( 'admin.php?page=cooked_settings' );
				$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

				wp_redirect( $redirect );
				exit();

			}

			// decode the license data
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			// $license_data->license will be either "deactivated" or "failed"
			if( $license_data->license == 'deactivated' ) {
				delete_option( 'cooked_pro_license_status' );
				delete_option( 'cooked_pro_license_data' );
				$_cooked_settings['cooked_pro_license_key'] = '';
				update_option( 'cooked_settings', $_cooked_settings );
			}

			wp_redirect( admin_url( 'admin.php?page=cooked_settings#license' ) );
			exit();

		}
	}

}

new Cooked_License();
