<?php

/**
 * Cooked Add Recipe
 *
 * @package     Cooked
 * @subpackage  Add Recipe
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

require_once 'fields.php';
require_once 'pending/pending.php';

class Cooked_Add_Recipe {

	function __construct(){

		global $table_prefix;

		// Actions
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );
		add_action( 'admin_enqueue_scripts', array(&$this, 'admin_enqueues'), 10, 1 );
		add_action( 'init', array(&$this, 'init'), 10 );
		add_action( 'add_meta_boxes', array(&$this, 'recipe_status_box') );
		add_action( 'save_post', array( &$this, 'save_recipe_meta_box' ) );
		add_action( 'manage_cp_recipe_posts_custom_column' , array(&$this, 'custom_cp_recipe_column'), 10, 2 );
		add_action( 'cooked_recipe_manage_before_name', array( &$this, 'add_status_tags' ), 10, 1 );
		add_action( 'template_redirect', array( &$this, 'redirect_pending_draft_recipes' ), 10 );
		add_action( 'cooked_recipe_meta_before_fields', array( &$this, 'add_admin_meta_guest_fields' ), 10, 1 );

		// Pending Recipes
		add_action( 'admin_menu', array( &$this, 'cooked_admin_menu_pending_bubble' ), 10 );
		add_action( 'admin_notices', array( &$this, 'cooked_pending_notice' ) );

		// Filters
		add_filter( 'cooked_single_recipe_settings', array(&$this,'single_recipe_settings'), 10, 2 );
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'engagement_settings'), 11, 3 );
		add_filter( 'cooked_profile_nav', array(&$this, 'profile_nav_filter'), 10, 1 );
		add_filter( 'cooked_recipe_content', array(&$this, 'pending_draft_content_filters'), 10, 2 );
		add_filter( 'cooked_recipe_content', array(&$this, 'show_edit_recipe' ), 11, 2 );
		add_filter( 'cooked_recipe_list_title', array(&$this, 'pending_draft_title_filters'), 10, 2 );
		add_filter( 'manage_cp_recipe_posts_columns', array(&$this, 'set_custom_edit_cp_recipe_columns') );
		add_filter( 'cooked_recipe_public_query_filters', array(&$this,'hide_drafts_pending'), 10, 1 );
		add_filter( 'cooked_query_where_filter', array(&$this,'hide_drafts_pending_query_filter'), 10, 1 );
		add_filter( 'cooked_single_recipe_classes', array(&$this,'single_recipe_classes'), 10, 2 );
		add_filter( 'cooked_can_show_recipe', array(&$this,'can_show_recipe'), 10, 2 );

		// Shortcodes
		add_shortcode('cooked-submit', array(&$this, 'submit_shortcode') );
		add_shortcode('cooked-edit-recipe', array(&$this, 'edit_recipe_shortcode') );

		// Delete a Recipe
		add_action( 'wp_ajax_cooked_delete_recipe', array(&$this,'ajax_delete_recipe') );

		// Recipe Status Changes
		add_action( 'transition_post_status', array( &$this, 'on_status_change' ), 10, 3 );


	}

	public static function init(){
		add_action( 'cooked_user_add_recipe', array('Cooked_Add_Recipe', 'process_add_recipe_form') );
		add_action( 'cooked_user_edit_recipe', array('Cooked_Add_Recipe', 'process_edit_recipe_form') );
	}

	public static function on_status_change( $new_status, $old_status, $post ){
		if ( $post->post_type == 'cp_recipe' && $old_status == 'publish' && $new_status != 'publish' ) {
			$_cooked_settings = Cooked_Settings::get();
			if ( $new_status == 'pending' && isset($_cooked_settings['recipe_submission_pending_default']) && in_array( 'active', $_cooked_settings['recipe_submission_pending_default'] ) ):
				$post->post_status = 'publish';
				wp_update_post( $post );
				update_post_meta( $post->ID, '_recipe_pending', true );
			elseif( $new_status == 'draft' && isset($_cooked_settings['recipe_submission_draft_mode']) && in_array( 'active', $_cooked_settings['recipe_submission_draft_mode'] ) ):
				$post->post_status = 'publish';
				wp_update_post( 'publish', 'draft', $post );
				update_post_meta( $post->ID, '_recipe_draft', true );
			endif;
	    }
	}

	public static function pending_recipes_count(){
		$args = array(
            'post_type' => 'cp_recipe',
            'meta_query' => array(
	            array(
		            'pending_zero' => array(
		                'key' => '_recipe_pending',
		                'compare' => '==',
		                'value' => 1
		        	)
		        )
	        )
        );
        $pending_recipes = new WP_Query( $args );
		return $pending_recipes->found_posts;
	}

	public function cooked_admin_menu_pending_bubble() {

		global $menu, $submenu, $_cooked_settings;

		if ( isset( $_cooked_settings['recipe_submission_pending_default'] ) && $_cooked_settings['recipe_submission_pending_default'] && current_user_can('edit_cooked_recipes') ):

			$pending = self::pending_recipes_count();
			add_submenu_page('cooked_recipes_menu', esc_html__('Pending','cooked-pro'), esc_html__('Pending','cooked-pro'), 'edit_cooked_recipes', 'cooked_pending', array(&$this, 'cooked_pending_page') );

			// Get and remove the Pending menu item
			foreach( $submenu as $key => $value ):
				if ( $key == 'cooked_recipes_menu' ):
					foreach ( $value as $skey => $svalue ):
						if ( $svalue[2] == 'cooked_pending' ):
							$submenu[$key][$skey][0] .= ( $pending ? " <span style='position:relative; top:1px; margin:0 0 0 2px' class='update-plugins count-$pending' title='$pending'><span style='padding:0; display:inline-block; min-width:4px; text-align:center;' class='update-count'>" . $pending . "</span></span>" : "" );
							$pending_menu_item = $submenu[$key][$skey];
							unset($submenu[$key][$skey]);
						endif;
					endforeach;
				endif;
			endforeach;

			// Loop through and add the Pending menu item where it belongs
			foreach( $submenu as $key => $value ):
				if ( $key == 'cooked_recipes_menu' ):
					$temp_counter = 0;
					foreach ( $value as $skey => $svalue ):
						$submenu[$key][$temp_counter] = $svalue;
						if ( $temp_counter == 0 ):
							$temp_counter++;
							$submenu[$key][$temp_counter] = $pending_menu_item;
						endif;
						$temp_counter++;
					endforeach;
				endif;
			endforeach;

			if ( $pending ):
				foreach ( $menu as $key => $value ) :
					if ( $value[2] == 'cooked_recipes_menu' ) :
						if ( $pending ) { $menu[$key][0] .= " <span style='position:relative; top:1px; margin:0 0 0 2px' class='update-plugins count-$pending' title='$pending'><span style='padding:0; display:inline-block; min-width:4px; text-align:center;' class='update-count'>" . $pending . "</span></span>"; }
					endif;
				endforeach;
			endif;

		endif;

	}

	public function cooked_pending_notice() {

		global $menu, $submenu, $_cooked_settings;

		if ( isset( $_cooked_settings['recipe_submission_pending_default'] ) && $_cooked_settings['recipe_submission_pending_default'] && current_user_can('edit_cooked_recipes') ):

			$pending = self::pending_recipes_count();
			$page = (isset($_GET['page']) ? $page = esc_html( $_GET['page'] ) : $page = false);
			if ($pending && $page != 'cooked_pending' && $page != 'cooked_welcome'):

				echo '<div class="update-nag" style="line-height:28px">';
					echo sprintf( _n( 'There is %s pending recipe.', 'There are %s pending recipes.', $pending, 'cooked-pro' ), $pending ).'&nbsp;&nbsp;&nbsp;<a href="' . get_admin_url( null, 'admin.php?page=cooked_pending' ) . '" class="button">'._n('View Pending Recipe','View Pending Recipes',$pending,'cooked-pro').'</a>';
				echo '</div>';

			endif;

		endif;

	}

	public function cooked_pending_page(){
		if(!current_user_can('edit_cooked_recipes')) {
			wp_die(__('You do not have sufficient permissions to access this page.', 'cooked'));
		}
		include( COOKED_PRO_DIR . 'templates/admin/pending.php' );
	}

	public static function add_admin_meta_guest_fields( $recipe_settings ){
		if ( isset($recipe_settings['guest_name']) || isset($recipe_settings['guest_email']) ):
			?><div class="recipe-setting-block"><div class="cooked-clearfix"><?php

				if ( isset( $recipe_settings['guest_name'] ) ):
					?><div class="cooked-setting-column-12">
						<div class="recipe-setting-block">
							<h3 class="cooked-settings-title"><?php esc_html_e( 'Guest Submission - Author Name', 'cooked' ); ?><span class="cooked-tooltip cooked-tooltip-icon" title="<?php echo esc_attr( esc_html__( 'The recipe was submitted by a guest. You can update their name here if needed.','cooked-pro') ); ?>"><i class="cooked-icon cooked-icon-question"></i></span></h3>
							<p><?php echo '<input type="text" name="_recipe_settings[guest_name]" value="' . esc_attr( $recipe_settings['guest_name'] ) . '">'; ?></p>
						</div>
					</div><?php
				endif;
				if ( isset( $recipe_settings['guest_email'] ) ):
					?><div class="cooked-setting-column-12">
						<div class="recipe-setting-block">
							<h3 class="cooked-settings-title"><?php esc_html_e( 'Guest Submission - Author Email', 'cooked' ); ?><span class="cooked-tooltip cooked-tooltip-icon" title="<?php echo esc_attr( esc_html__( 'The recipe was submitted by a guest. You can update their email here if needed.','cooked-pro') ); ?>"><i class="cooked-icon cooked-icon-question"></i></span></h3>
							<p><?php echo '<input type="text" name="_recipe_settings[guest_email]" value="' . esc_attr( $recipe_settings['guest_email'] ) . '">'; ?></p>
						</div>
					</div><?php
				endif;

			?></div></div><?php
		endif;
	}

	public static function is_draft( $recipe_id ){
		$is_draft = get_post_meta( $recipe_id, '_recipe_draft', true );
		return $is_draft;
	}

	public static function is_pending( $recipe_id ){
		$is_pending = get_post_meta( $recipe_id, '_recipe_pending', true );
		return $is_pending;
	}

	public static function can_show_recipe( $default = true, $recipe_id ){
		if ( self::is_draft($recipe_id) || self::is_pending($recipe_id) ):
			return false;
		endif;
		return $default;
	}

	public static function disable_comments_status(){
		return false;
	}

	public static function hide_existing_comments(){
		return array();
	}

	public static function remove_comments_are_closed( $translated_text, $text, $domain ) {
		if ( is_singular() ) {
	        switch ( $translated_text ) {
	            case 'Comments are closed.' :
	                $translated_text = '&nbsp;';
	                break;
	        }
	    }
	    return $translated_text;
	}

	public static function show_edit_recipe( $content, $recipe_id ){
		if ( isset( $_GET['edit_recipe'] ) && Cooked_Profiles::is_my_recipe( $recipe_id ) || isset( $_GET['edit_recipe'] ) && current_user_can( 'edit_cooked_recipes' ) ):

			// Show the edit form.
			$content = '[cooked-edit-recipe]';

		endif;
		return $content;
	}

	public static function redirect_pending_draft_recipes(){

		global $wp_query,$post;

		if ( 'cp_recipe' === get_post_type($post) ):
			if ( Cooked_Profiles::is_my_recipe( $post->ID ) || current_user_can('edit_cooked_recipes') ):
				return;
			else:
				$recipe = Cooked_Recipes::get( $post->ID, true );
				if ( isset($recipe['_recipe_pending']) && $recipe['_recipe_pending'] || isset($recipe['_recipe_draft']) && $recipe['_recipe_draft'] ):
					$wp_query->set_404();
					return;
				endif;
			endif;
		endif;

		return;

	}

	public function hide_drafts_pending_query_filter( $string ){
		return $string . " AND ( mt1.post_id IS NULL OR ( mt2.meta_key = '_recipe_draft' AND mt2.meta_value != '1' ) ) AND ( mt3.post_id IS NULL OR ( mt4.meta_key = '_recipe_pending' AND mt4.meta_value != '1' ) )";
	}

	public function edit_recipe_shortcode( $atts, $content = null ){

		if ( !is_user_logged_in() )
			return;

		ob_start();
		load_template( COOKED_PRO_DIR . 'templates/front/form-edit-recipe.php',false);
		$form_template = ob_get_clean();
		return $form_template;

	}

	public function hide_drafts_pending( $args ) {

		$args['meta_query']['relation'] = 'AND';
		$args['meta_query'][] = array(
			'relation' => 'OR',
			'draft_null' => array(
				'key' => '_recipe_draft',
        		'compare' => 'NOT EXISTS'
			),
			'draft_zero' => array(
				'key' => '_recipe_draft',
				'compare' => '!=',
				'value' => 1
			)
		);
		$args['meta_query'][] = array(
			'relation' => 'OR',
			'pending_null' => array(
				'key' => '_recipe_pending',
        		'compare' => 'NOT EXISTS'
			),
			'pending_zero' => array(
				'key' => '_recipe_pending',
				'compare' => '!=',
				'value' => 1
			)
		);

		return $args;

	}

	public function single_recipe_settings( $recipe_settings, $post_id ){

		$guest_name = ( isset( $recipe_settings['guest_name'] ) ? $recipe_settings['guest_name'] : false );
		$guest_email = ( isset( $recipe_settings['guest_email'] ) ? $recipe_settings['guest_email'] : false );

		if ( $guest_name || $guest_email ):
			$recipe_settings['author'] = array();
			$recipe_settings['author']['id'] = false;
			if ( $guest_name ):
				$recipe_settings['author']['name'] = Cooked_Users::format_author_name( $guest_name );
			endif;
			if ( $guest_email ):
				$recipe_settings['author']['profile_photo'] = get_avatar( $guest_email, 'cooked-square' );
				$recipe_settings['author']['profile_photo_src'] = get_avatar_url( $guest_email, 'cooked-square' );
			endif;
		endif;

		global $_cooked_settings,$recipe_terms_list;

		$taxonomies = array( 'category', 'cuisine', 'cooking_method', 'diet' );
		foreach ( $taxonomies as $tax_name ):

			if (in_array('cp_recipe_' . $tax_name,$_cooked_settings['recipe_taxonomies'])):
				$_recipe_terms_array = array();
				$recipe_terms_array = wp_get_object_terms( $post_id, 'cp_recipe_' . $tax_name );
				if ( !empty($recipe_terms_array) ):
					if ( ! is_wp_error( $recipe_terms_array ) ):
						foreach( $recipe_terms_array as $recipe_term ):
							$recipe_settings[$tax_name] = $recipe_term->slug;
							break;
						endforeach;
					endif;
				endif;
			endif;

		endforeach;

		$recipe_settings['_recipe_pending'] = get_post_meta( $post_id, '_recipe_pending', true );
		$recipe_settings['_recipe_draft'] = get_post_meta( $post_id, '_recipe_draft', true );
		return $recipe_settings;

	}

	public function save_recipe_meta_box( $post_id ) {

        if ( !isset( $_POST['cooked_recipe_custom_box_nonce'] ) )
            return $post_id;

        $nonce = $_POST['cooked_recipe_custom_box_nonce'];

        // Verify that the nonce is valid.
        if ( ! wp_verify_nonce( $nonce, 'cooked_recipe_custom_box' ) )
            return $post_id;

        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
            return $post_id;

        // Check the user's permissions.
        if ( ! current_user_can( 'edit_cooked_recipes', $post_id ) )
            return $post_id;

        /* OK, it's safe for us to save the data now. */
        $recipe_draft = isset($_POST['_recipe_draft']) ? esc_html( $_POST['_recipe_draft'] ) : false;
        $recipe_pending = isset($_POST['_recipe_pending']) ? esc_html( $_POST['_recipe_pending'] ) : false;

        if ( $recipe_draft ):
        	do_action( 'cooked_recipe_status_update_draft', $post_id );
        	update_post_meta( $post_id, '_recipe_draft', $recipe_draft );
        else:
        	do_action( 'cooked_recipe_status_update_public', $post_id );
        	delete_post_meta( $post_id, '_recipe_draft' );
        endif;

        if ( $recipe_pending ):
        	do_action( 'cooked_recipe_status_update_pending', $post_id );
        	update_post_meta( $post_id, '_recipe_pending', $recipe_pending );
        else:
        	do_action( 'cooked_recipe_status_update_approved', $post_id );
        	delete_post_meta( $post_id, '_recipe_pending' );
        endif;

    }

	public function set_custom_edit_cp_recipe_columns($columns) {

		$columns['recipe_status'] = __( 'Status', 'cooked-pro' );
		return $columns;

	}

	public function custom_cp_recipe_column( $column, $post_id ) {
	    switch ( $column ) {

	        case 'recipe_status' :
	        	$_recipe = Cooked_Recipes::get( $post_id, true );
		    	$draft = ( isset( $_recipe['_recipe_draft'] ) && $_recipe['_recipe_draft'] ? '<span class="cooked-tooltip cooked-tooltip-icon" title="' . esc_html__( 'Draft', 'cooked-pro' ) . '"><span class="cooked-status-bubble cooked-status-draft"></span></span>' : false );
		    	$pending = ( isset( $_recipe['_recipe_pending'] ) && $_recipe['_recipe_pending'] ? '<span class="cooked-tooltip cooked-tooltip-icon" title="' . esc_html__( 'Pending', 'cooked-pro' ) . '"><span class="cooked-status-bubble cooked-status-pending"></span></span>' : false );
		    	$published = ( !$draft && !$pending ? '<span class="cooked-tooltip cooked-tooltip-icon" title="' . esc_html__( 'Public', 'cooked-pro' ) . '"><span class="cooked-status-bubble cooked-status-published"></span></span>' : false );
	            echo ( $published ? $published : $pending . $draft );
	        break;

	    }
	}

	public function recipe_status_box(){
	    add_meta_box( 'cooked-status-meta-box', '<i class="cooked-icon cooked-icon-cooked-icon"></i>&nbsp;&nbsp;' . esc_html__( 'Status','cooked-pro' ), array(&$this, 'recipe_status_content'), 'cp_recipe', 'side', 'high' );
	}

	public function recipe_status_content( $post ){

		$recipe_pending = get_post_meta( $post->ID, '_recipe_pending', true );
		$recipe_draft = get_post_meta( $post->ID, '_recipe_draft', true );

		?><p>
        	<input type="checkbox" class="cooked-switch-yellow" id="recipe_pending" name="_recipe_pending" value="1" <?php checked( $recipe_pending, 1 ); ?> />
        	<label for="recipe_pending"><?php esc_html_e( 'Pending Review', 'cooked-pro' ); ?></label>
    	</p>
    	<p>
        	<input type="checkbox" class="cooked-switch-gray" id="recipe_draft" name="_recipe_draft" value="1" <?php checked( $recipe_draft, 1 ); ?> />
        	<label for="recipe_draft"><?php esc_html_e( 'Draft Recipe', 'cooked-pro' ); ?></label>
    	</p><?php
	}

	public function pending_draft_content_filters( $content, $recipe_id ){

		global $_cooked_settings;

		if ( !$recipe_id )
			return $content;

		$_user = wp_get_current_user();
		$recipe_author_id = get_post_field( 'post_author', $recipe_id );

		if ( $recipe_author_id == $_user->ID || current_user_can( 'edit_cooked_recipes' ) ):

			$_recipe = Cooked_Recipes::get( $recipe_id, true );
			$edit_link = ' <a href="' . add_query_arg( array('edit_recipe' => '' ) ) . '">' . esc_html__( 'Edit Recipe', 'cooked-pro' ) . '</a>';

			if ( isset($_recipe['_recipe_draft']) && $_recipe['_recipe_draft'] ):
				return '<div class="cooked-draft-banner"><i class="cooked-icon cooked-icon-pencil"></i>&nbsp;&nbsp;' . esc_html__( 'This recipe is in draft mode. No one else can see it at the moment.', 'cooked-pro' ) . $edit_link . '</div>' . $content;
			endif;

			if ( isset($_recipe['_recipe_pending']) && $_recipe['_recipe_pending'] ):
				return '<div class="cooked-pending-banner"><i class="cooked-icon cooked-icon-pencil"></i>&nbsp;&nbsp;' . esc_html__( 'This recipe is pending review. No one else can see it yet.', 'cooked-pro' ) . $edit_link . '</div>' . $content;
			endif;

			return '<div class="cooked-published-banner"><i class="cooked-icon cooked-icon-check"></i>&nbsp;&nbsp;' . esc_html__( 'This recipe has been published and is viewable by everyone.', 'cooked-pro' ) . $edit_link . '</div>' . $content;

		endif;

		return $content;

	}

	public static function send_email( $email = false ){

        if ( !$email )
            return false;

        add_filter( 'wp_mail_content_type', array( 'Cooked_Add_Recipe', 'set_html_content_type' ) );

        if ( file_exists( get_stylesheet_directory() . '/cooked/email-template.html' ) ):
            $template = file_get_contents( get_stylesheet_directory() . '/cooked/email-template.html', true );
        elseif ( file_exists( get_template_directory() . '/cooked/email-template.html' ) ):
            $template = file_get_contents( get_template_directory() . '/cooked/email-template.html', true );
        else:
            $template = file_get_contents( COOKED_PRO_DIR . '/templates/email/default.html', true );
        endif;

        $filter = '%content%';
        $replace = $email['content'];
        $headers[] = 'Content-Type: text/html; charset=UTF-8';
        $email_content = str_replace( $filter, $replace, $template );

        $email_response = wp_mail( $email['to'], $email['subject'], $email_content, $headers );

        remove_filter( 'wp_mail_content_type', array( 'Cooked_Add_Recipe', 'set_html_content_type' ) );

        return $email_response;

    }

    public static function set_html_content_type() {
        return 'text/html';
    }

	public function single_recipe_classes( $classes, $recipe ){

		if ( !is_array($recipe) || is_array($recipe) && !isset($recipe['id']) )
			return $classes;

		$_user = wp_get_current_user();
		$recipe_author_id = get_post_field( 'post_author', $recipe['id'] );

		if ( $recipe_author_id == $_user->ID ):

			$classes[] = 'cooked-my-recipe';
			$_recipe = Cooked_Recipes::get( $recipe['id'], true );

			if ( isset($_recipe['_recipe_pending']) && $_recipe['_recipe_pending'] ):
				$classes[] = 'cooked-pending';
			endif;
			if ( isset($_recipe['_recipe_draft']) && $_recipe['_recipe_draft'] ):
				$classes[] = 'cooked-draft';
			endif;

		endif;

		return $classes;

	}

	public function add_status_tags( $recipe ){

		global $is_own_profile;

		if ( !isset($recipe['id']) || !$is_own_profile )
			return;

		$_user = wp_get_current_user();
		$recipe_author_id = get_post_field( 'post_author', $recipe['id'] );

		if ( $recipe_author_id == $_user->ID ):

			$_recipe = Cooked_Recipes::get( $recipe['id'], true );
			$tagged = false;

			if ( isset($_recipe['_recipe_draft']) && $_recipe['_recipe_draft'] ):
				$tagged = true;
				echo '<span class="cooked-status-tag cooked-draft">' . esc_html__( 'Draft', 'cooked-pro' ) . '</span>';
			endif;

			if ( !$tagged && isset($_recipe['_recipe_pending']) && $_recipe['_recipe_pending'] ):
				$tagged = true;
				echo '<span class="cooked-status-tag cooked-pending">' . esc_html__( 'Pending', 'cooked-pro' ) . '</span>';
			endif;

			// if ( !$tagged ):
			// 	echo '<span class="cooked-status-tag cooked-published">' . esc_html__( 'Published', 'cooked-pro' ) . '</span>';
			// endif;

		endif;

	}

	public function pending_draft_title_filters( $title, $recipe_id ){

		if ( !$recipe_id )
			return $title;

		$_user = wp_get_current_user();
		$recipe_author_id = get_post_field( 'post_author', $recipe_id );

		if ( $recipe_author_id == $_user->ID ):

			$_recipe = Cooked_Recipes::get( $recipe_id, true );

			if ( isset($_recipe['_recipe_pending']) && $_recipe['_recipe_pending'] ):
				return $title . '<span class="cooked-pending-banner">' . esc_html__( 'Pending Review', 'cooked-pro' ) . '</span>';
			endif;
			if ( isset($_recipe['_recipe_draft']) && $_recipe['_recipe_draft'] ):
				return $title . '<span class="cooked-draft-banner">' . esc_html__( 'Draft Recipe', 'cooked-pro' ) . '</span>';
			endif;

		endif;

		return $title;

	}

	public function profile_nav_filter( $profile_nav ){
		global $atts,$_cooked_settings,$_cooked_user,$profile_user,$is_own_profile;
		$is_own_profile = ( $profile_user['id'] == $_cooked_user['id'] ? true : false );
		if ( isset( $_cooked_settings['recipe_submission'][0] ) && $is_own_profile ):
			$profile_nav['add'] = array(
				'name' => esc_html__('Add a Recipe','cooked-pro'),
				'default' => ( get_query_var('cooked_pn','recipes') == 'add' ? true : false ),
				'link_hook' => array( 'Cooked_Add_Recipe', 'add' )
			);
		endif;
		return $profile_nav;
	}

	public static function run( $tab_name ){

		switch( $tab_name ):
			case 'add':
				echo do_shortcode("[cooked-submit]");
			break;
		endswitch;

	}

	public static function guest_mode_active(){
		$_cooked_settings = Cooked_Settings::get();
		return ( isset($_cooked_settings['recipe_submission_guest_mode'][0]) && $_cooked_settings['recipe_submission_guest_mode'][0] == 'active' ? true : false );
	}

	public function submit_shortcode( $atts, $content = null ){

		global $submit_atts,$_cooked_settings;

		ob_start();

		$atts = shortcode_atts(
			array(
				'title' => true,
				'__forced_show' => false
			), $atts
		);

		$submit_atts = $atts;

		$guest_mode = self::guest_mode_active();

		if ( !is_user_logged_in() && !$guest_mode ):
			echo do_shortcode( '[cooked-login title=' . $submit_atts['title'] . ']' );
		else:
			load_template( COOKED_PRO_DIR . 'templates/front/form-add-recipe.php',false);
		endif;

		return ob_get_clean();

	}

	public function enqueues(){

		$min = ( COOKED_DEV ? '' : '.min' );

		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-draggable' );
		wp_enqueue_script( 'jquery-ui-resizable' );
		wp_enqueue_script( 'jquery-ui-sortable' );

		$max_upload_limit = ( wp_max_upload_size() > 5 ? 5 : wp_max_upload_size() );

		$cooked_pro_js_vars = array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'max_image_size' => $max_upload_limit,
			'i18n_image_too_large' => sprintf( esc_html__( 'This image exceeds the max upload limit of %s, please try a smaller file.','cooked-pro' ), $max_upload_limit . 'MB' ),
			'i18n_confirm_delete' => esc_html__( 'Are you sure you want to delete this recipe?','cooked-pro' ),
			'i18n_nutrition_toggle_enable' => esc_html__( 'Show Nutrition Facts','cooked-pro' ),
			'i18n_nutrition_toggle_disable' => esc_html__( 'Hide Nutrition Facts','cooked-pro' )
		);

		wp_enqueue_style( 'cooked-add-recipe-css', COOKED_PRO_URL . 'modules/add-recipe/add-recipe' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_register_script( 'cooked-add-recipe-js', COOKED_PRO_URL . 'modules/add-recipe/add-recipe' . $min . '.js', array('jquery-ui-sortable'), COOKED_PRO_VERSION, true );
		wp_localize_script( 'cooked-add-recipe-js', 'cooked_pro_js_vars', $cooked_pro_js_vars );

	}

	public function admin_enqueues(){

		$min = ( COOKED_DEV ? '' : '.min' );
		wp_enqueue_style( 'cooked-add-recipe-admin-css', COOKED_PRO_URL . 'modules/add-recipe/add-recipe-admin' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_enqueue_script( 'cooked-axios', COOKED_PRO_URL . 'modules/add-recipe/pending/axios'.$min.'.js', array('cooked-vue'), null, false );
	    wp_register_script( 'cooked-pending', COOKED_PRO_URL . 'modules/add-recipe/pending/pending-recipes.js', array('cooked-vue'), null, true );
	    wp_localize_script( 'cooked-pending', 'wpApiSettings', array(
		    'root' => esc_url_raw( rest_url() ),
		    'nonce' => wp_create_nonce( 'wp_rest' )
		) );

		wp_enqueue_script( 'cooked-pending' );

	}

	public static function engagement_settings( $settings, $pages, $categories ){

		$add_recipe_fields = Cooked_Add_Recipe_Fields::get();
		$add_recipe_fields_required = Cooked_Add_Recipe_Fields::get( true );
		$_nutrition_fields = Cooked_Measurements::nutrition_facts();
		foreach( $_nutrition_fields as $section => $fields ):
			foreach( $fields as $key => $field ):
				if ( isset( $field['subs'] ) ):
					$nutrition_fields[$key]['label'] = $field['name'];
					foreach( $field['subs'] as $sub_key => $sub_field ):
						$nutrition_fields[$sub_key]['label'] = $sub_field['name'];
					endforeach;
					unset( $nutrition_fields[$key]['subs'] );
				else:
					$nutrition_fields[$key]['label'] = $field['name'];
				endif;
			endforeach;
		endforeach;

		Cooked_Functions::array_splice_assoc( $settings['design']['fields'], 'main_color_hover', 0, array(
			'secondary_color' => array(
				'title' => esc_html__('Secondary Color', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the secondary buttons, etc.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#1491ff',
				'options' => '#1491ff'
			),
			'secondary_color_hover' => array(
				'title' => esc_html__('Secondary Color (on hover)', 'cooked-pro'),
				'desc' => esc_html__('Set the color when hovering for the secondary buttons, etc.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#117bd8',
				'options' => '#117bd8'
			)
		));

		$settings['recipe_submission'] = array(
			'name' => esc_html__('Recipe Submission','cooked-pro'),
			'icon' => 'recipe-icon',
			'fields' => array()
		);

		$settings['recipe_submission']['fields']['recipe_submission'] = array(
			'title' => esc_html__('Recipe Submission', 'cooked-pro'),
			'desc' => esc_html__('This will enable an "Add Recipe" tab within the user\'s profile that allows them to submit recipes.','cooked-pro') . '<br>' . sprintf( esc_html__('You can also use the %s shortcode anywhere to display the submission form.','cooked-pro'), '<code>[cooked-submit]</code>' ),
			'type' => 'checkboxes',
			'default' => array(),
			'options' => array(
				'enabled' => array(
					'conditional_value' => 'enable_recipe_submissions',
					'label' => esc_html__( 'Enable Recipe Submission', 'cooked-pro' )
				),
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_notification_enabled'] = array(
			'title' => esc_html__('Email Notification', 'cooked-pro'),
			'desc' => esc_html__('Send an email notification when a new recipe is submitted?','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => '',
			'options' => array(
				'active' => array(
					'conditional_value' => 'enable_recipe_notifications',
					'label' => esc_html__( 'Enable Notification', 'cooked-pro' )
				),
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_notification_email'] = array(
			'title' => esc_html__('Notification Email Address', 'cooked-pro'),
			'desc' => sprintf( esc_html__('To whom do you want the notification to be sent? %s','cooked-pro'), '<a href="https://www.wpbeginner.com/wp-tutorials/how-to-fix-wordpress-not-sending-email-issue/" target="_blank">' . esc_html__( 'Having email issues?', 'cooked-pro' ) . '</a>' ),
			'type' => 'text',
			'conditional_requirement' => array( 'enable_recipe_notifications', 'enable_recipe_submissions' ),
			'default' => get_option('admin_email')
		);

		$settings['recipe_submission']['fields']['recipe_submission_pending_default'] = array(
			'title' => esc_html__('Pending Mode', 'cooked-pro'),
			'desc' => sprintf( esc_html__( 'All new and user-edited submissions are hidden by default and will need to be reviewed by an Administrator or Editor from the %s screen.', 'cooked-pro' ), '<a href="' . esc_url( get_admin_url( null, 'admin.php?page=cooked_pending' ) ) . '">' . esc_html__( 'Pending', 'cooked-pro' ) . '</a>' ),
			'type' => 'checkboxes',
			'color' => 'yellow',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_pending_default', array( 'active' ) ),
			'options' => array(
				'active' => esc_html__( 'Enable Pending Mode', 'cooked-pro' )
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_draft_mode'] = array(
			'title' => esc_html__('Draft Mode', 'cooked-pro'),
			'desc' => esc_html__('Enabling this will allow your user to hide their own recipes (from the public) by setting the status to "Draft".','cooked-pro'),
			'type' => 'checkboxes',
			'color' => 'gray',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_draft_mode', array( 'active' ) ),
			'options' => array(
				'active' => esc_html__( 'Enable Draft Mode', 'cooked-pro' )
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_guest_mode'] = array(
			'title' => esc_html__('Guest Mode', 'cooked-pro'),
			'desc' => esc_html__('Enabling this will allow anyone who visits the site to submit a recipe, not just logged-in users.','cooked-pro') . '<br><strong>' . esc_html__( 'NOTE:', 'cooked-pro' ) . '</strong> ' . esc_html__('It is highly suggested that you use "Pending Mode" with this option.','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_draft_mode', array() ),
			'options' => array(
				'active' => array(
					'conditional_value' => 'enable_guest_mode',
					'label' => esc_html__( 'Enable Guest Mode', 'cooked-pro' )
				)
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_guest_name'] = array(
			'title' => esc_html__('Guest Name Field', 'cooked-pro'),
			'desc' => esc_html__('Applies to Guest submissions only.','cooked-pro'),
			'type' => 'radio',
			'conditional_requirement' => array( 'enable_recipe_submissions', 'enable_guest_mode' ),
			'default' => array( 'request' ),
			'options' => array(
				'disabled' => array(
					'label' => esc_html__( 'Disabled', 'cooked-pro' ),
				),
				'request' => array(
					'label' => esc_html__( 'Request a Name', 'cooked-pro' ),
				),
				'require' => array(
					'label' => esc_html__( 'Require a Name', 'cooked-pro' ),
				)
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_guest_email'] = array(
			'title' => esc_html__('Guest Email Field', 'cooked-pro'),
			'desc' => esc_html__('Applies to Guest submissions only.','cooked-pro'),
			'type' => 'radio',
			'conditional_requirement' => array( 'enable_recipe_submissions', 'enable_guest_mode' ),
			'default' => array( 'request' ),
			'options' => array(
				'disabled' => array(
					'label' => esc_html__( 'Disabled', 'cooked-pro' ),
				),
				'request' => array(
					'label' => esc_html__( 'Request an Email Address', 'cooked-pro' ),
				),
				'require' => array(
					'label' => esc_html__( 'Require an Email Address', 'cooked-pro' ),
				)
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_delete_allowed'] = array(
			'title' => esc_html__('User Recipe Deletion', 'cooked-pro'),
			'desc' => esc_html__( 'Enabling this will allow your users to delete their own recipes.', 'cooked-pro' ) . '<br><strong>' . esc_html__( 'NOTE:', 'cooked-pro' ) . '</strong> ' . esc_html__( 'User-deleted recipes are permanently deleted, not just moved to the trash.','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_delete_allowed', array( 'active' ) ),
			'options' => array(
				'active' => esc_html__( 'Allow User Recipe Deletion', 'cooked-pro' )
			)
		);

		$settings['recipe_submission']['fields']['recipe_submission_active_fields'] = array(
			'title' => esc_html__('Included Fields', 'cooked-pro'),
			'desc' => esc_html__('Choose which fields will be displayed on the "Add Recipe" form.','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_fields_active_defaults', array( 'post_title','recipe_image','excerpt','category','cuisine','cooking_method','diet','difficulty_level','prep_time','cook_time','ingredients','directions','nutrition' ) ),
			'options' => $add_recipe_fields
		);

		$settings['recipe_submission']['fields']['recipe_submission_active_nut_fields'] = array(
			'title' => esc_html__('Nutrition Fields', 'cooked-pro'),
			'desc' => esc_html__('Choose which nutrition fields will be displayed on the "Add Recipe" form.','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => array( 'enable_recipe_submissions', 'nutrition_fields_active' ),
			'default' => apply_filters( 'cooked_add_recipe_fields_active_nut_defaults', array( 'serving_size', 'servings', 'calories', 'fat', 'cholesterol', 'carbs', 'fiber' ) ),
			'options' => $nutrition_fields
		);

		$settings['recipe_submission']['fields']['recipe_submission_required_fields'] = array(
			'title' => esc_html__('Required Fields', 'cooked-pro'),
			'desc' => esc_html__('Choose which fields are required on the "Add Recipe" form.','cooked-pro'),
			'type' => 'checkboxes',
			'color' => 'red',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => apply_filters( 'cooked_add_recipe_fields_required_defaults', array( 'post_title','ingredients','directions' ) ),
			'options' => $add_recipe_fields_required
		);

		$settings['recipe_submission']['fields']['recipe_submission_options'] = array(
			'title' => esc_html__('Other Options', 'cooked-pro'),
			'desc' => esc_html__('Choose from the following options to include with the "Add Recipe" form.','cooked-pro'),
			'type' => 'checkboxes',
			'conditional_requirement' => 'enable_recipe_submissions',
			'default' => array(),
			'options' => array(
				'ingredient_links' => array(
					'label' => '<strong>' . esc_html__( 'Ingredient Links', 'cooked-pro' ) . '</strong> &mdash; ' . esc_html__( 'Users can include a URL to which the ingredient will be linked.', 'cooked-pro' ),
				),
				'ingredient_descriptions' => array(
					'label' => '<strong>' . esc_html__( 'Ingredient Descriptions', 'cooked-pro' ) . '</strong> &mdash; ' . esc_html__( 'Users can include additional text with each ingredient (optional).', 'cooked-pro' ),
				),
				'section_headings' => array(
					'label' => '<strong>' . esc_html__( 'Section Headings', 'cooked-pro' ) . '</strong> &mdash; ' . esc_html__( 'Users can add section headings to ingredients and directions.', 'cooked-pro' ),
				)
			)
		);

		return $settings;

	}

	public static function status_fields( $recipe ){

		$pending = ( isset($recipe['_recipe_pending']) && $recipe['_recipe_pending'] ? true : false );
		$draft = ( isset($recipe['_recipe_draft']) && $recipe['_recipe_draft'] ? true : false );

		$status_fields = apply_filters('cooked_edit_status_fields', array(
			'published' => esc_html__( 'Published', 'cooked-pro' ) . ( $pending ? ' ' . esc_html__( '(pending review)', 'cooked-pro' ) : '' ),
			'draft' => esc_html__( 'Draft (hidden)', 'cooked-pro' ),
		));

		echo '<div class="cooked-field">';
			echo '<div class="cooked-label-side">';
				echo '<label>' . esc_html( 'Status', 'cooked-pro' ) . '</label>';
			echo '</div>';
			echo '<div class="cooked-input-side">';
				foreach( $status_fields as $id => $label ):
					$field_html[] = '<input' . ( $draft && $id == 'draft' || $id == 'published' && !$draft ? ' checked' : '' ) . ' id="recipe_status_' . $id . '" class="cooked-radio" type="radio" name="recipe_status[]" value="' . $id . '">&nbsp;<label for="recipe_status_' . $id . '">' . $label . '</label>';
				endforeach;
				echo wpautop( implode( '&nbsp;&nbsp;&nbsp;', $field_html ) );
			echo '</div>';
		echo '</div>';

	}

	public static function submit_fields(){

		global $_cooked_settings;
		$_cooked_settings = ( is_array($_cooked_settings) ? $_cooked_settings : Cooked_Settings::get() );
		$add_recipe_fields = Cooked_Add_Recipe_Fields::get();

		$submit_fields = array();
		$required_fields = ( isset($_cooked_settings['recipe_submission_required_fields']) && is_array($_cooked_settings['recipe_submission_required_fields']) ? $_cooked_settings['recipe_submission_required_fields'] : false );

		$guest_mode = self::guest_mode_active();
		$guest_name = ( $guest_mode ? ( isset( $_cooked_settings['recipe_submission_guest_name'] ) && $_cooked_settings['recipe_submission_guest_name'] != 'disabled' ? $_cooked_settings['recipe_submission_guest_name'] : false ) : false );
		$guest_email = ( $guest_mode ? ( isset( $_cooked_settings['recipe_submission_guest_email'] ) && $_cooked_settings['recipe_submission_guest_email'] != 'disabled' ? $_cooked_settings['recipe_submission_guest_email'] : false ) : false );

		if ( $guest_email ):
			array_unshift( $_cooked_settings['recipe_submission_active_fields'], 'guest_email' );
			if ( $guest_email == 'require' ):
				array_unshift( $required_fields, 'guest_email' );
			endif;
		endif;

		if ( $guest_name ):
			array_unshift( $_cooked_settings['recipe_submission_active_fields'], 'guest_name' );
			if ( $guest_name == 'require' ):
				array_unshift( $required_fields, 'guest_name' );
			endif;
		endif;

		if ( isset($_cooked_settings['recipe_submission_active_fields']) && is_array($_cooked_settings['recipe_submission_active_fields']) && !empty($_cooked_settings['recipe_submission_active_fields']) ):
			foreach ( $_cooked_settings['recipe_submission_active_fields'] as $key ):
				if ( isset( $add_recipe_fields[$key] ) ):
					if ( $required_fields && in_array( $key, $required_fields ) ):
						$add_recipe_fields[$key]['required'] = true;
					endif;
					$submit_fields[$key] = $add_recipe_fields[$key];
				endif;
			endforeach;
		endif;

		$submit_fields = apply_filters( 'cooked_user_add_recipe_fields', $submit_fields );
		return $submit_fields;

	}

	public static function ingredients_builder( $field ){

		global $_cooked_settings;

		$ing_extra_class = '';
		$ingredients = $field['value'];
		$measurements = Cooked_Measurements::get();

		if ( isset( $_cooked_settings['recipe_submission_options'] ) && in_array( 'ingredient_links', $_cooked_settings['recipe_submission_options'] ) ):
			$ing_extra_class = ' cooked-linkable';
		endif;

		$section_headings_allowed = ( isset( $_cooked_settings['recipe_submission_options'] ) && in_array( 'section_headings', $_cooked_settings['recipe_submission_options'] ) ? true : false );

		?><div id="cooked-fe-ingredients">

			<div class="cooked-ingredient-headers cooked-clearfix" style="display:none;">
				<span class="cooked-ingredient-header-amount"><?php esc_html_e('Amount','cooked'); ?></span>
				<span class="cooked-ingredient-header-measurement"><?php esc_html_e('Measurement','cooked'); ?></span>
				<span class="cooked-ingredient-header-item"><?php esc_html_e('Item','cooked'); ?></span>
			</div>

			<div id="cooked-ingredients-builder" class="cooked-sortable" style="margin-bottom:0;">

				<?php if ( isset($ingredients) && !empty($ingredients) ): ?>

					<?php foreach($ingredients as $ing_key => $value): ?>

						<?php if ( !isset($value['section_heading_name']) ): ?>

							<?php $ingredient_classes = apply_filters( 'cooked_ingredient_field_classes', 'recipe-setting-block cooked-ingredient-block cooked-clearfix', $value ); ?>

							<div class="<?php echo $ingredient_classes . $ing_extra_class; ?>">

								<i class="cooked-icon cooked-icon-drag"></i>

								<?php do_action( 'cooked_before_ingredient_fields', $ing_key, $value ); ?>

								<div class="cooked-ingredient-amount">
									<input type="text" data-ingredient-part="amount" name="_recipe_settings[ingredients][<?php echo $ing_key; ?>][amount]" value="<?php echo stripslashes( esc_attr( $value['amount'] ) ); ?>" placeholder="--">
								</div>

								<?php do_action( 'cooked_after_ingredient_amount_field', $ing_key, $value ); ?>

								<div class="cooked-ingredient-measurement">
									<select data-ingredient-part="measurement" name="_recipe_settings[ingredients][<?php echo $ing_key; ?>][measurement]">
										<option value="">--</option>
										<?php foreach($measurements as $key => $measurement):
											echo '<option value="' . $key . '"' . ( $value['measurement'] == $key ? ' selected' : '' ) . '>' . esc_attr( $measurement['plural_abbr'] ) . '</option>';
										endforeach; ?>
									</select>
								</div>

								<?php do_action( 'cooked_after_ingredient_measurement_field', $ing_key, $value ); ?>

								<div class="cooked-ingredient-name">
									<input type="text" data-ingredient-part="name" name="_recipe_settings[ingredients][<?php echo $ing_key; ?>][name]" value="<?php echo stripslashes( esc_attr( $value['name'] ) ); ?>" placeholder="<?php esc_html_e('ex. Eggs, Milk, etc.','cooked'); ?> ...">
								</div>

								<?php do_action( 'cooked_after_ingredient_name_field', $ing_key, $value ); ?>

								<span href="#" class="cooked-delete-ingredient"><i class="cooked-icon cooked-icon-times"></i></span>

							</div>

						<?php elseif ( isset($value['section_heading_name']) && $section_headings_allowed ): ?>

							<div class="recipe-setting-block cooked-ingredient-block cooked-ingredient-heading cooked-clearfix">
								<i class="cooked-icon cooked-icon-drag"></i>
								<div class="cooked-heading-name">
									<input type="text" data-ingredient-part="section_heading_name" name="_recipe_settings[ingredients][<?php echo $ing_key; ?>][section_heading_name]" value="<?php echo stripslashes( esc_attr( $value['section_heading_name'] ) ); ?>" placeholder="<?php esc_html_e('Section Heading','cooked'); ?> ...">
								</div>
								<span href="#" class="cooked-delete-ingredient"><i class="cooked-icon cooked-icon-times"></i></span>
							</div>

						<?php endif; ?>

					<?php endforeach; ?>

				<?php endif; ?>

			</div>

			<p>
				<a href="#" class="cooked-button button cooked-add-ingredient-button cooked-secondary-button"><?php esc_html_e('Add Ingredient','cooked'); ?></a>
				<?php if ( $section_headings_allowed ): ?>&nbsp;<a href="#" class="cooked-button button cooked-add-heading-button cooked-secondary-button"><?php esc_html_e('Add Section Heading','cooked'); ?></a><?php endif; ?>
			</p>

			<!-- TEMPLATES -->
			<div class="recipe-setting-block cooked-template cooked-ingredient-template cooked-clearfix<?php echo $ing_extra_class; ?>">

				<i class="cooked-icon cooked-icon-drag"></i>

				<?php do_action( 'cooked_before_ingredient_fields', false, false ); ?>

				<div class="cooked-ingredient-amount">
					<input type="text" data-ingredient-part="amount" name="" value="" placeholder="--">
				</div>

				<?php do_action( 'cooked_after_ingredient_amount_field', false, false ); ?>

				<div class="cooked-ingredient-measurement">
					<select data-ingredient-part="measurement" name="">
						<option value="">--</option>
						<?php foreach($measurements as $key => $measurement):
							echo '<option value="'.$key.'">'.esc_attr($measurement['plural_abbr']).'</option>';
						endforeach; ?>
					</select>
				</div>

				<?php do_action( 'cooked_after_ingredient_measurement_field', false, false ); ?>

				<div class="cooked-ingredient-name">
					<input type="text" data-ingredient-part="name" name="" value="" placeholder="<?php esc_html_e('ex. Eggs, Milk, etc.','cooked'); ?> ...">
				</div>

				<?php do_action( 'cooked_after_ingredient_name_field', false, false ); ?>

				<span href="#" class="cooked-delete-ingredient"><i class="cooked-icon cooked-icon-times"></i></span>

			</div>
			<?php if ( $section_headings_allowed ): ?>
			<div class="recipe-setting-block cooked-template cooked-heading-template cooked-clearfix">
				<i class="cooked-icon cooked-icon-drag"></i>
				<div class="cooked-heading-name">
					<input type="text" data-ingredient-part="section_heading_name" name="" value="" placeholder="<?php esc_html_e('Section Heading','cooked'); ?> ...">
				</div>
				<span href="#" class="cooked-delete-ingredient"><i class="cooked-icon cooked-icon-times"></i></span>
			</div>
			<?php endif; ?>
			<!-- END TEMPLATES -->

		</div><?php

	}

	public static function directions_builder( $field ){

		global $_cooked_settings;

		$directions = $field['value'];
		$section_headings_allowed = ( isset( $_cooked_settings['recipe_submission_options'] ) && in_array( 'section_headings', $_cooked_settings['recipe_submission_options'] ) ? true : false );

		?><div id="cooked-fe-directions">

			<div id="cooked-directions-builder" class="cooked-sortable">

				<?php if ( isset($directions) && !empty($directions) ): ?>

					<?php foreach($directions as $dir_key => $value): ?>

						<?php
							$dir_image = ( isset($value['image']) && $value['image'] ? esc_attr( $value['image'] ) : false );
							$image_thumb = ( $dir_image ? wp_get_attachment_image_src( $dir_image, 'cooked-square' ) : false );
						 	$image_thumb = ( isset($image_thumb[0]) && $image_thumb[0] ? esc_attr( $image_thumb[0] ) : false );
						?>

						<?php if ( !isset($value['section_heading_name']) ): ?>

							<div class="recipe-setting-block cooked-direction-block cooked-clearfix">
								<i class="cooked-icon cooked-icon-drag"></i>
								<div class="cooked-direction-image<?php echo ( isset($value['image']) && $value['image'] ? ' cooked-has-image' : '' ); ?>">
									<input type="file" data-direction-part="image" class="cooked_image_uploader" name="directions_<?php echo $dir_key; ?>_image" data-target-id="image_<?php echo $dir_key; ?>">
									<input type="hidden" data-direction-part="current_image_removable" name="current_directions_<?php echo $dir_key; ?>_image_removable" value="<?php echo ( isset($value['image']) && $value['image'] ? esc_attr( $value['image'] ) : '' ); ?>">
									<input type="hidden" data-direction-part="current_image" name="current_directions_<?php echo $dir_key; ?>_image" value="<?php echo ( isset($value['image']) && $value['image'] ? esc_attr( $value['image'] ) : '' ); ?>">
									<img data-direction-part="image" id="image_<?php echo $dir_key; ?>" src="<?php echo ( isset($value['image']) && $value['image'] ? esc_url( wp_get_attachment_thumb_url( $value['image'] ) ) : '' ); ?>" />
								</div>
								<div class="cooked-direction-content">
									<textarea data-direction-part="content" name="_recipe_settings[directions][<?php echo $dir_key; ?>][content]"><?php echo stripslashes( $value['content'] ); ?></textarea>
								</div>
								<a href="#" class="cooked-delete-direction"><i class="cooked-icon cooked-icon-times"></i></a>
							</div>

						<?php elseif ( isset($value['section_heading_name']) && $section_headings_allowed ): ?>

							<div class="recipe-setting-block cooked-direction-block cooked-direction-heading cooked-clearfix">
								<i class="cooked-icon cooked-icon-drag"></i>
								<div class="cooked-heading-name">
									<input type="text" data-direction-part="section_heading_name" name="_recipe_settings[directions][<?php echo $dir_key; ?>][section_heading_name]" value="<?php echo stripslashes( esc_attr( $value['section_heading_name'] ) ); ?>" placeholder="<?php esc_html_e('Section Heading','cooked'); ?> ...">
								</div>
								<a href="#" class="cooked-delete-direction"><i class="cooked-icon cooked-icon-times"></i></a>
							</div>

						<?php endif; ?>

					<?php endforeach; ?>

				<?php endif; ?>

			</div>

			<p>
				<a href="#" class="cooked-button cooked-add-direction-button cooked-secondary-button"><?php esc_html_e('Add Direction','cooked'); ?></a>
				<?php if ( $section_headings_allowed ): ?>&nbsp;<a href="#" class="cooked-button cooked-add-heading-button cooked-secondary-button"><?php esc_html_e('Add Section Heading','cooked'); ?></a><?php endif; ?>
			</p>

			<!-- TEMPLATES -->
			<div class="recipe-setting-block cooked-template cooked-direction-template cooked-clearfix">
				<i class="cooked-icon cooked-icon-drag"></i>
				<div class="cooked-direction-image">
					<input type="file" data-direction-part="image" class="cooked_image_uploader" name="" data-target-id="">
					<img data-direction-part="image" id="" src="" />
				</div>
				<div class="cooked-direction-content">
					<textarea data-direction-part="content" name=""></textarea>
				</div>
				<a href="#" class="cooked-delete-direction"><i class="cooked-icon cooked-icon-times"></i></a>
			</div>
			<?php if ( $section_headings_allowed ): ?>
			<div class="recipe-setting-block cooked-template cooked-heading-template cooked-clearfix">
				<i class="cooked-icon cooked-icon-drag"></i>
				<div class="cooked-heading-name">
					<input type="text" data-direction-part="section_heading_name" name="" value="" placeholder="<?php esc_html_e('Section Heading','cooked'); ?> ...">
				</div>
				<a href="#" class="cooked-delete-direction"><i class="cooked-icon cooked-icon-times"></i></a>
			</div>
			<?php endif; ?>
			<!-- END TEMPLATES -->

		</div><?php

	}

	public static function nutrition_facts( $field ){

		global $_cooked_settings;

		$nutrition = $field['value'];
		$_nutrition_facts = Cooked_Measurements::nutrition_facts();

		foreach ( $_nutrition_facts as $section => $fields ):
			foreach( $fields as $key => $field ):
				if ( isset( $field['subs'] ) ):
					if ( !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$key]) ||
						 !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$key]) && !$nutrition[$key] ):
						unset( $_nutrition_facts[$section][$key] );
					endif;
					foreach ( $field['subs'] as $sub_key => $sub_field ):
						if ( !in_array( $sub_key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$sub_key]) ||
						 	 !in_array( $sub_key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$sub_key]) && !$nutrition[$sub_key] ):
							unset( $_nutrition_facts[$section][$key]['subs'][$sub_key] );
						endif;
					endforeach;
				else:
					if ( !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$key]) ||
						 !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$key]) && !$nutrition[$key] ):
						unset( $_nutrition_facts[$section][$key] );
					endif;
				endif;
			endforeach;
		endforeach;

		$_nutrition_facts = ( empty( $_nutrition_facts['top'] ) && empty( $_nutrition_facts['mid'] ) && empty( $_nutrition_facts['main'] ) && empty( $_nutrition_facts['bottom'] ) ? array() : $_nutrition_facts );

		// Show the Nutrition Facts section by default?
		$visible = ( !empty($nutrition) ? true : false );

		if ( empty($nutrition) && empty($_nutrition_facts) )
			return false;

		if ( $visible ):
			$visible = false;
			foreach( $nutrition as $val ):
				if ( $val ): $visible = true; endif;
			endforeach;
		endif;

		echo '<div id="cooked-fe-nutrition" class="cooked-clearfix">';

			$nut_button_text = ( $visible ? esc_html__( 'Hide Nutrition Facts', 'cooked-pro' ) : esc_html__( 'Show Nutrition Facts', 'cooked-pro' ) );

			echo '<p><a href="#" class="cooked-button cooked-add-nutrition-button cooked-secondary-button' . ( $visible ? ' cooked-active' : '' ) . '">' . $nut_button_text . '</a></p>';

			echo '<div class="cooked-nutrition-toggle"' . ( $visible ? ' style="display:block;"' : '' ) . '>';
				echo '<div class="cooked-nut-fields">';

					$nut_loops = 0;
					foreach( $_nutrition_facts as $nutrition_facts ):
						foreach( $nutrition_facts as $slug => $nf ):

							$nut_loops++;
							if ( $nut_loops == 1 ): echo '<div class="cooked-clearfix">'; endif;
							echo '<span class="cooked-field cooked-field-column-50">';
								echo '<label for="' . $slug . '">' . $nf['name'] . '</label>';
								echo '<input type="' . $nf['type'] . '"' . ( $nf['type'] == 'number' ? ' step="any"' : '' ) . ' placeholder="--" name="_recipe_settings[nutrition][' . $slug . ']" id="' . $slug . '" value="' . ( isset($nutrition[$slug]) ? stripslashes( esc_attr($nutrition[$slug]) ) : '' ) . '" />';
							echo '</span>';
							if ( $nut_loops == 2 ): echo '</div>'; $nut_loops = 0; endif;
							if ( isset($nf['subs']) ):
								foreach( $nf['subs'] as $sub_slug => $sub_nf ):
									$nut_loops++;
									if ( $nut_loops == 1 ): echo '<div class="cooked-clearfix">'; endif;
									echo '<span class="cooked-field cooked-field-column-50">';
										echo '<label for="' . $sub_slug . '">' . $sub_nf['name'] . '</label>';
										echo '<input type="' . $sub_nf['type'] . '"' . ( $nf['type'] == 'number' ? ' step="any"' : '' ) . ' placeholder="--" name="_recipe_settings[nutrition][' . $sub_slug . ']" id="' . $sub_slug . '" value="' . ( isset($nutrition[$sub_slug]) ? stripslashes( esc_attr($nutrition[$sub_slug]) ) : '' ) . '" />';
									echo '</span>';
									if ( $nut_loops == 2 ): echo '</div>'; $nut_loops = 0; endif;
								endforeach;
							endif;
						endforeach;
					endforeach;

					if ( $nut_loops == 1 ): echo '</div>'; $nut_loops = 0; endif;

				echo '</div>';
				echo '<div class="cooked-nut-label-wrap">';

					?><section id="cooked-nutrition-label" class="cooked-nut-label-1">

						<div class="cooked-h2"><?php esc_html_e('Nutrition Facts','cooked'); ?></div>

						<?php $nutrition_facts = $_nutrition_facts['top'];
						foreach( $nutrition_facts as $slug => $nf ):
							echo '<div class="cooked-p">' . $nf['name'] . ' <strong class="cooked-nut-label" data-labeltype="' . $slug . '">___</strong></div>';
						endforeach; ?>

						<hr class="cooked-nut-hr" />

						<div class="cooked-ul">
							<div class="cooked-li"><strong class="cooked-nut-heading"><?php esc_html_e('Amount Per Serving','cooked'); ?></strong></div>

							<?php $nutrition_facts = $_nutrition_facts['mid'];
							foreach( $nutrition_facts as $slug => $nf ):
								if ( $slug != 'calories_fat' ):
									echo '<div class="cooked-li"><strong>' . $nf['name'] . '</strong> <strong class="cooked-nut-label" data-labeltype="' . $slug . '">___</strong>';
										echo '<div class="cooked-ul cooked-calories-fat cooked-right"><div class="cooked-li">' . esc_attr($nutrition_facts['calories_fat']['name']) . ' <strong class="cooked-nut-label" data-labeltype="calories_fat">___</strong></div></div>';
									echo '</div>';
								endif;
							endforeach; ?>

							<div class="cooked-li cooked-nut-spacer"></div>
							<div class="cooked-li cooked-nut-no-border"><strong class="cooked-nut-heading cooked-nut-right"><?php esc_html_e('% Daily Value *','cooked'); ?></strong></div>

							<?php $nutrition_facts = $_nutrition_facts['main'];
							$nut_loops = 0;

							foreach( $nutrition_facts as $slug => $nf ):

								echo '<div class="cooked-li">';
								echo '<strong>' . $nf['name'] . '</strong> <strong class="cooked-nut-label" data-labeltype="' . $slug . '">___ </strong>' . ( isset($nf['measurement']) ? '<strong class="cooked-nut-label" data-labeltype="' . $slug . '_measurement">' . $nf['measurement'] . '</strong>' : '' );
								echo ( isset( $nf['pdv'] ) ? '<strong class="cooked-nut-right"><span class="cooked-nut-percent" data-pdv="' . $nf['pdv'] . '" data-labeltype="' . $slug . '">0</span>%</strong>' : '' );

								if ( isset($nf['subs']) ):
									foreach( $nf['subs'] as $sub_slug => $sub_nf ):
										echo '<div class="cooked-ul"><div class="cooked-li">';
											echo '<strong>' . $sub_nf['name'] . '</strong> <strong class="cooked-nut-label" data-labeltype="' . $sub_slug . '">___ </strong>' . ( isset($sub_nf['measurement']) ? '<strong class="cooked-nut-label" data-labeltype="' . $sub_slug . '_measurement">' . $sub_nf['measurement'] . '</strong>' : '' );
											echo ( isset( $sub_nf['pdv'] ) ? '<strong class="cooked-nut-right"><span class="cooked-nut-percent" data-pdv="' . $sub_nf['pdv'] . '" data-labeltype="' . $sub_slug . '">0</span>%</strong>' : '' );
										echo '</div></div>';
									endforeach;
								endif;

								echo '</div>';

							endforeach; ?>
						</div>

						<hr class="cooked-nut-hr" />

						<div class="cooked-ul cooked-nut-bottom cooked-clearfix">
							<?php $nutrition_facts = $_nutrition_facts['bottom'];
							foreach( $nutrition_facts as $slug => $nf ):
								echo '<div class="cooked-li">';
								echo '<strong>' . $nf['name'] . ' <span class="cooked-nut-right"><span class="cooked-nut-percent cooked-nut-label" data-labeltype="' . $slug . '">0</span>%</span></strong>';
								echo '</div>';
							endforeach; ?>
						</div>

						<div class="cooked-p cooked-daily-value-text"><?php esc_html_e('Percent Daily Values are based on a 2,000 calorie diet. Your daily value may be higher or lower depending on your calorie needs.','cooked'); ?></div>

					</section><?php

				echo '</div>';
			echo '</div>';
		echo '</div>';

	}

	public static function recipe_has_nutrition_facts( $field ){

		global $_cooked_settings;

		$nutrition = $field['value'];
		$_nutrition_facts = Cooked_Measurements::nutrition_facts();

		foreach ( $_nutrition_facts as $section => $fields ):
			foreach( $fields as $key => $field ):
				if ( isset( $field['subs'] ) ):
					if ( !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$key]) ||
						 !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$key]) && !$nutrition[$key] ):
						unset( $_nutrition_facts[$section][$key] );
					endif;
					foreach ( $field['subs'] as $sub_key => $sub_field ):
						if ( !in_array( $sub_key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$sub_key]) ||
						 	 !in_array( $sub_key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$sub_key]) && !$nutrition[$sub_key] ):
							unset( $_nutrition_facts[$section][$key]['subs'][$sub_key] );
						endif;
					endforeach;
				else:
					if ( !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && !isset($nutrition[$key]) ||
						 !in_array( $key, $_cooked_settings['recipe_submission_active_nut_fields'] ) && isset($nutrition[$key]) && !$nutrition[$key] ):
						unset( $_nutrition_facts[$section][$key] );
					endif;
				endif;
			endforeach;
		endforeach;

		$_nutrition_facts = ( empty( $_nutrition_facts['top'] ) && empty( $_nutrition_facts['mid'] ) && empty( $_nutrition_facts['main'] ) && empty( $_nutrition_facts['bottom'] ) ? array() : $_nutrition_facts );

		return ( empty( $_nutrition_facts ) ? false : true );

	}

	public static function process_recipe_data( $data, $type = 'add' ){

		$guest_mode = self::guest_mode_active();

		if( !is_user_logged_in() && !$guest_mode ) {
			return false;
		}

		if( empty( $_POST['cooked_' . $type . '_recipe'] ) ) {
			return false;
		}

		do_action( 'cooked_pre_process_' . $type . '_recipe_form' );

		// Video URL
		if ( isset( $data['_recipe_settings']['video_url'] ) ):
			$data['_recipe_settings']['gallery']['video_url'] = $data['_recipe_settings']['video_url'];
			$data['_recipe_settings']['gallery']['type'] = 'cooked';
		endif;

		// Nutrition Facts
		$has_nutrition = ( !empty($data['_recipe_settings']['nutrition']) ? true : false );
		if ( $has_nutrition ):
			$has_nutrition = false;
			foreach( $data['_recipe_settings']['nutrition'] as $val ):
				if ( $val ): $has_nutrition = true; endif;
			endforeach;
		endif;

		// Check for required fields.
		$submit_fields = self::submit_fields();
		$ing_dir_error = false;
		foreach( $submit_fields as $field_name => $field ):
			if ( isset($field['required']) && $field['required'] ):
				if ( $field_name == 'ingredients' || $field_name == 'directions' ):
					if ( !isset($data['_recipe_settings'][$field_name]) ):
						$ing_dir_error = true;
					else:
						foreach( $data['_recipe_settings'][$field_name] as $id => $values ):
							foreach( $values as $key => $value ):
								if ( $field_name == 'directions' ):
									if ( isset( $data['current_directions_' . $id . '_image'] ) && $data['current_directions_' . $id . '_image'] &&
										 isset( $_FILES['directions_' . $id . '_image']['error'] ) && !$_FILES['directions_' . $id . '_image']['error'] ):
										wp_delete_attachment( $data['current_directions_' . $id . '_image'], true );
									elseif ( isset( $data['current_directions_' . $id . '_image_removable'] ) && !$data['current_directions_' . $id . '_image_removable'] ):
										wp_delete_attachment( $data['current_directions_' . $id . '_image'], true );
										$data['current_directions_' . $id . '_image'] = false;
										$data['_recipe_settings']['directions'][$id]['image'] = false;
									endif;
									if ( isset( $data['current_directions_' . $id . '_image'] ) && !$data['current_directions_' . $id . '_image'] ||
										 isset( $_FILES['directions_' . $id . '_image']['error'] ) && $_FILES['directions_' . $id . '_image']['error'] ):
										$ing_dir_error = true;
									endif;
								elseif ( $key != 'url' && $key != 'measurement' && $key != 'amount' && $key != 'image' && !$value ):
									$ing_dir_error = true;
								endif;
							endforeach;
						endforeach;
					endif;
				elseif ( !isset($data['_recipe_settings'][$field_name]) || isset($data['_recipe_settings'][$field_name]) && !$data['_recipe_settings'][$field_name] || $field_name == 'nutrition' && !$has_nutrition ):
					cooked_set_error( 'required_fields', __( 'Please fill out all required fields.', 'cooked-pro' ) );
				endif;
			endif;
		endforeach;

		// Check for direction images
		if ( isset($_FILES) && !empty($_FILES) && isset($data['_recipe_settings']['directions']) && !empty($data['_recipe_settings']['directions']) ):
			foreach( $data['_recipe_settings']['directions'] as $key => $direction ):
				if ( $type == 'add' ):
					if ( isset( $_FILES['directions_' . $key . '_image']['name'] ) && isset( $_FILES['directions_' . $key . '_image']['error'] ) && !$_FILES['directions_' . $key . '_image']['error'] ):
						$data['_recipe_settings']['directions'][$key]['image'] = $_FILES['directions_' . $key . '_image'];
					endif;
				elseif ( $type == 'edit' ):
					if ( isset( $_FILES['directions_' . $key . '_image']['name'] ) && isset( $_FILES['directions_' . $key . '_image']['error'] ) && !$_FILES['directions_' . $key . '_image']['error'] ):
						$data['_recipe_settings']['directions'][$key]['image'] = $_FILES['directions_' . $key . '_image'];
					else:
						$data['_recipe_settings']['directions'][$key]['image'] = ( isset($data['current_directions_' . $key . '_image']) ? $data['current_directions_' . $key . '_image'] : false );
					endif;
				endif;
			endforeach;
		endif;

		do_action( 'cooked_process_' . $type . '_recipe_form' );

		if ( !isset($data['_recipe_settings']['ingredients']) || isset($data['_recipe_settings']['ingredients']) && empty($data['_recipe_settings']['ingredients']) ):
			cooked_set_error( 'required_fields', __( 'Please be sure to include at least one ingredient.', 'cooked-pro' ) );
		elseif ( !isset($data['_recipe_settings']['directions']) || isset($data['_recipe_settings']['directions']) && empty($data['_recipe_settings']['directions']) ):
			cooked_set_error( 'required_fields', __( 'Please be sure to include at least one direction.', 'cooked-pro' ) );
		endif;

		return $data;

	}

	public static function process_add_recipe_form( $data ) {

		$guest_mode = self::guest_mode_active();

		if ( wp_verify_nonce( $data['cooked_add_recipe_nonce'], 'cooked-add-recipe-nonce' ) ) {

			$data = self::process_recipe_data( $data, 'add' );

			// Check for errors and redirect if none present
			$errors = cooked_get_errors();

			if ( empty( $errors ) && $data ) {

				// Remove Empty Ingredients
				foreach( $data['_recipe_settings']['ingredients'] as $key => $ingredient ):
					if ( isset($ingredient['name']) && !$ingredient['name'] || isset($ingredient['section_heading_name']) && !$ingredient['section_heading_name'] ):
						unset( $data['_recipe_settings']['ingredients'][$key] );
					endif;
					if ( isset($ingredient['measurement']) && $ingredient['measurement'] && isset($ingredient['amount']) && !$ingredient['amount'] ):
						$data['_recipe_settings']['ingredients'][$key]['measurement'] = false;
					endif;
				endforeach;

				// Remove Empty Directions
				foreach( $data['_recipe_settings']['directions'] as $key => $direction ):
					if ( isset($direction['content']) && !$direction['content'] && isset($direction['section_heading_name']) && !$direction['section_heading_name'] ):
						unset( $data['_recipe_settings']['directions'][$key] );
					endif;
				endforeach;

				// Move taxonomies to seperate array
				$category_id = ( isset( $data['_recipe_settings']['category'] ) ? $data['_recipe_settings']['category'] : false );
				$cuisine_id = ( isset( $data['_recipe_settings']['cuisine'] ) ? $data['_recipe_settings']['cuisine'] : false );
				$cooking_method_id = ( isset( $data['_recipe_settings']['cooking_method'] ) ? $data['_recipe_settings']['cooking_method'] : false );
				$diet_id = ( isset( $data['_recipe_settings']['diet'] ) ? $data['_recipe_settings']['diet'] : false );

				unset( $data['_recipe_settings']['category'] );
				unset( $data['_recipe_settings']['cuisine'] );
				unset( $data['_recipe_settings']['cooking_method'] );
				unset( $data['_recipe_settings']['diet'] );

				$_cooked_settings = Cooked_Settings::get();
				$_recipe_settings = Cooked_Recipe_Meta::meta_cleanup( $data['_recipe_settings'] );

				$recipe = array(
					'post_content' => ( isset($_cooked_settings['default_content']) ? $_cooked_settings['default_content'] : Cooked_Recipes::default_content() ),
					'post_title' => $_recipe_settings['post_title'],
					'post_excerpt' => ( $_recipe_settings['excerpt'] ? $_recipe_settings['excerpt'] : $_recipe_settings['post_title'] ),
					'post_status' => 'publish',
					'post_type' => 'cp_recipe',
					'comment_status' => esc_attr( get_option('default_comment_status') )
				);

				if ( $guest_mode && !is_user_logged_in() ):
					$admins = get_users( array( 'role' => 'administrator' ) );
					$guest_author = $admins[0]->ID;
					$recipe['post_author'] = $guest_author;
				else:
					$recipe['post_author'] = get_current_user_id();
				endif;

				$recipe_id = wp_insert_post( $recipe );
				if ( $recipe_id ):

					// Set taxonomy values
					if ( $category_id ): wp_set_object_terms( $recipe_id, $category_id, 'cp_recipe_category' ); endif;
					if ( $cuisine_id ): wp_set_object_terms( $recipe_id, $cuisine_id, 'cp_recipe_cuisine' ); endif;
					if ( $cooking_method_id ): wp_set_object_terms( $recipe_id, $cooking_method_id, 'cp_recipe_cooking_method' ); endif;
					if ( $diet_id ): wp_set_object_terms( $recipe_id, $diet_id, 'cp_recipe_diet' ); endif;

					// Get Status Defaults
					if ( isset($_cooked_settings['recipe_submission_pending_default']) ):
						$pending = ( in_array( 'active', $_cooked_settings['recipe_submission_pending_default'] ) ? true : false );
					else:
						$pending = false;
					endif;

					$_recipe_settings['cooked_version'] = COOKED_VERSION;
					update_post_meta( $recipe_id, '_recipe_pending', $pending );
					update_post_meta( $recipe_id, '_recipe_draft', false );
					update_post_meta( $recipe_id, '_recipe_settings', $_recipe_settings );

					// Set Featured Image
					if ( isset($data['_recipe_settings']['featured_image']) && $data['_recipe_settings']['featured_image'] ):
						$attachment_id = self::set_featured_image( $data['_recipe_settings']['featured_image'], $recipe_id );
						$_recipe_settings['featured_image'] = ( $attachment_id ? $attachment_id : false );
					endif;

					// Set Direction Images
					foreach( $data['_recipe_settings']['directions'] as $key => $direction ):
						if ( isset( $direction['image']['name'] ) ):
							$attachment_id = self::add_direction_image( $direction['image'], $recipe_id, $key );
							$_recipe_settings['directions'][$key]['image'] = $attachment_id;
						endif;
					endforeach;

					update_post_meta( $recipe_id, '_recipe_settings', $_recipe_settings );

					// Redirect to Profile page?
					if ( is_user_logged_in() && isset( $_cooked_settings['profile_page'] ) && $_cooked_settings['profile_page'] ):
						$redirect_url = add_query_arg( array( 'cooked_pn' => 'add', 'cooked_msg' => 'submit_success', 'rid' => $recipe_id ), get_permalink( $_cooked_settings['profile_page'] ) );
					else:
						$redirect_url = add_query_arg( array( 'cooked_msg' => 'submit_success', 'rid' => $recipe_id ), get_permalink() );
					endif;

					// Send Notification Email?
					if ( isset( $_cooked_settings['recipe_submission_notification_enabled'] )
					&& $_cooked_settings['recipe_submission_notification_enabled']
					&& isset( $_cooked_settings['recipe_submission_notification_email'] )
					&& $_cooked_settings['recipe_submission_notification_email'] ):

						$subject = ( $pending ? apply_filters( 'cooked_recipe_submission_email_subject', esc_html__( 'New Recipe - Approval Required', 'cooked-pro' ) ) : apply_filters( 'cooked_recipe_submission_email_subject', esc_html__( 'New Recipe Submission', 'cooked-pro' ) ) );
						$content = ( $pending ? apply_filters( 'cooked_recipe_submission_email_content', sprintf( esc_html__( 'A new recipe has been submitted on your website. You can %s or go directly to your %s screen.', 'cooked-pro' ), '<a href="' . get_permalink( $recipe_id ) . '">' . esc_html__( 'view the recipe here', 'cooked-pro' ) . '</a>', '<a href="' . get_admin_url( null, 'admin.php?page=cooked_pending' ) . '">' . esc_html__( 'pending recipes', 'cooked-pro' ) . '</a>' ) ) : apply_filters( 'cooked_recipe_submission_email_content', sprintf( esc_html__( 'A new recipe has been submitted on your website. You can %s.', 'cooked-pro' ), '<a href="' . get_permalink( $recipe_id ) . '">' . esc_html__( 'view the recipe here', 'cooked-pro' ) . '</a>' ) ) );

						$email = array(
							'to' => esc_html( $_cooked_settings['recipe_submission_notification_email'] ),
							'subject' => $subject,
							'content' => $content,
						);
						self::send_email( $email );

					endif;

					wp_redirect( $redirect_url );
					exit;

				else:
					cooked_set_error( 'recipe_submit', __( 'There was an error submitting your recipe. Please try again.', 'cooked-pro' ) );
				endif;

			}

		}

	}

	public static function ajax_delete_recipe(){

		global $_cooked_settings;

		if ( !isset( $_cooked_settings['recipe_submission_delete_allowed'] ) || isset( $_cooked_settings['recipe_submission_delete_allowed'] ) && !$_cooked_settings['recipe_submission_delete_allowed'] ):
			echo 'error';
			wp_die();
		endif;

		if ( isset( $_POST['recipe_id'] ) && is_numeric( $_POST['recipe_id'] ) ):

			$recipe_id = esc_attr( $_POST['recipe_id'] );
			if ( Cooked_Profiles::is_my_recipe( $recipe_id ) || current_user_can('edit_cooked_recipes') ):
				self::delete_recipe_meta( $recipe_id );
				$delete_recipe = wp_delete_post( $recipe_id, true );
				if ( !$delete_recipe ):
					echo 'error';
				endif;
			else:
				echo 'error';
			endif;

		endif;

		wp_die();

	}

	public static function delete_recipe_meta( $recipe_id ){

		global $_cooked_settings;

		if ( !isset( $_cooked_settings['recipe_submission_delete_allowed'] ) || isset( $_cooked_settings['recipe_submission_delete_allowed'] ) && !$_cooked_settings['recipe_submission_delete_allowed'] )
			return;

		if ( Cooked_Profiles::is_my_recipe( $recipe_id ) || current_user_can('edit_cooked_recipes') ):

			global $wpdb;

		    $args = array(
		        'post_type'         => 'attachment',
		        'post_status'       => 'any',
		        'posts_per_page'    => -1,
		        'post_parent'       => $recipe_id
		    );
		    $attachments = new WP_Query($args);
		    $attachment_ids = array();
		    if($attachments->have_posts()) : while($attachments->have_posts()) : $attachments->the_post();
		            $attachment_ids[] = get_the_id();
		        endwhile;
		    endif;
		    wp_reset_postdata();

		    if(!empty($attachment_ids)) :
		        $delete_attachments_query = $wpdb->prepare('DELETE FROM %1$s WHERE %1$s.ID IN (%2$s)', $wpdb->posts, join(',', $attachment_ids));
		        $wpdb->query($delete_attachments_query);
		    endif;

		endif;

	}

	public static function process_delete_recipe() {

		global $_cooked_settings, $post;

		if ( isset( $_GET['rid'] ) && is_numeric( $_GET['rid'] ) ):

			$recipe_id = esc_attr( $_GET['rid'] );
			if ( Cooked_Profiles::is_my_recipe( $recipe_id ) || current_user_can('edit_cooked_recipes') ):

				// Redirect to Profile page?
				if ( isset( $_cooked_settings['profile_page'] ) && $_cooked_settings['profile_page'] ):
					$redirect_url = add_query_arg( array( 'cooked_pn' => 'recipes', 'cooked_msg' => 'delete_success' ), get_permalink( $_cooked_settings['profile_page'] ) );
				else:
					$redirect_url = add_query_arg( array( 'cooked_msg' => 'delete_success' ), get_permalink( $post->ID ) );
				endif;

				$redirect_url = remove_query_arg( array( 'cooked_action' ), $redirect_url );

				wp_redirect( $redirect_url );
				exit;

			endif;

		endif;

	}

	public static function process_edit_recipe_form( $data ) {

		if ( wp_verify_nonce( $data['cooked_edit_recipe_nonce'], 'cooked-edit-recipe-nonce' ) ) {

			$data = self::process_recipe_data( $data, 'edit' );

			// Check for errors and redirect if none present
			$errors = cooked_get_errors();

			if ( empty( $errors ) && $data ) {

				// Remove Empty Ingredients
				foreach( $data['_recipe_settings']['ingredients'] as $key => $ingredient ):
					if ( isset($ingredient['name']) && !$ingredient['name'] || !isset($ingredient['name']) && isset($ingredient['measurement']) || isset($ingredient['section_heading_name']) && !$ingredient['section_heading_name'] ):
						unset( $data['_recipe_settings']['ingredients'][$key] );
					elseif ( isset($ingredient['measurement']) && $ingredient['measurement'] && isset($ingredient['amount']) && !$ingredient['amount'] ):
						$data['_recipe_settings']['ingredients'][$key]['measurement'] = false;
					endif;
				endforeach;

				// Remove Empty Directions
				foreach( $data['_recipe_settings']['directions'] as $key => $direction ):
					if ( !isset($direction['content']) && !isset($direction['image']) && !isset($direction['section_heading_name']) || isset($direction['content']) && !$direction['content'] && isset($direction['image']) && !$direction['image'] || isset($direction['section_heading_name']) && !$direction['section_heading_name'] ):
						unset( $data['_recipe_settings']['directions'][$key] );
					endif;
				endforeach;

				// Move taxonomies to seperate array
				$category_id = ( isset( $data['_recipe_settings']['category'] ) ? $data['_recipe_settings']['category'] : false );
				$cuisine_id = ( isset( $data['_recipe_settings']['cuisine'] ) ? $data['_recipe_settings']['cuisine'] : false );
				$cooking_method_id = ( isset( $data['_recipe_settings']['cooking_method'] ) ? $data['_recipe_settings']['cooking_method'] : false );
				$diet_id = ( isset( $data['_recipe_settings']['diet'] ) ? $data['_recipe_settings']['diet'] : false );

				unset( $data['_recipe_settings']['category'] );
				unset( $data['_recipe_settings']['cuisine'] );
				unset( $data['_recipe_settings']['cooking_method'] );
				unset( $data['_recipe_settings']['diet'] );

				$_cooked_settings = Cooked_Settings::get();
				$_recipe_settings = Cooked_Recipe_Meta::meta_cleanup( $data['_recipe_settings'] );
				$recipe_id = ( isset($_POST['recipe_id']) ? esc_html( $_POST['recipe_id'] ) : false );

				if ( Cooked_Profiles::is_my_recipe( $recipe_id ) || current_user_can('edit_cooked_recipes') ):

					$recipe = array(
						'ID' => $recipe_id,
						'post_content' => ( isset($_cooked_settings['default_content']) ? $_cooked_settings['default_content'] : Cooked_Recipes::default_content() ),
						'post_title' => $_recipe_settings['post_title'],
						'post_excerpt' => ( $_recipe_settings['excerpt'] ? $_recipe_settings['excerpt'] : $_recipe_settings['post_title'] ),
						'post_status' => 'publish',
						'post_type' => 'cp_recipe'
					);

					$recipe_id = wp_update_post( $recipe, true );
					if (is_wp_error($recipe_id)):

						$errors = $recipe_id->get_error_messages();
						foreach ($errors as $error):
							cooked_set_error( 'required_fields', $error );
						endforeach;

					elseif( $recipe_id ):

						// Set taxonomy values
						if ( $category_id ): wp_set_object_terms( $recipe_id, array( $category_id ), 'cp_recipe_category' ); endif;
						if ( $cuisine_id ): wp_set_object_terms( $recipe_id, array( $cuisine_id ), 'cp_recipe_cuisine' ); endif;
						if ( $cooking_method_id ): wp_set_object_terms( $recipe_id, array( $cooking_method_id ), 'cp_recipe_cooking_method' ); endif;
						if ( $diet_id ): wp_set_object_terms( $recipe_id, array( $diet_id ), 'cp_recipe_diet' ); endif;

						// Get Status Defaults
						if ( isset($_cooked_settings['recipe_submission_pending_default']) ):
							$pending = ( in_array( 'active', $_cooked_settings['recipe_submission_pending_default'] ) ? true : false );
						else:
							$pending = false;
						endif;

						// Setting recipe to draft?
						$draft = ( isset( $_POST['recipe_status'][0] ) && $_POST['recipe_status'][0] == 'draft' ? true : false );

						// Set Featured Image
						if ( isset($data['_recipe_settings']['featured_image']) && $data['_recipe_settings']['featured_image'] ):
							if ( !is_numeric( $data['_recipe_settings']['featured_image'] ) ):
								$current_attachment_id = get_post_thumbnail_id( $recipe_id );
								if ( $current_attachment_id ): wp_delete_attachment( $current_attachment_id ); endif;
								$attachment_id = self::set_featured_image( $data['_recipe_settings']['featured_image'], $recipe_id );
								if ( $attachment_id ):
									$_recipe_settings['featured_image'] = ( $attachment_id ? $attachment_id : false );
								else:
									$_recipe_settings['featured_image'] = false;
								endif;
							else:
								$_recipe_settings['featured_image'] = esc_attr( $data['_recipe_settings']['featured_image'] );
							endif;
						else:
							$current_attachment_id = get_post_thumbnail_id( $recipe_id );
							if ( $current_attachment_id ): wp_delete_attachment( $current_attachment_id ); endif;
							$_recipe_settings['featured_image'] = false;
							$_recipe_settings['featured_image_url'] = false;
						endif;

						// Set Direction Images
						foreach( $data['_recipe_settings']['directions'] as $key => $direction ):
							if ( isset( $direction['image']['name'] ) ):
								$attachment_id = self::add_direction_image( $direction['image'], $recipe_id, $key );
								$_recipe_settings['directions'][$key]['image'] = $attachment_id;
							else:
								$_recipe_settings['directions'][$key]['image'] = $direction['image'];
							endif;
						endforeach;

						$_recipe_settings['cooked_version'] = COOKED_VERSION;
						update_post_meta( $recipe_id, '_recipe_pending', $pending );
						update_post_meta( $recipe_id, '_recipe_draft', $draft );
						update_post_meta( $recipe_id, '_recipe_settings', $_recipe_settings );

						// Send Notification Email?
						if ( $pending
						&& isset( $_cooked_settings['recipe_submission_notification_enabled'] )
						&& $_cooked_settings['recipe_submission_notification_enabled']
						&& isset( $_cooked_settings['recipe_submission_notification_email'] )
						&& $_cooked_settings['recipe_submission_notification_email'] ):

							$email = array(
								'to' => esc_html( $_cooked_settings['recipe_submission_notification_email'] ),
								'subject' => apply_filters( 'cooked_recipe_update_email_subject', esc_html__( 'Recipe Update - Approval Required', 'cooked-pro' ) ),
								'content' => apply_filters( 'cooked_recipe_update_email_content', sprintf( esc_html__( 'A recipe has been updated on your website and requires your approval. You can %s or go directly to your %s screen.', 'cooked-pro' ), '<a href="' . get_permalink( $recipe_id ) . '">' . esc_html__( 'view the recipe here', 'cooked-pro' ) . '</a>', '<a href="' . get_admin_url( null, 'admin.php?page=cooked_pending' ) . '">' . esc_html__( 'pending recipes', 'cooked-pro' ) . '</a>' ) ),
							);
							self::send_email( $email );

						endif;

						$redirect_url = add_query_arg( array( 'cooked_msg' => 'edit_success', 'edit_recipe' => '' ), get_permalink( $recipe_id ) );
						wp_redirect( $redirect_url );
						exit;

					else:
						cooked_set_error( 'recipe_edit', __( 'There was an error while editing your recipe. Please try again.', 'cooked-pro' ) );
					endif;

				else:
					cooked_set_error( 'recipe_edit', __( 'You do not have permission to edit this recipe.', 'cooked-pro' ) );
				endif;

			} else {

				$recipe_id = ( isset($_POST['recipe_id']) ? esc_html( $_POST['recipe_id'] ) : false );
				$redirect_url = add_query_arg( array( 'edit_recipe' => '' ), get_permalink( $recipe_id ) );
				wp_redirect( $redirect_url );
				exit;

			}

		}

	}

	public function set_featured_image( $image_url = false, $id = false ){

		if ( $image_url && $id ):

            $_uploads = wp_upload_dir();
            $recipe_image_dir = trailingslashit( $_uploads['basedir'] ) . 'cooked/images/recipes';
            $recipe_image_url = trailingslashit( $_uploads['baseurl'] ) . 'cooked/images/recipes';

            if ( wp_mkdir_p( $recipe_image_dir ) ):
                if ( file_exists($image_url) || file_exists($recipe_image_dir . "/recipe_" . $id . ".jpg") ):
                    if ( file_exists($recipe_image_dir . "/recipe_" . $id . ".jpg") ):
                        unlink( $recipe_image_dir . "/recipe_" . $id . ".jpg" );
                    endif;
                    if ( file_exists($image_url) ):
                        rename( $image_url, $recipe_image_dir . "/recipe_" . $id . ".jpg" );
                    endif;
                endif;
            endif;

            $_image_file = $recipe_image_dir . "/recipe_" . $id . ".jpg";

			$_wp_filetype = wp_check_filetype( $_image_file, null );
		    $attachment = array(
		        'post_mime_type' => $_wp_filetype['type'],
		        'post_title' => sanitize_file_name( "recipe_" . $id . ".jpg" ),
		        'post_content' => '',
		        'post_status' => 'inherit'
		    );
		    $attach_id = wp_insert_attachment( $attachment, $_image_file, $id );
		    if ( defined( 'ABSPATH' ) && file_exists( ABSPATH . 'wp-admin/includes/image.php' ) ):
		    	require_once( ABSPATH . 'wp-admin/includes/image.php' );
    			$attach_data = wp_generate_attachment_metadata( $attach_id, $_image_file );
    			$update_attachment = wp_update_attachment_metadata( $attach_id, $attach_data );
		    	$set_thumbnail = set_post_thumbnail( $id, $attach_id );
		    	return $attach_id;
		    endif;

        endif;

        return false;

	}

	public function add_direction_image( $file = false, $id = false, $key = false ){

		if ( isset($file['name']) && $id && $key ):

			$_uploads = wp_upload_dir();
	        $recipe_direction_dir = trailingslashit( $_uploads['basedir'] ) . 'cooked/images/recipes/' . $id . '/directions';
            $recipe_direction_url = trailingslashit( $_uploads['baseurl'] ) . 'cooked/images/recipes/' . $id . '/directions';

	        if ( wp_mkdir_p( $recipe_direction_dir ) ):

	        	$source = $file['tmp_name'];
	        	$file_name = $key . '-' . $file['name'];
	        	$destination = trailingslashit( $recipe_direction_dir ) . $file_name;

	        	if ( file_exists($recipe_direction_dir . "/" . $file_name ) ):
                    unlink( $recipe_direction_dir . "/" . $file_name );
                endif;

	            if ( move_uploaded_file( $source, $destination ) ):

	                $upload_url = trailingslashit( $recipe_direction_url ) . $file_name;
	                $filetype = wp_check_filetype( basename( $destination ), null );

	                $attachment = array(
	                	'guid'           	=> $upload_url,
				        'post_mime_type'	=> $filetype['type'],
				        'post_title' 		=> sanitize_file_name( $file_name ),
				        'post_content' 		=> '',
				        'post_status' 		=> 'inherit'
				    );
				    $attach_id = wp_insert_attachment( $attachment, $destination, $id );
				    if ( defined( 'ABSPATH' ) && file_exists( ABSPATH . 'wp-admin/includes/image.php' ) ):

				    	add_filter( 'intermediate_image_sizes_advanced', array( 'Cooked_Add_Recipe','direction_image_sizes' ), 10, 1 );

				    	require_once( ABSPATH . 'wp-admin/includes/image.php' );
		    			$attach_data = wp_generate_attachment_metadata( $attach_id, $destination );
		    			$update_attachment = wp_update_attachment_metadata( $attach_id, $attach_data );
				    	return $attach_id;

				    endif;

	            endif;

	        endif;

	   	endif;

	   	return false;

	}

	public static function direction_image_sizes($sizes){
		$saved_sizes = $sizes;
		$new_sizes = array();
		$new_sizes['thumbnail'] = $saved_sizes['thumbnail'];
		return $new_sizes;
	}

}

new Cooked_Add_Recipe();
