<?php

/**
 * Cooked Add Recipe Fields
 *
 * @package     Cooked
 * @subpackage  Add Recipe Fields
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Add_Recipe_Fields {

	public static function get( $required_fields = false ){

		global $_cooked_settings;

		$difficulty_levels = Cooked_Recipes::difficulty_levels();
		$difficulty_levels[0] = '--';
		ksort($difficulty_levels);

		$select_categories = self::select_field_taxonomy( 'cp_recipe_category' );
		$select_cuisines = self::select_field_taxonomy( 'cp_recipe_cuisine' );
		$select_cooking_methods = self::select_field_taxonomy( 'cp_recipe_cooking_method' );
		$select_diets = self::select_field_taxonomy( 'cp_recipe_diet' );

		$guest_mode = ( isset( $_cooked_settings['recipe_submission_guest_mode'][0] ) && $_cooked_settings['recipe_submission_guest_mode'][0] == 'active' ? true : false );
		$guest_name = ( $guest_mode && !is_user_logged_in() ? ( isset( $_cooked_settings['recipe_submission_guest_name'] ) && $_cooked_settings['recipe_submission_guest_name'] != 'disabled' ? $_cooked_settings['recipe_submission_guest_name'] : false ) : false );
		$guest_email = ( $guest_mode && !is_user_logged_in() ? ( isset( $_cooked_settings['recipe_submission_guest_email'] ) && $_cooked_settings['recipe_submission_guest_email'] != 'disabled' ? $_cooked_settings['recipe_submission_guest_email'] : false ) : false );

		$default_fields = array(
			'post_title' => array(
				'required' => true,
				'read_only' => true,
				'setting' => 'post', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['post_title'] ) ? $_POST['_recipe_settings']['post_title'] : '' ),
				'label' => esc_html__('Recipe Title','cooked-pro'),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'text'				// Function (required)
				)
			),
			'featured_image' => array(
				'required' => false,
				'read_only' => false,
				'action' => 'cooked_fe_featured_image_upload',
				'setting' => 'meta', // "post" or "meta"
				'parent' => '_recipe_settings',
				'value' => ( isset( $_POST['_recipe_settings']['featured_image'] ) ? $_POST['_recipe_settings']['featured_image'] : '' ),
				'url' => ( isset( $_POST['_recipe_settings']['featured_image_url'] ) ? $_POST['_recipe_settings']['featured_image_url'] : '' ),
				'label' => esc_html__('Recipe Image','cooked-pro'),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'dropzone_image_single'	// Function (required)
				)
			),
			'video_url' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['video_url'] ) ? $_POST['_recipe_settings']['video_url'] : '' ),
				'label' => esc_html__( 'Video URL', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'video_url'				// Function (required)
				)
			),
			'excerpt' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['excerpt'] ) ? $_POST['_recipe_settings']['excerpt'] : '' ),
				'label' => esc_html__( 'Short Description', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'textarea'				// Function (required)
				)
			),
			'category' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'taxonomy', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['category'] ) ? $_POST['_recipe_settings']['category'] : '' ),
				'label' => esc_html__( 'Category', 'cooked-pro' ),
				'options' => $select_categories,
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'select'					// Function (required)
				)
			),
			'cuisine' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'taxonomy', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['cuisine'] ) ? $_POST['_recipe_settings']['cuisine'] : '' ),
				'label' => esc_html__( 'Cuisine', 'cooked-pro' ),
				'options' => $select_cuisines,
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'select'					// Function (required)
				)
			),
			'cooking_method' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'taxonomy', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['cooking_method'] ) ? $_POST['_recipe_settings']['cooking_method'] : '' ),
				'label' => esc_html__( 'Cooking Method', 'cooked-pro' ),
				'options' => $select_cooking_methods,
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'select'					// Function (required)
				)
			),
			'diet' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'taxonomy', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['diet'] ) ? $_POST['_recipe_settings']['diet'] : '' ),
				'label' => esc_html__( 'Restricted Diet', 'cooked-pro' ),
				'options' => $select_diets,
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'select'					// Function (required)
				)
			),
			'difficulty_level' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['difficulty_level'] ) ? $_POST['_recipe_settings']['difficulty_level'] : '' ),
				'label' => esc_html__( 'Difficulty', 'cooked-pro' ),
				'options' => $difficulty_levels,
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'select'					// Function (required)
				)
			),
			'prep_time' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['prep_time'] ) ? $_POST['_recipe_settings']['prep_time'] : '' ),
				'label' => esc_html__( 'Prep Time', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'time_picker'				// Function (required)
				)
			),
			'cook_time' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['cook_time'] ) ? $_POST['_recipe_settings']['cook_time'] : '' ),
				'label' => esc_html__( 'Cook Time', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'time_picker'				// Function (required)
				)
			),
			'total_time' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['total_time'] ) ? $_POST['_recipe_settings']['total_time'] : '' ),
				'label' => esc_html__( 'Total Time', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'time_picker'				// Function (required)
				)
			),
			'ingredients' => array(
				'required' => true,
				'read_only' => true,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['ingredients'] ) ? $_POST['_recipe_settings']['ingredients'] : '' ),
				'label' => esc_html__( 'Ingredients', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'ingredients'				// Function (required)
				)
			),
			'directions' => array(
				'required' => true,
				'read_only' => true,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['directions'] ) ? $_POST['_recipe_settings']['directions'] : '' ),
				'label' => esc_html__( 'Directions', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'directions'				// Function (required)
				)
			),
			'nutrition' => array(
				'required' => false,
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['nutrition'] ) ? $_POST['_recipe_settings']['nutrition'] : '' ),
				'label' => esc_html__( 'Nutrition', 'cooked-pro' ),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'nutrition'				// Function (required)
				)
			)
		);

		if ( !$required_fields ):
			$default_fields['nutrition']['conditional_value'] = 'nutrition_fields_active';
		endif;

		$add_recipe_fields = array();

		if ( $guest_name ):
			$add_recipe_fields['guest_name'] = array(
				'required' => ( $guest_name == 'required' ? true : false ),
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['guest_name'] ) ? $_POST['_recipe_settings']['guest_name'] : '' ),
				'label' => esc_html__('Your Name','cooked-pro'),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'text'					// Function (required)
				)
			);
		endif;

		if ( $guest_email ):
			$add_recipe_fields['guest_email'] = array(
				'required' => ( $guest_email == 'required' ? true : false ),
				'read_only' => false,
				'setting' => 'meta', // "post" or "meta"
				'value' => ( isset( $_POST['_recipe_settings']['guest_email'] ) ? $_POST['_recipe_settings']['guest_email'] : '' ),
				'label' => esc_html__('Your Email','cooked-pro'),
				'callback' => array(
					'class' => 'Cooked_Add_Recipe_Fields', 	// Class (optional)
					'function' => 'text'					// Function (required)
				)
			);
		endif;

		foreach( $default_fields as $key => $field ):
			$add_recipe_fields[$key] = $field;
		endforeach;

		return apply_filters( 'cooked_add_recipe_fields_defaults', $add_recipe_fields, $_POST );

	}

	public static function select_field_taxonomy( $taxonomy ){

		$terms = get_terms( array(
		    'taxonomy' => $taxonomy,
		    'hide_empty' => false,
		) );

		if ( !is_wp_error( $terms ) ):
			$select_terms[0] = '--';
			foreach( $terms as $term ):
				$select_terms[$term->slug] = $term->name;
			endforeach;
		else:
			$select_terms = array();
		endif;

		return $select_terms;

	}

	public static function text( $field_id, $field ){
		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' class="cooked-required"';
		echo '<input' . $required . ' id="' . esc_attr( $field_id ) . '" type="text" name="_recipe_settings[' . $field_id . ']" value="' . stripslashes( esc_attr( $field['value'] ) ) . '">';
	}

	public static function video_url( $field_id, $field ){
		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' class="cooked-required"';
		echo '<input' . $required . ' id="' . esc_attr( $field_id ) . '" type="text" name="_recipe_settings[' . $field_id . ']" value="' . stripslashes( esc_attr( $field['value'] ) ) . '" placeholder="ex. https://www.youtube.com/watch?v=abc123">';
	}

	public static function textarea( $field_id, $field ){
		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' class="cooked-required"';
		echo '<textarea' . $required . ' id="' . esc_attr( $field_id ) . '" name="_recipe_settings[' . $field_id . ']">' . esc_textarea( $field['value'] ) . '</textarea>';
	}

	public static function select( $field_id, $field ){
		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' class="cooked-required"';
		echo '<select' . $required . ' id="' . esc_attr( $field_id ) . '" name="_recipe_settings[' . $field_id . ']">';
			foreach( $field['options'] as $val => $opt ):
				echo '<option value="' . $val . '"' . ( $field['value'] == $val ? ' selected' : '' ) . '>' . $opt . '</option>';
			endforeach;
		echo '</select>';
	}

	public static function time_picker( $field_id, $field ){
		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' cooked-required';
		echo '<input' . $required . ' id="' . esc_attr( $field_id ) . '" type="text" pattern="[0-9]*" class="cooked-time-picker" name="_recipe_settings[' . $field_id . ']" value="' . stripslashes( esc_attr( $field['value'] ) ) . '">';
		echo '<span class="cooked-time-picker-text">' . esc_html__( 'minutes', 'cooked-pro' ) . '</span>';
	}

	public static function dropzone_image_single( $field_id, $field ){

		// Do we have an attachment id?
		if ( is_numeric($field['value']) ):
			$attachment_id = esc_attr( $field['value'] );
			$field['url'] = wp_get_attachment_url( $attachment_id );
		endif;

		$required = ( isset($field['required']) && $field['required'] ? ' required' : '' );
		$required_class = ' cooked-required';
		echo do_shortcode( '[cooked-dz-uploader id="' . esc_attr( $field_id ) . '" parent="' . esc_attr( $field['parent'] ) . '" value="' . esc_attr( $field['value'] ) . '" url="' . esc_attr( $field['url'] ) . '" action="' . esc_attr( $field['action'] ) . '" message="' . esc_html__( 'Upload an image for this recipe', 'cooked-pro' ) . '"]' );

	}

	public static function ingredients( $field_id, $field ){
		Cooked_Add_Recipe::ingredients_builder( $field );
	}

	public static function directions( $field_id, $field ){
		Cooked_Add_Recipe::directions_builder( $field );
	}

	public static function nutrition( $field_id, $field ){
		Cooked_Add_Recipe::nutrition_facts( $field );
	}

}
