<?php

add_action( 'wp_ajax_cooked_pending_action_delete', 'cooked_pending_action_delete' );
function cooked_pending_action_delete(){
    if ( current_user_can( 'edit_cooked_recipes' ) ):
        $recipe_id = esc_html( $_POST['recipe_id'] );
        wp_delete_post( $recipe_id, true );
        echo $recipe_id;
    endif;
    wp_die();
}

add_action( 'wp_ajax_cooked_pending_action_approve', 'cooked_pending_action_approve' );
function cooked_pending_action_approve(){
    if ( current_user_can( 'edit_cooked_recipes' ) ):
        $recipe_id = esc_html( $_POST['recipe_id'] );
        delete_post_meta( $recipe_id, '_recipe_pending' );
        echo $recipe_id;
    endif;
    wp_die();
}

add_action( 'rest_api_init', 'register_cooked_pro_endpoints' );
function register_cooked_pro_endpoints(){
    register_rest_route( 'cooked/v2', '/pending-recipes/', array(
        array(
            'methods'             => WP_REST_Server::READABLE,
            'callback'            => 'cooked_get_pending_recipes',
            'permission_callback' => 'cooked_rest_permissions',
        )
    ));
}

function cooked_rest_permissions(){
    return current_user_can( 'edit_cooked_recipes' );
}

function cooked_get_pending_recipes(){
    $args = array(
        'post_type' => 'cp_recipe',
        'posts_per_page' => -1,
        'meta_query' => array(
            array(
                'pending_zero' => array(
                    'key' => '_recipe_pending',
                    'compare' => '==',
                    'value' => 1
                )
            )
        )
    );
    $pending_recipes = new WP_Query( $args );
    $recipes = array();
    if ( $pending_recipes->have_posts() ) :
        while ( $pending_recipes->have_posts() ) :
            $pending_recipes->the_post();
            $author_link = '<a href="' . get_edit_user_link( $pending_recipes->post->post_author ) . '">' . get_the_author_meta( 'display_name', $pending_recipes->post->post_author ) . '</a>';
            $recipe_settings = get_post_meta( $pending_recipes->post->ID, '_recipe_settings', true );
            $recipes['post-' . $pending_recipes->post->ID]['id'] = $pending_recipes->post->ID;
            $recipes['post-' . $pending_recipes->post->ID]['meta'] = sprintf( esc_html__( 'Originally submitted on %s by %s', 'cooked-pro' ), date_i18n( get_option('date_format'), strtotime( $pending_recipes->post->post_date ) ), $author_link );
            $recipes['post-' . $pending_recipes->post->ID]['title'] = get_the_title();
            $recipes['post-' . $pending_recipes->post->ID]['url'] = get_permalink( $pending_recipes->post->ID );
            $recipes['post-' . $pending_recipes->post->ID]['edit_url'] = get_edit_post_link( $pending_recipes->post->ID, 'edit' );
            $recipes['post-' . $pending_recipes->post->ID]['image'] = get_the_post_thumbnail_url( $post, 'cooked-square' );
            $recipes['post-' . $pending_recipes->post->ID]['content'] = do_shortcode( $recipe_settings['content'] );
        endwhile;
    endif;
    return $recipes;
}