<?php
/**
 * Cooked Content Defaults
 *
 * @package     Cooked
 * @subpackage  Content Defaults
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Content {

	function __construct() {

		$_cooked_settings = Cooked_Settings::get();

		// Filters
		add_filter( 'cooked_default_content', array(&$this, 'default_content'),10, 1 );

		// New AMP Setting
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'disable_amp_setting'), 10, 1 );

		// Google AMP Support
		if ( isset( $_cooked_settings['advanced'] ) && is_array( $_cooked_settings['advanced'] ) && !in_array( 'disable_amp', $_cooked_settings['advanced'] ) || isset( $_cooked_settings['advanced'] ) && !is_array( $_cooked_settings['advanced'] ) || !isset( $_cooked_settings['advanced'] ) ):
			add_filter( 'cooked_directions_shortcode_atts', array(&$this, 'directions_amp_support'), 10, 1 );
			add_filter( 'cooked_direction_image_html', array(&$this, 'direction_image_html'), 10, 2 );
			add_action( 'template_redirect', array(&$this, 'amp_recipe_template'), 10, 1 );
			add_action( 'wp_head', array( &$this, 'amp_tags' ), 5 );
		endif;

	}

	public static function disable_amp_setting( $settings ){
		$settings['recipe_settings']['fields']['advanced']['options']['disable_amp'] = '<strong>' . esc_html__('Disable AMP Support','cooked-pro') . '</strong> &mdash; ' . sprintf( esc_html__('Disable %s enabled recipes.','cooked-pro'), '<a href="https://www.ampproject.org/learn/overview/" target="_blank">Google AMP</a>' );
		return $settings;
	}

	public static function default_content(){
		return '<p>[cooked-info left="author,print,fullscreen" right="rating,difficulty,favorite"]</p><p>[cooked-excerpt]</p><p>[cooked-sharing]</p><p>[cooked-image]</p><p>[cooked-info left="servings" right="prep_time,cook_time,total_time"]</p><p>[cooked-ingredients]</p><p>[cooked-directions]</p><p>[cooked-gallery]</p><p>[cooked-info include="taxonomies"]</p>';
	}

	public static function amp_recipe_template(){

		if ( is_singular('cp_recipe') && isset($_GET['amp']) ):

			load_template( COOKED_PRO_DIR . 'templates/front/recipe-amp.php',false);
			exit;

		endif;

	}

	public static function amp_tags(){

		global $post;

		if ( isset( $post->post_type ) && $post->post_type == 'cp_recipe' ):

			?><link rel="amphtml" href="<?php echo get_permalink( $post->ID ); ?>?amp"><?php

		endif;

	}

	public static function directions_amp_support( $atts ){
		$atts['amp'] = false;
		return $atts;
	}

	public static function direction_image_html( $image, $atts ){
		if ( isset( $atts['amp'] ) ):
			$image = str_replace( '<img', '<amp-img', $image );
		endif;
		return $image;
	}

}

new Cooked_Content();
