<?php
/**
 * Dropzone Front-End Image Uploader
 *
 * @package     Cooked
 * @subpackage  Front-End Image Uploader
 * @since       1.1.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

define( 'COOKED_DZ_VERSION', '5.2.0' );

class Cooked_DZ {

    function __construct() {

        // Shortcode
        add_shortcode( 'cooked-dz-uploader', array( &$this, 'shortcode' ) );

        // Actions
        add_action( 'wp_enqueue_scripts', array( &$this, 'enqueue_scripts' ) );

        // Uploaders
        add_action( 'wp_ajax_cooked_fe_featured_image_upload', array(&$this,'dz_upload') );
        add_action( 'wp_ajax_nopriv_cooked_fe_featured_image_upload', array(&$this,'dz_upload') );

        add_action( 'wp_ajax_cooked_profile_photo_upload', array(&$this,'dz_upload') );
        add_action( 'wp_ajax_nopriv_cooked_profile_photo_upload', array(&$this,'dz_upload') );

    }

    public static function dz_upload() {

        if ( !empty( $_FILES ) && wp_verify_nonce( $_REQUEST['nonce'], 'cooked_nonce_field' ) ):

            $_uploads = wp_upload_dir();
            $cooked_tmp_dir = trailingslashit( $_uploads['basedir'] ) . 'cooked/tmp';
            $cooked_tmp_url = str_replace( array( 'http://', 'https://' ), '//', trailingslashit( $_uploads['baseurl'] ) . 'cooked/tmp' );

            if ( wp_mkdir_p( $cooked_tmp_dir ) ):

                $source = $_FILES['file']['tmp_name'];
                $salt = md5( $_FILES['file']['name'] . time() );
                $file_name = $salt . '-' . $_FILES['file']['name'];
                $destination = trailingslashit( $cooked_tmp_dir ) . $file_name;

                if ( move_uploaded_file( $source, $destination ) ):

                    $upload_url = trailingslashit( $cooked_tmp_url ) . $file_name;
                    $uploaded_filetype = wp_check_filetype( basename( $destination ), null );

                    $file_array = array( 'file' => $destination, 'url' => $upload_url, 'type' => $uploaded_filetype );
                    echo json_encode( $file_array );

                endif;

            endif;

        endif;
        wp_die();

    }

    public function enqueue_scripts() {
        $min = ( COOKED_DEV ? '' : '.min' );
        wp_register_script( 'cooked-dropzone', COOKED_PRO_URL . 'modules/dropzone/dropzone' . $min . '.js', array('jquery'), COOKED_DZ_VERSION, true );
        wp_enqueue_style( 'cooked-dropzone', COOKED_PRO_URL . 'modules/dropzone/dropzone' . $min . '.css', array(), COOKED_DZ_VERSION );
        wp_add_inline_script( 'cooked-dropzone', 'Dropzone.autoDiscover = false;' );
        wp_enqueue_script( 'cooked-dropzone' );
    }

    public function shortcode( $atts ) {

        $default_max_size = ( wp_max_upload_size() > 15 ? 15 : wp_max_upload_size() );

        // Shortcode Attributes
        $atts = shortcode_atts(
            array(
                'id' => false,
                'action' => false,
                'single' => true,
                'value' => false,
                'url' => false,
                'parent' => false,
                'max_size' => $default_max_size,
                'message' => esc_html__( 'Click to upload', 'cooked-pro' )
            ), $atts
        );

        if ( !$atts['id'] || !$atts['action'] )
            return false;

        $ajax_url = admin_url( 'admin-ajax.php' );
        $id = esc_attr( $atts['id'] );
        $parent = esc_attr( $atts['parent'] );
        $input_name = ( $parent ? $parent . '[' . $id . ']' : $id );
        $input_url_name = ( $parent ? $parent . '[' . $id . '_url]' : $id . '_url' );
        $value = esc_attr( $atts['value'] );
        $url = esc_attr( $atts['url'] );
        $action = esc_attr( $atts['action'] );
        $multiple = ( $atts['single'] ? 'false' : 'true' );
        $max_size = ( is_numeric( $atts['max_size'] ) ? $atts['max_size'] : $default_max_size );
        $nonce = wp_create_nonce( 'cooked_nonce_field' );

        $html = '<div id="' . $id . '" class="dropzone needsclick dz-clickable' . ( $value && $url ? ' dz-hidden' : '' ) . '">';
            $html .= '<div class="dz-message needsclick">';
                $html .= '<label>' . esc_html( $atts['message'] ) . '</label>';
                $html .= '<span class="dz-note needsclick">';
                    $html .= sprintf( esc_html__( 'Up to %s MB', 'cooked-pro' ), $max_size );
                $html .= '</span>';
            $html .= '</div>';
            $html .= '<input type="hidden" name="' . $input_name . '" value="' . $value . '">';
            $html .= '<input type="hidden" name="' . $input_url_name . '" value="' . $url . '">';
        $html .= '</div>';

        $html .= ( $value && $url ? '<div class="dz-fake-image"><img src="' . $url . '"><div class="dz-remove-fake-image"><i class="cooked-icon cooked-icon-times"></i></div></div>' : '' );

        ob_start();

        // Full Source
        ?><script type="text/javascript">

            (function( $ ) {

                $(document).ready(function() {

                    var thisDZContainer_<?php echo esc_js( $id ); ?> = $("#<?php echo esc_js( $id ); ?>");

                    var DZOBJ_<?php echo esc_js( $id ); ?> = new Dropzone("#<?php echo esc_js( $id ); ?>", {
                        acceptedFiles: "image/jpeg, image/gif, image/png",
                        maxFiles: 1,
                        url: "<?php echo esc_js( $ajax_url ); ?>",
                        uploadMultiple: <?php echo esc_js( $multiple ); ?>,
                        resizeWidth: 1500,
                        resizeMimeType: 'image/jpeg',
                        resizeMethod: 'crop',
                        resizeQuality: 65,
                        createImageThumbnails: false,
                        maxFilesize: <?php echo esc_js( $max_size ); ?>,
                        previewTemplate: "<div class='dz-preview dz-file-preview'><div class='dz-image'><img data-dz-thumbnail /></div><div class='dz-details'><div class='dz-filename'><span data-dz-name></span></div><div class='dz-size' data-dz-size></div></div><div class='dz-progress'><span class='dz-upload' data-dz-uploadprogress></span></div><div class='dz-success-mark'><i class='cooked-icon cooked-icon-check-bold'></i></div><div class='dz-remove-image'><i class='cooked-icon cooked-icon-times'></i></div></div>",
                    });

                    DZOBJ_<?php echo esc_js( $id ); ?>.on("sending", function( file, xhr, formData ) {
                        thisDZContainer_<?php echo esc_js( $id ); ?>.parents('.cooked-input-side').addClass('dz-replaced');
                        var customErrorMessage = thisDZContainer_<?php echo esc_js( $id ); ?>.parent().find( ".dz-custom-error-message");
                        if ( customErrorMessage.length ){
                            customErrorMessage.removeClass( "active" );
                        }
                        formData.append( "action", "<?php echo esc_js( $action ); ?>" );
                        formData.append( "nonce", "<?php echo esc_js( $nonce ); ?>" );
                    });

                    DZOBJ_<?php echo esc_js( $id ); ?>.on("success", function( file, response ) {
                        if ( response !== "error" ){
                            var file_data = JSON.parse( response );
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find(".dz-image").html( "<img src='" + file_data.url + "'>" );
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_name ); ?>']").val( file_data.file );
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_url_name ); ?>']").val( file_data.url );
                        }
                    });

                    DZOBJ_<?php echo esc_js( $id ); ?>.on("addedfile", function(file) {
                        thisDZContainer_<?php echo esc_js( $id ); ?>.find(".dz-remove-image").on("click", function(e){
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_name ); ?>']").val('');
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_url_name ); ?>']").val('');
                            thisDZContainer_<?php echo esc_js( $id ); ?>.parents('.cooked-input-side').removeClass('dz-replaced');
                            DZOBJ_<?php echo esc_js( $id ); ?>.removeFile(file);
                        });
                    });

                    DZOBJ_<?php echo esc_js( $id ); ?>.on("error", function(file, errorMessage, xhr) {
                        thisDZContainer_<?php echo esc_js( $id ); ?>.parents('.cooked-input-side').removeClass('dz-replaced');
                        DZOBJ_<?php echo esc_js( $id ); ?>.removeFile(file);
                        thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_name ); ?>']").val('');
                        thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_url_name ); ?>']").val('');
                        var customErrorMessage = thisDZContainer_<?php echo esc_js( $id ); ?>.parent().find( ".dz-custom-error-message");
                        if ( customErrorMessage.length ){
                            customErrorMessage.html( errorMessage );
                            customErrorMessage.addClass("active");
                        } else {
                            thisDZContainer_<?php echo esc_js( $id ); ?>.before("<div class='dz-custom-error-message active'>" + errorMessage + "</div>");
                        }
                    });

                    <?php if ( $value && $url ): ?>

                        $( 'body' ).on( 'click', '.dz-remove-fake-image', function(){
                            thisDZContainer_<?php echo esc_js( $id ); ?>.removeClass('dz-hidden');
                            thisDZContainer_<?php echo esc_js( $id ); ?>.parents('.cooked-input-side').removeClass('dz-replaced');
                            DZOBJ_<?php echo esc_js( $id ); ?>.removeAllFiles();
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_name ); ?>']").val('');
                            thisDZContainer_<?php echo esc_js( $id ); ?>.find("input[name='<?php echo esc_js( $input_url_name ); ?>']").val('');
                            $(this).parents('.dz-fake-image').remove();
                        });

                    <?php endif; ?>

                });

            })( jQuery );

        </script><?php

        $script = ob_get_clean();

        return $html . $script;

    }

}

new Cooked_DZ();
