<?php
/**
 * Cooked Content Defaults
 *
 * @package     Cooked
 * @subpackage  Content Defaults
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Ingredient_Enhancements {

	function __construct() {

		// Actions
		add_action( 'cooked_after_ingredient_name_field', array(&$this, 'ingredient_link_field'),10, 2 );
		add_action( 'cooked_after_ingredient_name_field', array(&$this, 'ingredient_description_field'),10, 2 );
		add_action( 'cooked_ingredient_after_name', array(&$this, 'show_ingredient_description'),10, 1 );
		//add_action( 'cooked_ingredient_buttons_end', array(&$this, 'bulk_ingredients_button'), 10, 2 );
		add_action( 'admin_enqueue_scripts', array(&$this, 'admin_enqueues'), 10, 1 );

		// Filters
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'ingredient_settings'), 11, 3 );
		add_filter( 'cooked_ingredient_field_classes', array(&$this, 'ingredient_field_classes'), 10, 2 );
		add_filter( 'cooked_ingredient_name', array(&$this, 'ingredient_name'), 10, 2 );

	}

	public static function cooked_info_nutrition( $recipe ){
		self::nutrition_donut( $recipe );
	}

	public static function nutrition_breakdown( $recipe ){

		$fat = (int)esc_html( $recipe['nutrition']['fat'] );
		$carbs = (int)esc_html( $recipe['nutrition']['carbs'] );
		$protein = (int)esc_html( $recipe['nutrition']['protein'] );

		if ( $fat || $protein || $carbs ):
			echo '<span class="cooked-recipe-fitness-report">';
				echo '<span class="cooked-fitness-report-fat"><span><span>' . esc_html__( 'Fat', 'cooked-pro' ) . '</span><span>' . $fat . 'g' . '</span></span></span>';
				echo '<span class="cooked-fitness-report-carbs"><span><span>' . esc_html__( 'Carbs', 'cooked-pro' ) . '</span><span>' . $carbs . 'g' . '</span></span></span>';
				echo '<span class="cooked-fitness-report-protein"><span><span>' . esc_html__( 'Protein', 'cooked-pro' ) . '</span><span>' . $protein . 'g' . '</span></span></span>';
			echo '</span>';
		endif;

	}

	public static function nutrition_donut( $recipe ){

		$fat = (int)esc_html( $recipe['nutrition']['fat'] );
		$carbs = (int)esc_html( $recipe['nutrition']['carbs'] );
		$protein = (int)esc_html( $recipe['nutrition']['protein'] );

		if ( $fat || $protein || $carbs ):

			$total = $fat + $protein + $carbs;
			$fat_ratio = round( ( 345 * $fat ) / $total );
			$carbs_ratio = round( ( 345 * $carbs ) / $total );
			$protein_ratio = round( ( 345 * $protein ) / $total );

			$ratio_total = $fat_ratio + $carbs_ratio + $protein_ratio;
			$overage = $ratio_total - 345;
			$ratios = array();

			// Fix any overages (very minor)
			if ( $fat_ratio ):
				$fat_ratio = $fat_ratio - $overage;
			elseif ( $carbs_ratio ):
				$carbs_ratio = $carbs_ratio - $overage;
			elseif ( $protein_ratio ):
				$protein_ratio = $protein_ratio - $overage;
			endif;

			$ratios['fat'] = $fat_ratio;
			$ratios['carbs'] = $carbs_ratio;
			$ratios['protein'] = $protein_ratio;
			$rand = rand( 100000,999999 );

			?><div class="cooked-donut" id="cooked-donut-<?php echo $rand; ?>"></div>
			<script type="text/javascript">
				new Chartist.Pie('#cooked-donut-<?php echo $rand; ?>', {
				 	series: [ <?php echo $fat; ?>, <?php echo $carbs; ?>, <?php echo $protein; ?> ]
				}, {
				  donut: true,
				  donutWidth: 10,
				  donutSolid: true,
				  startAngle: 0,
				  showLabel: false
				});
			</script><?php

    	endif;

	}

	public function admin_enqueues(){

		$min = ( COOKED_DEV ? '' : '.min' );
		wp_enqueue_style( 'cooked-ingredient-enhancements-css', COOKED_PRO_URL . 'modules/ingredients/ingredient-enhancements' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_enqueue_script('cooked-ingredient-enhancements-js', COOKED_PRO_URL . 'modules/ingredients/ingredient-enhancements' . $min . '.js', array('jquery'), COOKED_PRO_VERSION );

	}

	public static function cooked_info_ingredients( $recipe ){

		$total_ingredients = 0;
		if ( isset($recipe['ingredients']) && !empty($recipe['ingredients']) ):
			foreach( $recipe['ingredients'] as $ingredient ):
				if ( !isset($recipe['section_heading_name']) ):
					$total_ingredients++;
				endif;
			endforeach;
		endif;
		echo ( $total_ingredients ? '<span class="cooked-ingredients"><span class="cooked-ingredients-icon"><i class="cooked-icon cooked-icon-shopping-cart"></i></span>' . esc_html( sprintf( _n( '%s Ingredient', '%s Ingredients', $total_ingredients, 'cooked-pro' ), $total_ingredients ) ) . '</span>' : '' );

	}

	public function parse_ingredient_string( $string = false ){

		$_ingredient = array();

		if ( preg_match( "/[0-9][\/][0-9]|[0-9]/", $string ) ):

			preg_match( '/[0-9][\/][0-9]|[0-9]/', $string, $matches );

			$_ingredient['amount'] = trim( $matches[0] );

			$units = Cooked_Measurements::get();

			//Check To See If We Can Find a Matching Unit
			foreach( $units as $key => $unit ):
				foreach( $unit['variations'] as $varation ):
					if ( preg_match( "/\s+" . $varation . "\s+/", $string ) ):
						preg_match( "/\s+" . $varation . "\s+/", $string, $matches );
						$_ingredient['measurement'] = trim( $matches[0] );
						break;
					endif;
				endforeach;
			endforeach;

			$fixed_string = $string;
			if ( isset( $_ingredient['measurement'] ) ):
				$fixed_string = str_replace( $_ingredient['measurement'], "", $fixed_string );
			endif;

			$fixed_string = str_replace( $_ingredient['amount'], "", $fixed_string );
			$_ingredient['item'] = trim( $fixed_string );

		else:

			$array_string = explode( ",", $string );
			$_ingredient['item'] = trim( $string );

		endif;

		return $_ingredient;

	}

	public function ingredient_name( $string = false, $ing = false ){

		global $_cooked_settings;

		$url = ( isset($ing['url']) && $ing['url'] ? esc_html( $ing['url'] ) : false );
		$target = ( isset( $_cooked_settings['ingredient_options'] ) && in_array( 'ingredient_target_blank', $_cooked_settings['ingredient_options'] ) ? ' target="_blank"' : '' );
		if ( $url ):
			return '<a' . $target . ' href="' . esc_url( $url ) . '">' . $string . '</a>';
		endif;

		return $string;

	}

	public function ingredient_field_classes( $classes, $value = false ){

		return 'recipe-setting-block cooked-ingredient-block cooked-clearfix' . ( isset($value['url']) && $value['url'] ? ' cooked-has-url' : '' );

	}

	public static function ingredient_settings( $settings, $pages, $categories ){

		Cooked_Functions::array_splice_assoc( $settings['design']['fields'], 'main_color_hover', 0, array(
			'fat_color' => array(
				'title' => esc_html__('Fat Color (fitness layout)', 'cooked-pro'),
				'desc' => esc_html__('Set the "Fat" color for the fitness list style.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#228edd',
				'options' => '#228edd'
			),
			'carbs_color' => array(
				'title' => esc_html__('Carbs Color (fitness layout)', 'cooked-pro'),
				'desc' => esc_html__('Set the "Carbs" color for the fitness list style.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#33bf99',
				'options' => '#33bf99'
			),
			'protein_color' => array(
				'title' => esc_html__('Protein Color (fitness layout)', 'cooked-pro'),
				'desc' => esc_html__('Set the "Protein" color for the fitness list style.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#fb5d3b',
				'options' => '#fb5d3b'
			)
		));

		Cooked_Functions::array_splice_assoc( $settings['recipe_settings']['fields'], 'browse_default_sort', 0, array(
			'ingredient_options' => array(
				'title' => esc_html__('Ingredient Options', 'cooked-pro'),
				'desc' => '',
				'type' => 'checkboxes',
				'default' => array(),
				'options' => array(
					'ingredient_target_blank' => array(
						'label' => esc_html__( 'Ingredient links open in a new window/tab.', 'cooked-pro' ),
					)
				)
			)
		));

		return $settings;

	}

	public function bulk_ingredients_button(){
		//echo '&nbsp;<a href="#" class="button button-primary cooked-add-bulk-button">' . esc_html__('Add Bulk Ingredients','cooked-pro') . '</a>';
	}

	public function ingredient_link_field( $ing_key = false, $value = false ){

		global $_cooked_settings;

		if ( !is_admin() && !isset( $_cooked_settings['recipe_submission_options'] ) || !is_admin() && isset( $_cooked_settings['recipe_submission_options'] ) && !in_array( 'ingredient_links', $_cooked_settings['recipe_submission_options'] ) )
			return false;

		echo '<div class="cooked-ingredient-url">';
			echo '<input type="text" data-ingredient-part="url" name="' . ( $ing_key ? '_recipe_settings[ingredients][' . $ing_key . '][url]' : '' ) . '" value="' . ( isset($value['url']) ? $value['url'] : '' ) . '" placeholder="http://">';
		echo '</div>';
		echo '<span href="#" class="cooked-show-url"><i class="cooked-icon cooked-icon-link"></i></span>';

	}

	public function ingredient_description_field( $ing_key = false, $value = false ){

		global $_cooked_settings;

		$desc_val = ( isset($value['description']) && $value['description'] ? $value['description'] : false );

		if ( !is_admin() && !isset( $_cooked_settings['recipe_submission_options'] ) && !$desc_val || !is_admin() && isset( $_cooked_settings['recipe_submission_options'] ) && !in_array( 'ingredient_descriptions', $_cooked_settings['recipe_submission_options'] ) && !$desc_val ):
			echo '<input type="hidden" data-ingredient-part="description" name="' . ( $ing_key ? '_recipe_settings[ingredients][' . $ing_key . '][description]' : '' ) . '" value="' . ( isset($value['description']) ? $value['description'] : '' ) . '" placeholder="' . esc_attr__( 'Optional Description...', 'cooked-pro' ) . '">';
		else:
			echo '<div class="cooked-ingredient-description">';
				echo '<input type="text" data-ingredient-part="description" name="' . ( $ing_key ? '_recipe_settings[ingredients][' . $ing_key . '][description]' : '' ) . '" value="' . ( isset($value['description']) ? $value['description'] : '' ) . '" placeholder="' . esc_attr__( 'Optional Description...', 'cooked-pro' ) . '">';
			echo '</div>';
		endif;

	}

	public function show_ingredient_description( $ing ){
		if ( isset($ing['description']) && $ing['description'] ):
			echo '<span class="cooked-ing-description">' . esc_html( $ing['description'] ) . '</span>';
		endif;
	}

}

new Cooked_Ingredient_Enhancements();
