<?php
/**
 * Cooked User Interactions
 *
 * @package     Cooked
 * @subpackage  User Interactions
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Interactions {

	function __construct(){

		// Actions
		add_action( 'cooked_recipe_grid_after_image', array(&$this,'recipe_list_add_heart'), 10, 1 );
		add_action( 'cooked_recipe_grid_before_name', array(&$this,'recipe_list_add_rating'), 10, 1 );
		add_action( 'cooked_recipe_modern_after_name', array(&$this,'recipe_list_add_rating'), 10, 1 );
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );

		// Filters
		add_filter( 'cooked_available_info_shortcode_methods', array(&$this,'available_info_shortcode_methods'), 10, 1 );
		add_filter( 'cooked_browse_sorting_types', array(&$this,'browse_sorting_types'), 10, 1 );
		add_filter( 'cooked_recipe_query_args', array(&$this,'recipe_query_args'), 10, 3 );
		add_filter( 'cooked_recipe_info_display_options', array(&$this,'recipe_info_display_options'), 10, 1 );
		add_filter( 'cooked_recipe_info_display_options_defaults', array(&$this,'recipe_info_display_options_defaults'), 10, 1 );
		add_filter( 'cooked_settings_sort_options', array(&$this,'settings_sort_options'), 10, 1 );
		add_filter( 'cooked_schema_array', array(&$this,'schema_array'), 10, 3 );
		add_filter( 'cooked_available_info_vars', array(&$this,'available_info_vars'), 10, 1 );
		add_filter( 'cooked_single_recipe_settings', array(&$this,'single_recipe_settings'), 10, 2 );
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'engagement_settings'),10, 3 );
		add_filter( 'cooked_recipe_list_info_shortcode', array(&$this, 'recipe_list_info_content'), 10, 2 );
		add_filter( 'cooked_sync_c2_recipe_settings', array(&$this, 'sync_c2_recipe_settings'), 10, 2 );
		add_filter( 'cooked_profile_nav', array(&$this, 'profile_nav_filter'), 10, 1 );
		add_filter( 'get_comment_text', array(&$this, 'add_rating_to_comment'), 10, 2 );

		/**
		 * Front-End Ajax
		 */

		// Favorite a Recipe
		add_action( 'wp_ajax_cooked_favorite_recipe', array(&$this,'favorite_recipe') );

		// Rate a Recipe
		add_action( 'wp_ajax_cooked_rate_recipe', array(&$this,'rate_recipe') );

	}

	public static function sync_c2_recipe_settings( $recipe_settings, $recipe_id ){

		if ( $recipe_id ):
			$_recipe_settings = Cooked_Recipes::get_settings( $recipe_id, false );
			$recipe_settings['thumbs_up'] = $_recipe_settings['thumbs_up'];
			$recipe_settings['thumbs_down'] = $_recipe_settings['thumbs_down'];
			$recipe_settings['votes'] = $_recipe_settings['votes'];
			$recipe_settings['favorites'] = $_recipe_settings['favorites'];
			$recipe_settings['rating'] = $_recipe_settings['rating'];
		endif;

		return $recipe_settings;

	}

	public static function add_rating_to_comment( $comment_text, $comment ) {

		global $_cooked_settings;

		if ( isset($comment->user_id) && isset($comment->comment_post_ID) && $_post_id = $comment->comment_post_ID ):

			$_post = get_post( $_post_id );
			if ( $_post->post_type == 'cp_recipe' ):
				$recipe = Cooked_Recipes::get( $_post_id, true );
				if ( isset($recipe['votes'][$comment->user_id]) && $recipe['votes'][$comment->user_id] ):
					if ( $_cooked_settings['rating_type'] == 'stars' ):
						$comment_text = '<div class="cooked-comment-rating">' . self::rating( 0, 0, array(), $recipe['votes'][$comment->user_id] ) . '</div>' . $comment_text;
					elseif ( $_cooked_settings['rating_type'] == 'thumbs' ):
						$comment_text = '<div class="cooked-comment-rating"><span class="cooked-rating-thumb cooked-is-selected"><i class="cooked-icon cooked-icon-thumbs-' . ( $recipe['votes'][$comment->user_id] >= 3 ? 'up' : 'down' ) . '-solid"></i></div>' . $comment_text;
					endif;
				endif;
			endif;

		endif;

		return $comment_text;

	}

	public function profile_nav_filter( $profile_nav ){
		global $atts,$_cooked_settings,$_cooked_user,$profile_user;
		if ( isset( $profile_user['favorites'] ) && isset( $profile_user['favorites'] ) && !empty( $profile_user['favorites'] ) ):
			$profile_nav['favorites'] = array(
				'name' => esc_html__('My Favorites','cooked-pro'),
				'default' => ( get_query_var('cooked_pn','recipes') == 'favorites' ? true : false ),
				'link_hook' => array( 'Cooked_Interactions','favorites')
			);
		endif;
		return $profile_nav;
	}

	public function recipe_list_add_heart( $recipe ){
		echo self::favorite( $recipe['id'] );
	}

	public function recipe_list_add_rating( $recipe ){
		global $_cooked_settings, $_cooked_user;
		if ( $_cooked_settings['rating_type'] == 'stars' ):
			echo '<span class="cooked-rating" data-recipe-id="' . $recipe['id'] . '">' . self::rating( $recipe['rating'], null, $recipe, $recipe['rating'] ) . '</span>';
		elseif ( $_cooked_settings['rating_type'] == 'thumbs' ):
			echo '<span class="cooked-rating" data-recipe-id="' . $recipe['id'] . '">' . self::rating( $recipe['thumbs_up'], $recipe['thumbs_down'], $recipe, $recipe['rating'] ) . '</span>';
		endif;
	}

	public static function run( $tab_name ){

		global $atts,$profile_user;

		switch( $tab_name ):

			case 'favorites':

				if ( isset($profile_user['favorites']) && !empty( $profile_user['favorites'] ) ):

					$list_atts = apply_filters( 'cooked_profile_list_atts', array(
						'include' => $profile_user['favorites'],
						'columns' => esc_attr( $atts['columns'] ),
						'category' => false,
						'order' => false,
						'orderby' => false,
						'show' => esc_attr( $atts['show'] ),
						'search' => false,
						'pagination' => 'true',
						'layout' => esc_attr( $atts['layout'] ),
						'author' => false
					));

					echo Cooked_Recipes::list_view( $list_atts );

				endif;

			break;

		endswitch;

	}

	public static function engagement_settings( $settings, $pages, $categories ){

		Cooked_Functions::array_splice_assoc( $settings['design']['fields'], 'main_color_hover', 0, array(
			'heart_color' => array(
				'title' => esc_html__('Favorite Heart Color', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the favorite heart icon (when selected).','cooked-pro'),
				'type' => 'color_field',
				'default' => '#d54e21',
				'options' => '#d54e21'
			),
			'star_color_default' => array(
				'title' => esc_html__('Star Rating (Default)', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the rating star when not rated.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#fdaf09',
				'options' => '#fdaf09'
			),
			'star_color_rated' => array(
				'title' => esc_html__('Star Rating (Rated)', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the rating star when rated.','cooked-pro'),
				'type' => 'color_field',
				'default' => '#ed8a00',
				'options' => '#ed8a00'
			),
			'thumbs_up_color' => array(
				'title' => esc_html__('Thumbs Up Color', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the thumbs up icon (when selected).','cooked-pro'),
				'type' => 'color_field',
				'default' => '#1491ff',
				'options' => '#1491ff'
			),
			'thumbs_down_color' => array(
				'title' => esc_html__('Thumbs Down Color', 'cooked-pro'),
				'desc' => esc_html__('Set the color for the thumbs down icon (when selected).','cooked-pro'),
				'type' => 'color_field',
				'default' => '#d54e21',
				'options' => '#d54e21'
			)
		));

		$settings['engagement'] = array(
			'name' => esc_html__('User Engagement','cooked-pro'),
			'icon' => 'user',
			'fields' => array(
				'cooked_registration' => array(
					'title' => esc_html__('User Registration', 'cooked-pro'),
					'desc' => esc_html__('When displaying the login form, do you want visitors to have the ability to register?','cooked-pro'),
					'type' => 'select',
					'default' => 'enabled',
					'options' => array(
						'enabled' => esc_html__('Enabled', 'cooked-pro'),
						'disabled' => esc_html__('Disabled', 'cooked-pro'),
					)
				),
				'profile_page' => array(
					'title' => esc_html__('User Profile Page', 'cooked-pro'),
					'desc' => sprintf( esc_html__('Create a page with the %s shortcode on it, then choose it from this dropdown.','cooked-pro'), '[cooked-profile]' ),
					'type' => 'select',
					'default' => 0,
					'options' => $pages
				),
				'rating_type' => array(
					'title' => esc_html__('Rating Type', 'cooked-pro'),
					'desc' => esc_html__('Choose between the tried-and-true Rating Stars or a more modern Thumbs Up/Down system.','cooked-pro'),
					'type' => 'select',
					'default' => 'stars',
					'options' => array(
						'stars' => esc_html__( 'Rating Stars', 'cooked-pro' ),
						'thumbs' => esc_html__( 'Thumbs Up/Down', 'cooked-pro' )
					)
				)
			)
		);

		return $settings;

	}

	public function single_recipe_settings( $recipe_settings, $post_id ){

		if ( $rating = get_post_meta( $post_id, '_recipe_rating', true ) ):
			$recipe_settings['rating'] = $rating;
			$recipe_settings['thumbs_up'] = get_post_meta( $post_id, '_recipe_thumbs_up', true );
			$recipe_settings['thumbs_down'] = get_post_meta( $post_id, '_recipe_thumbs_down', true );
			$recipe_settings['votes'] = get_post_meta( $post_id, '_recipe_votes', true );
		else:
			$recipe_settings['rating'] = 0;
			$recipe_settings['thumbs_up'] = 0;
			$recipe_settings['thumbs_down'] = 0;
			$recipe_settings['votes'] = array();
		endif;

		if ( $favorites = get_post_meta( $post_id, '_recipe_favorites', true ) ):
			$recipe_settings['favorites'] = $favorites;
		else:
			$recipe_settings['favorites'] = 0;
		endif;

		return $recipe_settings;

	}

	public static function recipe_list_info_content( $shortcode, $type = false ){
		switch( $type ):
			case 'grid':
				$shortcode = '[cooked-info left="author" right="rating"]';
			break;
		endswitch;

		return $shortcode;

	}

	public function available_info_vars( $vars ){

		$vars['taxonomies'] = esc_html__( 'Taxonomies','cooked-pro' );
		$vars['rating'] = esc_html__( 'Rating','cooked-pro' );
		$vars['favorite'] = esc_html__( 'Favorite Heart','cooked-pro' );

		return $vars;

	}

	public function schema_array( $schema_array, $rpost, $recipe ){

	if ( isset($recipe['rating']) && !empty($recipe['votes']) ):
		$schema_array['aggregateRating'] = array(
			'@type' => 'AggregateRating',
			'ratingValue' => ( isset($recipe['rating']) ? $recipe['rating'] : 0 ),
			'ratingCount' => ( isset($recipe['votes']) ? count($recipe['votes']) : 0 ),
			'bestRating' => 5,
			'worstRating' => 1
		);
	endif;

	return $schema_array;

}

	public function settings_sort_options( $sorting_options ){

		$sorting_options['rating_desc'] = esc_html__('Highest Rated First','cooked-pro');
		$sorting_options['rating_asc'] = esc_html__('Lowest Rated First','cooked-pro');
		return $sorting_options;

	}

	public function recipe_info_display_options( $display_options ){

		$display_options['taxonomies'] = esc_html__('Taxonomies','cooked-pro');
		$display_options['rating'] = esc_html__('Rating','cooked-pro');
		$display_options['favorite'] = esc_html__('Favorite Heart','cooked-pro');
		return $display_options;

	}

	public function recipe_info_display_options_defaults( $defaults ){

		$defaults[] = 'rating';
		$defaults[] = 'favorite';
		$defaults[] = 'ingredients';
		return $defaults;

	}

	public function enqueues(){

		$cooked_js_vars = array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
		);

		$min = ( COOKED_DEV ? '' : '.min' );

		wp_enqueue_style( 'cooked-interactions-css', COOKED_PRO_URL . 'modules/interactions/interactions' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_register_script( 'cooked-interactions-js', COOKED_PRO_URL . 'modules/interactions/interactions' . $min . '.js', array('jquery'), COOKED_PRO_VERSION, true );
		wp_localize_script( 'cooked-interactions-js', 'cooked_js_vars', $cooked_js_vars );
		wp_enqueue_script( 'cooked-interactions-js' );

	}

	public function recipe_query_args( $args, $atts, $sorting_types ){

		$orderby = ( $atts['orderby'] && $atts['orderby'] != 'rating' ? esc_attr( $atts['orderby'] ) : ( $atts['orderby'] && $atts['orderby'] == 'rating' ? 'meta_value_num' : ( $sorting_types[0] != 'rating' ? $sorting_types[0] : 'meta_value' ) ) );
		$meta_sort =  ( $atts['orderby'] && $atts['orderby'] != 'rating' ? false : ( $atts['orderby'] && $atts['orderby'] == 'rating' ? '_recipe_rating' : ( $sorting_types[0] != 'rating' ? false : '_recipe_rating' ) ) );

		$args['order'] = ( $atts['order'] ? esc_attr( $atts['order'] ) : $sorting_types[1] );
		$args['orderby'] = $orderby;

		if ( $meta_sort ):

			$args['meta_query']['relation'] = 'AND';
			$args['meta_query']['rating'] = array(
				'key' => '_recipe_rating',
	            'value' => '1',
	            'compare' => '>='
		    );
			$args['meta_query']['rating_exists'] = array(
	            'key' => '_recipe_rating',
	            'compare' => 'EXISTS',
	        );
			$args['orderby'] = array(
		        'rating' => $args['order']
		    );

			unset($args['order']);

		endif;

		return $args;

	}

	public function browse_sorting_types( $sorting_types ){

		global $_cooked_settings;

		if ( isset($_cooked_settings['recipe_info_display_options']) && is_array($_cooked_settings['recipe_info_display_options']) && in_array( 'rating', $_cooked_settings['recipe_info_display_options'] ) ):
			$sorting_types['rating_desc'] = array(
				'slug' => 'rating_desc',
				'name' => esc_html__("Highest Rated","cooked-pro")
			);
			$sorting_types['rating_asc'] = array(
				'slug' => 'rating_asc',
				'name' => esc_html__("Lowest Rated","cooked-pro")
			);
		endif;

		return $sorting_types;

	}

	public function available_info_shortcode_methods( $available_methods ){
		$available_methods['cooked_info_favorite'] = 'Cooked_Interactions';
		$available_methods['cooked_info_rating'] = 'Cooked_Interactions';
		$available_methods['cooked_info_ingredients'] = 'Cooked_Ingredient_Enhancements';
		$available_methods['cooked_info_comments'] = 'Cooked_Social';
		$available_methods['cooked_info_nutrition'] = 'Cooked_Ingredient_Enhancements';
		return $available_methods;
	}

	public static function cooked_info_favorite( $recipe ){
		global $_cooked_settings,$_cooked_user;
		if ( $favorite_html = self::favorite( $recipe['id'] ) ):
			echo '<span class="cooked-favorite">' . $favorite_html . '</span>';
		endif;
	}

	public static function cooked_info_rating( $recipe_meta ){
		global $_cooked_settings,$_cooked_user;

		$my_recipe = Cooked_Profiles::is_my_recipe($recipe_meta['id']);
		$user_voted = ( isset( $_cooked_user ) && isset( $_cooked_user['id'] ) && isset( $recipe_meta['votes'][$_cooked_user['id']] ) ? $recipe_meta['votes'][$_cooked_user['id']] : false );

		if (in_array('rating',$_cooked_settings['recipe_info_display_options'])):

			$recipe_meta['rating'] = ( isset($recipe_meta['rating']) ? $recipe_meta['rating'] : 0 );
			$recipe_meta['votes'] = ( isset($recipe_meta['votes']) ? $recipe_meta['votes'] : array() );
			$recipe_meta['thumbs_up'] = ( isset($recipe_meta['thumbs_up']) ? $recipe_meta['thumbs_up'] : 0 );
			$recipe_meta['thumbs_down'] = ( isset($recipe_meta['thumbs_down']) ? $recipe_meta['thumbs_down'] : 0 );

			if ( $_cooked_settings['rating_type'] == 'stars' ):
				echo '<span class="cooked-rating' . ( $_cooked_user ? ' cooked-ratable' : '' ) . '" data-recipe-id="' . $recipe_meta['id'] . '">' . self::rating( $recipe_meta['rating'], null, $recipe_meta ) . '</span>';
			elseif ( $_cooked_settings['rating_type'] == 'thumbs' ):
				echo '<span class="cooked-rating' . ( $_cooked_user ? ' cooked-ratable' : '' ) . '" data-recipe-id="' . $recipe_meta['id'] . '">' . self::rating( $recipe_meta['thumbs_up'], $recipe_meta['thumbs_down'], $recipe_meta ) . '</span>';
			endif;

		endif;
	}

	public static function rating( $rating_up = 0, $rating_down = 0, $recipe_meta = array(), $static_value = false ){

		global $_cooked_user,$_cooked_settings;

		if ( in_array('rating',$_cooked_settings['recipe_info_display_options']) ):

			$profile_page_url = get_permalink( $_cooked_settings['profile_page'] );
			$user_voted = ( isset($_cooked_user) && isset( $_cooked_user['id'] ) && isset($recipe_meta['votes'][$_cooked_user['id']]) ? $recipe_meta['votes'][$_cooked_user['id']] : false );

			if ( $static_value || $static_value === 0 ):
				$profile_page_url = false;
				$user_voted = $static_value;
			endif;

			if ( $_cooked_settings['rating_type'] == 'stars' ):

				$counter = 5;

				if ( isset($recipe_meta['rating']) && $recipe_meta['rating'] ):
					$average_rating = $recipe_meta['rating'];
				else:
					$average_rating = 0;
				endif;

				if ( $static_value !== false ):
					$average_rating = $static_value;
				endif;

				$rating_html = '';

				if ( $static_value === false ):
					$rating_html .= '<strong class="cooked-meta-title">';
						$rating_html .= esc_html__('Rating','cooked-pro');
						//$rating_html .= '<span class="cooked-current-rating">' . number_format( $average_rating, 1 ) . '</span>';
					$rating_html .= '</strong>';
				endif;
				$rating_html .= '<span class="cooked-rating-stars' . ( $user_voted ? ' cooked-user-rated cooked-rated-' . $user_voted : '' ) . '">';

				if ( $user_voted ):
					$rating_up = $user_voted;
				endif;

				if ( $rating_up < 5 ):

					do {
						if ( !empty($_cooked_user) || !$profile_page_url ):
							$rating_html .= '<span data-rating-value="' . $counter . '" class="cooked-rating-choice cooked-rating-star cooked-rating-star-empty' . ($user_voted == $counter ? ' cooked-is-selected' : '' ) . '"><i class="cooked-icon cooked-icon-star"></i></span>';
						else:
							$rating_html .= '<a href="' . $profile_page_url . '" class="cooked-rating-choice cooked-rating-star cooked-rating-star-empty"><i class="cooked-icon cooked-icon-star"></i></a>';
						endif;
						$counter--;
					} while ( $counter > $rating_up );

				endif;

				if ( $counter > 0 ):
					do {
						if ( !empty($_cooked_user) || !$profile_page_url ):
							$rating_html .= '<span data-rating-value="' . $counter . '" class="cooked-rating-choice cooked-rating-star' . ($user_voted == $counter ? ' cooked-is-selected' : '' ) . '"><i class="cooked-icon cooked-icon-star"></i></span>';
						else:
							$rating_html .= '<a href="' . $profile_page_url . '" class="cooked-rating-choice cooked-rating-star"><i class="cooked-icon cooked-icon-star"></i></a>';
						endif;
						$counter--;
					} while ( $counter > 0 );
				endif;

				$rating_html .= '</span>';

			elseif ( $_cooked_settings['rating_type'] == 'thumbs' ):

				$thumbs_up_total = 0;
				$thumbs_down_total = 0;

				if ( isset($recipe_meta['votes']) && !empty($recipe_meta['votes']) ):
					foreach( $recipe_meta['votes'] as $vote ):
						if ( $vote >= 3 ):
							$thumbs_up_total++;
						else:
							$thumbs_down_total++;
						endif;
					endforeach;
				endif;

				if ( $static_value !== false ):
					$user_voted = $static_value;
				endif;

				$rating_html = '<span class="cooked-rating-thumbs">';

					if ( !empty($_cooked_user) || !$profile_page_url ):

						$rating_html .= '<span data-rating-value="5" class="cooked-rating-thumb' . ( $user_voted && $user_voted >= 3 ? ' cooked-is-selected' : '' ) . '"><i class="cooked-icon cooked-icon-thumbs-up-solid"></i><span class="cooked-rating-thumb-total">' . $thumbs_up_total . '</span></span>';
						$rating_html .= '<span data-rating-value="1" class="cooked-rating-thumb' . ( $user_voted && $user_voted < 3 ? ' cooked-is-selected' : '' ) . '"><i class="cooked-icon cooked-icon-thumbs-down-solid"></i><span class="cooked-rating-thumb-total">' . $thumbs_down_total . '</span></span>';

					else:

						$rating_html .= '<a href="' . $profile_page_url . '" class="cooked-rating-thumb"><i class="cooked-icon cooked-icon-thumbs-up-solid"></i><span class="cooked-rating-thumb-total">' . $thumbs_up_total . '</span></a>';
						$rating_html .= '<a href="' . $profile_page_url . '" class="cooked-rating-thumb"><i class="cooked-icon cooked-icon-thumbs-down-solid"></i><span class="cooked-rating-thumb-total">' . $thumbs_down_total . '</span></a>';

					endif;

				$rating_html .= '</span>';

			endif;

			return $rating_html;

		endif;

		return false;

	}

	public static function favorite( $id = false ){

		global $post,$_cooked_user,$_cooked_settings;

		$favorite_html = '';

		if ( !$id && 'cp_recipe' == get_post_type( $post->ID ) ):
			$id = $post->ID;
		endif;

		if ( $id && is_user_logged_in() && in_array('favorite',$_cooked_settings['recipe_info_display_options']) ):

			$recipe = Cooked_Recipes::get( $id, true );

			$favorite_html .= '<span data-recipe-id="' . $id . '" class="' . ( isset($_cooked_user['favorites']) && in_array($id,$_cooked_user['favorites']) ? 'cooked-is-favorite ' : '' ) . 'cooked-favorite-heart">';
				$favorite_html .= '<i class="cooked-icon cooked-icon-heart cooked-icon-fw"></i>';
				$favorite_html .= '<span class="cooked-favorite-total">' . ( isset( $recipe['favorites'] ) && $recipe['favorites'] ? number_format( $recipe['favorites'], 0 ) : '0' ) . '</span>';
			$favorite_html .= '</span>';

		elseif ( $id && !is_user_logged_in() && isset($_cooked_settings['profile_page']) && in_array('favorite',$_cooked_settings['recipe_info_display_options']) ):

			if ( $profile_page_url = get_permalink( $_cooked_settings['profile_page'] ) ):
				$favorite_html .= '<a href="' . $profile_page_url . '" class="cooked-favorite-heart">';
					$favorite_html .= '<i class="cooked-icon cooked-icon-heart cooked-icon-fw"></i>';
				$favorite_html .= '</a>';
			endif;

		endif;

		return $favorite_html;

	}

	public static function sync_total_favorites(){

		$users = get_users();
		$recipe_favorites = array();
		foreach( $users as $user ):
			$user_meta = get_user_meta( $user->ID, 'cooked_user_meta', true );
			$favorites = ( isset( $user_meta['favorites'] ) && !empty($user_meta['favorites']) ? $user_meta['favorites'] : array() );
			if ( !empty($favorites) ):
				foreach( $favorites as $recipe_id ):
					$recipe_favorites[$recipe_id] = ( isset($recipe_favorites[$recipe_id]) ? $recipe_favorites[$recipe_id]++ : 1 );
				endforeach;
			endif;
		endforeach;

		foreach( $recipe_favorites as $recipe_id => $total ):
			update_post_meta( $recipe_id, '_recipe_favorites', $total );
		endforeach;

	}

	public function favorite_recipe(){

		global $_cooked_user;

		if ( $_cooked_user && isset($_POST['recipe_id']) && isset($_POST['add_remove']) ):

			$add_remove = esc_attr( $_POST['add_remove'] );
			$recipe_id = esc_attr( $_POST['recipe_id'] );
			$favorites = ( isset($_cooked_user['favorites']) && !empty($_cooked_user['favorites']) ? $_cooked_user['favorites'] : array() );
			$current_total = get_post_meta( $recipe_id, '_recipe_favorites', true );

			if ( $add_remove === 'add' ):
				$favorites[] = $recipe_id;
				$current_total++;
			elseif ( $add_remove === 'remove' ):
				if ( ( $key = array_search($recipe_id, $favorites) ) !== false ):
					$current_total--;
					if ( $current_total <= 0 ): $current_total = ''; update_post_meta( $recipe_id, '_recipe_favorites', 0 ); endif;
    				unset( $favorites[$key] );
    			endif;
			else:
				echo 'error';
				wp_die();
			endif;

			$favorites = array_unique( $favorites );
			$_cooked_user['favorites'] = $favorites;

			update_user_meta( $_cooked_user['id'], 'cooked_user_meta', $_cooked_user );
			update_post_meta( $recipe_id, '_recipe_favorites', $current_total );

			echo $current_total;

		else:
			echo 'error';
		endif;
		wp_die();

	}

	public function rate_recipe(){

		global $_cooked_user,$_cooked_settings;

		if ( $_cooked_user && isset($_POST['recipe_id']) && isset($_POST['rating_val']) ):

			$rating_val = esc_html( $_POST['rating_val'] );
			$recipe_id = esc_html( $_POST['recipe_id'] );
			$_recipe = Cooked_Recipes::get( $recipe_id, true );

			$rating = $_recipe['rating'];
			$thumbs_up = $_recipe['thumbs_up'];
			$thumbs_down = $_recipe['thumbs_down'];
			$votes = $_recipe['votes'];

			if ( $rating_val >= 3 ):
				$thumbs_up++;
			elseif ( $rating_val < 3 ):
				$thumbs_down++;
			endif;

			// Add this Vote (or update it)
			if ( isset( $votes[$_cooked_user['id']] ) && $votes[$_cooked_user['id']] == $rating_val && $_cooked_settings['rating_type'] == 'thumbs' ):
				unset($votes[$_cooked_user['id']]);
			else:
				$votes[$_cooked_user['id']] = $rating_val;
			endif;

			// Recalculate the Rating
			$total = 0;
			foreach ( $votes as $uid => $val ):
				$total = $total + $val;
				$vote_counts++;
			endforeach;

			if ( !empty($votes) ):
				$cumulative_rating = 0;
				$total_votes = count( $votes );
				foreach( $votes as $vote ):
					$cumulative_rating = $cumulative_rating + $vote;
				endforeach;
				$new_avg_rating = number_format( ( $cumulative_rating / $total_votes ), 1 );
			else:
				$new_avg_rating = 0;
			endif;

			update_post_meta( $recipe_id, '_recipe_rating', $new_avg_rating );
			update_post_meta( $recipe_id, '_recipe_thumbs_up', $thumbs_up );
			update_post_meta( $recipe_id, '_recipe_thumbs_down', $thumbs_down );
			update_post_meta( $recipe_id, '_recipe_votes', $votes );

			$new_recipe['rating'] = $new_avg_rating;
			$new_recipe['thumbs_up'] = $thumbs_up;
			$new_recipe['thumbs_down'] = $thumbs_down;
			$new_recipe['votes'] = $votes;

			if ( $_cooked_settings['rating_type'] == 'thumbs' ):
				echo self::rating( $thumbs_up, $thumbs_down, $new_recipe );
			elseif ( $_cooked_settings['rating_type'] == 'stars' ):
				echo self::rating( $new_avg_rating, null, $new_recipe );
			endif;

		else:
			echo 'User is not logged in or data was not provided.';
		endif;
		wp_die();

	}

}

new Cooked_Interactions();
