<?php
/**
 * Cooked Recipe List Styles
 *
 * @package     Cooked
 * @subpackage  Recipe List Styles
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_List_Styles {

	function __construct() {

		// Filters
		add_filter( 'cooked_recipe_list_style', array(&$this, 'recipe_list_style'),10, 2 );
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'settings_tabs_fields'), 10, 1 );

		// Actions
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );

	}

	public function enqueues(){
		$min = ( COOKED_DEV ? '' : '.min' );
		wp_enqueue_style( 'cooked-list-styles-css', COOKED_PRO_URL . 'modules/list-styles/list-styles' . $min . '.css', array(), COOKED_PRO_VERSION );
	}

	public function settings_tabs_fields( $settings ){

		Cooked_Functions::array_splice_assoc( $settings['recipe_settings']['fields'], 'browse_page', 0, array(
			'recipe_list_style' => array(
				'title' => esc_html__('Recipe List Style', 'cooked-pro'),
				'desc' => esc_html__('Choose between a stacked grid, modern grid, fitness grid, full-width, or compact (text-only) list style.','cooked-pro'),
				'type' => 'select',
				'default' => 'grid',
				'options' => apply_filters( 'cooked_list_display_types', array(
					'grid' => esc_html__('Stacked Grid','cooked-pro'),
					'modern' => esc_html__('Modern Grid','cooked-pro'),
					'fitness' => esc_html__('Fitness Grid','cooked-pro'),
					'full' => esc_html__('Full Width','cooked-pro'),
					'compact' => esc_html__('Compact List','cooked-pro')
				))
			)
		));

		return $settings;

	}

	public static function recipe_list_style( $style, $requested_style = false ){
		global $_cooked_settings;
		if ( $requested_style && !method_exists( 'Cooked_List_Styles', 'list_style_' . $requested_style ) ):
			return $style;
		elseif ( $requested_style && method_exists( 'Cooked_List_Styles', 'list_style_' . $requested_style ) ):
			$style = array( $requested_style => 'Cooked_List_Styles' );
		else:
			$style = ( isset($_cooked_settings['recipe_list_style']) && $_cooked_settings['recipe_list_style'] != 'grid' ? array( $_cooked_settings['recipe_list_style'] => 'Cooked_List_Styles' ) : $style );
		endif;
		return $style;
	}

	public static function list_style_modern(){
		$module_dir = trailingslashit( esc_attr( plugin_dir_path( __FILE__ ) ) );
		( $theme_template_override = locate_template( 'recipe-single_modern.php')) ? load_template( $theme_template_override, false ) : load_template( $module_dir . 'templates/recipe-single_modern.php', false );
	}

	public static function list_style_fitness(){
		$module_dir = trailingslashit( esc_attr( plugin_dir_path( __FILE__ ) ) );
		( $theme_template_override = locate_template( 'recipe-single_fitness.php')) ? load_template( $theme_template_override, false ) : load_template( $module_dir . 'templates/recipe-single_fitness.php', false );
	}

	public static function list_style_full(){
		$module_dir = trailingslashit( esc_attr( plugin_dir_path( __FILE__ ) ) );
		( $theme_template_override = locate_template( 'recipe-single_full.php')) ? load_template( $theme_template_override, false ) : load_template( $module_dir . 'templates/recipe-single_full.php', false );
	}

	public static function list_style_compact(){
		$module_dir = trailingslashit( esc_attr( plugin_dir_path( __FILE__ ) ) );
		( $theme_template_override = locate_template( 'recipe-single_compact.php')) ? load_template( $theme_template_override, false ) : load_template( $module_dir . 'templates/recipe-single_compact.php', false );
	}

}

new Cooked_List_Styles();
