<?php
/**
 * Cooked Pagination Styles
 *
 * @package     Cooked
 * @subpackage  Pagination Styles
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Pagination {

	function __construct() {

		// Filters
		add_filter( 'cooked_pagination_style', array(&$this, 'pagination_style'),10, 1 );
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'settings_tabs_fields'), 10, 1 );

		// Actions
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );
		add_action( 'cooked_init_pagination', array( &$this, 'init'), 10, 4 );

		// Ajax Pagination
		add_action('wp_ajax_cooked_loadmore', array(&$this, 'load_more_ajax_handler'), 10 );
		add_action('wp_ajax_nopriv_cooked_loadmore', array(&$this, 'load_more_ajax_handler'), 10 );

	}

	public function init( $args, $current_page, $total_pages, $atts ){
		global $is_own_profile;
		wp_localize_script( 'cooked-pagination-js', 'cooked_pagination_vars', array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
			'recipe_args' => serialize( $args ),
			'current_page' => $current_page,
			'max_page' => $total_pages,
			'atts' => $atts,
			'is_own_profile' => $is_own_profile
		) );
		wp_enqueue_script( 'cooked-pagination-js' );
	}

	public function enqueues(){
		$min = ( COOKED_DEV ? '' : '.min' );
		wp_enqueue_style( 'cooked-pagination-css', COOKED_PRO_URL . 'modules/pagination/pagination' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_register_script( 'cooked-pagination-js', COOKED_PRO_URL . 'modules/pagination/pagination' . $min . '.js', array('jquery'), COOKED_PRO_VERSION, true );
	}

	public function settings_tabs_fields( $settings ){

		Cooked_Functions::array_splice_assoc( $settings['recipe_settings']['fields'], 'browse_page', 0, array(
			'pagination_style' => array(
				'title' => esc_html__('Pagination Style', 'cooked-pro'),
				'desc' => esc_html__('There are three unique pagination styles to choose from.','cooked-pro'),
				'type' => 'select',
				'default' => 'numbered_pagination',
				'options' => apply_filters( 'cooked_pagination_types', array(
					'numbered_pagination' => esc_html__('Numbered Pagination','cooked-pro'),
					'load_more_button' => esc_html__('Load More Button','cooked-pro'),
					'load_more_on_scroll' => esc_html__('Load More on Scroll','cooked-pro')
				))
				)
			)
		);

		return $settings;

	}

	public static function pagination_style( $style ){
		global $_cooked_settings;
		$style = ( isset($_cooked_settings['pagination_style']) && $_cooked_settings['pagination_style'] != 'numbered_pagination' ? array( $_cooked_settings['pagination_style'] => 'Cooked_Pagination' ) : $style );
		return $style;
	}

	public static function load_more_button( $current_recipe_page, $total_recipe_pages ){
		if ( $current_recipe_page < $total_recipe_pages ):
			return '<div class="cooked-pages-load-wrap cooked-load-more-button"><span class="cooked-icon-loading-wrap"><i class="cooked-icon cooked-icon-loading cooked-icon-spin"></i></span><a href="#">' . esc_html__( 'More recipes...','cooked-pro' ) . '</a></div>';
		endif;
	}

	public static function load_more_on_scroll( $current_recipe_page, $total_recipe_pages ){
		if ( $current_recipe_page < $total_recipe_pages ):
			return '<div class="cooked-pages-load-wrap cooked-load-more-scroll"><span class="cooked-icon-loading-wrap"><i class="cooked-icon cooked-icon-loading cooked-icon-spin"></i></span></div>';
		endif;
	}

	public static function load_more_ajax_handler(){

		global $recipes,$recipe,$_cooked_settings,$current_recipe_page,$atts,$is_own_profile;

		$atts = $_POST['atts'];
		$is_own_profile = esc_html( $_POST['is_own_profile'] );

		if ( $atts['layout'] ):

			$recipe_list_style = apply_filters( 'cooked_recipe_list_style', array( 'grid' => 'Cooked_Recipes' ), esc_html( $atts['layout'] ) );
			$list_style = esc_attr( key( $recipe_list_style ) );
			$ls_method = 'list_style_' . $list_style;
			$ls_class = current( $recipe_list_style );

			$_cooked_settings['recipe_list_style'] = $list_style;

		endif;

		$recipe_args = unserialize( stripslashes( $_POST['recipe_args'] ) );
		$recipe_args['paged'] = esc_html( $_POST['page'] ) + 1;
		$recipes = Cooked_Recipes::get( $recipe_args );

		foreach( $recipes as $key => $recipe ):

			if ( $key === 'raw' ):
				continue;
			endif;

			$ls_class::$ls_method();

		endforeach;
		die;

	}

}

new Cooked_Pagination();
