<?php

/**
 * Cooked Profiles
 *
 * @package     Cooked
 * @subpackage  Profiles
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'Cooked_Profiles' ) ) :

class Cooked_Profiles {

	function __construct(){

		// Filters
		add_filter( 'query_vars', array( &$this, 'add_query_vars_filter' ) );
		add_filter( 'cooked_author_permalink', array( &$this, 'author_permalink' ), 10, 2 );
		add_filter( 'cooked_author_template_override', array(&$this, 'author_template'), 10, 2 );
		add_filter( 'cooked_recipe_list_style', array(&$this, 'recipe_list_style'),15, 2 );
		add_filter( 'get_avatar', array(&$this, 'custom_avatar'), 1, 5 );
		add_filter( 'get_avatar_url', array(&$this, 'custom_avatar_url'), 1, 3 );

		// Actions
		add_action( 'init', array( &$this, 'user_session_data' ) );
		add_action( 'init', array(&$this, 'recipe_profile_rewrite'), 10 );
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );
		add_action( 'cooked_recipe_manage_before_name', array( &$this, 'recipe_manage_buttons' ), 9, 1 );
		add_action( 'cooked_recipe_manage_before_name', array( &$this, 'add_rating_stars' ), 10, 1 );

		// Form Actions
		add_action( 'cooked_user_login', array(&$this, 'process_login_form') );
		add_action( 'cooked_user_register', array(&$this, 'process_register_form') );
		add_action( 'cooked_user_edit_profile', array(&$this, 'process_edit_profile_form') );

		// Shortcodes
		add_shortcode('cooked-profile', array($this, 'profile_shortcode') );
		add_shortcode('cooked-edit-profile', array($this, 'edit_profile_shortcode') );
		add_shortcode('cooked-login', array($this, 'login_shortcode') );
		add_shortcode('cooked-register', array($this, 'register_shortcode') );
		add_shortcode('cooked-user-recipes', array($this, 'user_recipes_shortcode') );
		add_shortcode('cooked-user-favorites', array($this, 'user_favorites_shortcode') );

	}

	public static function author_permalink( $permalink, $author_id ){

		global $_cooked_settings;
		if ( $_cooked_settings['profile_page'] ):
			if ( get_option( 'permalink_structure' ) ):
				$permalink = untrailingslashit( get_permalink( $_cooked_settings['profile_page'] ) ) . '/' . $author_id . '/';
			else:
				$permalink = get_permalink( $_cooked_settings['profile_page'] ) . '&cooked_pid=' . $author_id;
			endif;
		endif;

		return $permalink;

	}

	public static function recipe_list_style( $style, $requested_style = false ){
		global $_cooked_settings;
		if ( $requested_style && !method_exists( 'Cooked_Profiles', 'list_style_' . $requested_style ) ):
			return $style;
		elseif ( $requested_style && method_exists( 'Cooked_Profiles', 'list_style_' . $requested_style ) ):
			$style = array( $requested_style => 'Cooked_Profiles' );
		else:
			$style = ( isset($_cooked_settings['recipe_list_style']) && $_cooked_settings['recipe_list_style'] != 'grid' ? array( $_cooked_settings['recipe_list_style'] => 'Cooked_List_Styles' ) : $style );
		endif;
		return $style;
	}

	public static function list_style_manage(){
		$module_dir = trailingslashit( esc_attr( plugin_dir_path( __FILE__ ) ) );
		load_template( $module_dir . 'templates/recipe-single_manage.php', false );
	}

	public static function recipe_profile_rewrite() {

		$_cooked_settings = Cooked_Settings::get();

		$profile_page_id = ( isset($_cooked_settings['profile_page']) && $_cooked_settings['profile_page'] ? $_cooked_settings['profile_page'] : false );
		$profile_page_slug = ( $profile_page_id ? basename( get_permalink( $profile_page_id ) ) : false );
		if ( $profile_page_id ):

			add_rewrite_tag('%cooked_pid%', '([^&]+)');
			add_rewrite_rule('^' . $profile_page_slug . '/([^/]*)/page/([^/]*)/?', 'index.php?page_id=' . $profile_page_id . '&paged=$matches[2]&page=$matches[2]&cooked_pid=$matches[1]', 'top' );
			add_rewrite_rule('^' . $profile_page_slug . '/page/([^/]*)/?', 'index.php?page_id=' . $profile_page_id . '&paged=$matches[1]&page=$matches[1]', 'top' );
			add_rewrite_rule('^' . $profile_page_slug . '/([^/]*)/?', 'index.php?page_id=' . $profile_page_id . '&cooked_pid=$matches[1]', 'top' );

	  	endif;

	}

	public static function author_template( $html, $author ){
		$html = '[cooked-profile user_id="' . $author['id'] . '"]';
		return do_shortcode( $html );
	}

	public function enqueues(){
		$min = ( COOKED_DEV ? '' : '.min' );
		wp_enqueue_style( 'cooked-profiles-css', COOKED_PRO_URL . 'modules/profiles/profiles' . $min . '.css', array(), COOKED_PRO_VERSION );
	}

	public static function add_query_vars_filter( $vars ){
		$vars[] = "cooked_pn";
		$vars[] = "cooked_pid";
		return $vars;
	}

	public static function user_recipes_shortcode( $atts, $content = null ){

		global $recipe_atts, $is_own_profile, $_cooked_user;

		$atts = shortcode_atts(
			array(
				'title' => true,
				'columns' => 3,
				'show' => 25,
				'user_id' => $_cooked_user['id']
			), $atts
		);

		$recipe_atts = $atts;

		ob_start();

		if ( !is_user_logged_in() && $atts['user_id'] == $_cooked_user['id'] ):

			echo do_shortcode( '[cooked-login title=' . $recipe_atts['title'] . ']' );

		else:

			$is_own_profile = ( $recipe_atts['user_id'] == $_cooked_user['id'] ? true : false );
			$public = ( isset( $is_own_profile ) && $is_own_profile ? false : true );
			$this_user = Cooked_Users::get( $recipe_atts['user_id'] );

			if ( isset($this_user['recipes']) && !empty( $this_user['recipes'] ) ):

				$recipe_ids = wp_list_pluck( $this_user['recipes'], 'ID' );

				$list_atts = apply_filters( 'cooked_profile_list_atts', array(
					'public_recipes' => $public,
					'include' => $recipe_ids,
					'columns' => esc_attr( $recipe_atts['columns'] ),
					'category' => false,
					'order' => false,
					'orderby' => false,
					'show' => esc_attr( $recipe_atts['show'] ),
					'search' => false,
					'pagination' => 'true',
					'layout' => ( $is_own_profile ? 'manage' : ( isset($_cooked_settings['recipe_list_style']) && $_cooked_settings['recipe_list_style'] ? $_cooked_settings['recipe_list_style'] : 'grid' ) ),
					'author' => $recipe_atts['user_id'],
					'skip_heading' => 1
				));

				echo Cooked_Recipes::list_view( $list_atts );

				wp_enqueue_script( 'cooked-add-recipe-js' );

			endif;

		endif;

		return ob_get_clean();

	}

	public static function user_favorites_shortcode( $atts, $content = null ){

		global $recipe_atts, $is_own_profile, $_cooked_user;

		$atts = shortcode_atts(
			array(
				'title' => true,
				'columns' => 3,
				'show' => 25,
				'layout' => 'grid',
				'user_id' => $_cooked_user['id']
			), $atts
		);

		$recipe_atts = $atts;

		ob_start();

		if ( !is_user_logged_in() ):

			echo do_shortcode( '[cooked-login title=' . $recipe_atts['title'] . ']' );

		else:

			if ( isset($_cooked_user['favorites']) && !empty( $_cooked_user['favorites'] ) ):

				$list_atts = apply_filters( 'cooked_profile_list_atts', array(
					'include' => $_cooked_user['favorites'],
					'columns' => esc_attr( $atts['columns'] ),
					'category' => false,
					'order' => false,
					'orderby' => false,
					'show' => esc_attr( $atts['show'] ),
					'search' => false,
					'pagination' => 'true',
					'layout' => esc_attr( $atts['layout'] ),
					'author' => false
				));

				echo Cooked_Recipes::list_view( $list_atts );

			endif;

		endif;

		return ob_get_clean();

	}

	public static function run( $tab_name ){

		global $atts,$_cooked_settings,$_cooked_user,$current_recipe_page,$paged,$profile_user,$is_own_profile;

		$current_recipe_page = Cooked_Recipes::current_page();
		$recipes_per_page = ( isset($_cooked_settings['recipes_per_page']) && $_cooked_settings['recipes_per_page'] ? $_cooked_settings['recipes_per_page'] : get_option( 'posts_per_page' ) );

		$public = ( $is_own_profile ? false : true );

		switch( $tab_name ):

			case 'recipes':

				echo do_shortcode( "[cooked-user-recipes" . ( isset($atts['columns']) && $atts['columns'] ? " columns=" . esc_attr( $atts['columns'] ) : "" ) . ( isset($atts['show']) && $atts['show'] ? " show=" . esc_attr( $atts['show'] ) : "" ) . " user_id=" . $profile_user['id'] . "]" );

			break;

			case 'edit':

				echo do_shortcode("[cooked-edit-profile]");

			break;

		endswitch;

	}

	public function add_rating_stars( $recipe ){
		global $is_own_profile;
		$profile_tab = esc_attr( get_query_var('cooked_pn','recipes') );
		if ( $profile_tab === 'recipes' && $is_own_profile && Cooked_Profiles::is_my_recipe( $recipe['id'] ) ):
			echo Cooked_Interactions::rating( 0, 0, $recipe, round($recipe['rating']) );
		endif;
	}

	public function recipe_manage_buttons( $recipe ){
		global $is_own_profile,$_cooked_settings,$post;
		$profile_tab = esc_attr( get_query_var('cooked_pn','recipes') );
		$delete_url = ( isset( $_cooked_settings['profile_page'] ) && $_cooked_settings['profile_page'] ? get_permalink( $_cooked_settings['profile_page'] ) : false );
		$recipe_url = esc_url( get_permalink( $recipe['id'] ) );
		$edit_query_args = array( 'edit_recipe' => '' );
		$delete_query_args = array( 'cooked_action' => 'user_delete_recipe', 'rid' => $recipe['id'] );
		if ( $profile_tab === 'recipes' && $is_own_profile && Cooked_Profiles::is_my_recipe( $recipe['id'] ) ):
			echo '<div class="cooked-recipe-actions">';

				// Edit Recipe
				echo '<a href="' . add_query_arg( $edit_query_args, $recipe_url ) . '" class="cooked-button cooked-edit-button"><i class="cooked-icon cooked-icon-pencil"></i>&nbsp;&nbsp;' . esc_html__( 'Edit Recipe', 'cooked-pro' ) . '</a>';

				// Delete Recipe
				if ( isset( $_cooked_settings['recipe_submission_delete_allowed'] ) && $_cooked_settings['recipe_submission_delete_allowed'] ):
					echo '<a href="#" class="cooked-button cooked-delete-button cooked-delete-trigger"><i class="cooked-icon cooked-icon-close"></i></a>';
					echo '<div class="cooked-confirm-block"><strong>' . esc_html__( 'Delete this recipe?', 'cooked-pro' ) . '</strong>&nbsp;<a href="' . add_query_arg( $delete_query_args, $delete_url ) . '" class="cooked-button cooked-delete-button cooked-delete-final" data-recipe="' . esc_attr( $recipe['id'] ) . '">' . esc_html__( 'Delete', 'cooked-pro' ) . '</a><a href="#" class="cooked-button cooked-edit-button cooked-cancel-delete">' . esc_html__( 'Cancel', 'cooked-pro' ) . '</a></div>';
				endif;

			echo '</div>';
		endif;
	}

	public function form_field( $field_type, $field_id, $field ){

		switch( $field_type ):
			case 'text':
				echo '<input type="text" name="' . $field_id . '" value="' . esc_attr( $field['value'] ) . '">';
			break;
			case 'read_only':
				echo '<input type="text" readonly name="' . $field_id . '" value="' . esc_attr( $field['value'] ) . '">';
			break;
			case 'password':
				echo '<input type="password" name="' . $field_id . '" value="">';
			break;
			case 'number':
				echo '<input type="number" step="any" name="' . $field_id . '" value="' . esc_attr( $field['value'] ) . '">';
			break;
		endswitch;

	}

	public static function is_my_recipe( $recipe_id = false ){
		global $_cooked_settings,$_cooked_user;
		if ( !isset($_cooked_user) || isset($_cooked_user) && empty($_cooked_user) ):
			$_user = wp_get_current_user();
			$_cooked_user = Cooked_Users::get( $_user->ID );
		endif;
		if ( isset($_cooked_user) && !empty($_cooked_user) && $recipe_id ):
			$recipe_author_id = get_post_field( 'post_author', $recipe_id );
			if ( $recipe_author_id == $_cooked_user['id'] ):
				return true;
			else:
				return false;
			endif;
		else:
			return false;
		endif;
	}

	public static function user_session_data(){

		global $_cooked_user;

		if ( is_user_logged_in() ):
			$current_user = wp_get_current_user();
			$_cooked_user = Cooked_Users::get( $current_user->ID );
		else:
			$_cooked_user = false;
		endif;

	}

	public function profile_shortcode( $sc_atts, $content = null ){

		global $post,$atts,$_cooked_settings,$_cooked_user,$profile_user,$is_own_profile;

		// Shortcode Attributes
		$atts = shortcode_atts(
			array(
				'user_id' => false,
				'columns' => 3,
				'layout' => 'grid',
				'show' => false
			), $sc_atts
		);

		$cooked_pid = get_query_var( 'cooked_pid' );

		if ( $atts['user_id'] ):
			$profile_user = Cooked_Users::get( $atts['user_id'] );
		elseif ( $cooked_pid ):
			$profile_user = Cooked_Users::get( $cooked_pid );
		else:
			$profile_user = $_cooked_user;
		endif;

		if ( empty($profile_user) )
			return do_shortcode( '[cooked-login]' );

		ob_start();

		$is_own_profile = ( $profile_user['id'] == $_cooked_user['id'] ? true : false );
		$permalink_user_id = ( $is_own_profile ? '' : '/' . $profile_user['id'] . '/' );

		$profile_nav = apply_filters( 'cooked_profile_nav', array(
			'recipes' => array(
				'name' => esc_html__('My Recipes','cooked-pro'),
				'default' => ( get_query_var('cooked_pn','recipes') == 'recipes' ? true : false ),
				'link_hook' => array( &$this,'recipes')
			)
		));

		if ( $is_own_profile || current_user_can('administrator') ):

			$profile_nav['edit'] = array(
				'name' => esc_html__('Edit Profile','cooked-pro') . ( !$is_own_profile && current_user_can('administrator') ? ' (admin)' : '' ),
				'default' => ( get_query_var('cooked_pn','recipes') == 'edit' ? true : false ),
				'link_hook' => array( &$this,'edit')
			);

			$profile_nav['edit_recipe'] = array(
				'link_hook' => array( &$this,'edit_recipe' ),
				'hidden' => true
			);

		endif;

		// Profile Banner
		do_action( 'cooked_profile_banner_before' );

		$hide_avatars = ( isset( $_cooked_settings['hide_author_avatars'][0] ) && $_cooked_settings['hide_author_avatars'][0] == 'hidden' ? true : false );

		echo '<div class="cooked-author-list-heading' . ( $hide_avatars ? ' cooked-no-avatar' : '' ) . '">';
			do_action( 'cooked_profile_banner_start' );
			echo ( isset($profile_user['profile_photo']) && $profile_user['profile_photo'] ? ( !$hide_avatars ? '<span class="cooked-author-avatar">' . $profile_user['profile_photo'] . '</span>' : '' ) : '' );
			echo '<strong class="cooked-meta-title">';
				echo $profile_user['name'];
			echo '</strong><br>';
			echo '<span class="cooked-profile-nav">';

				$link_set = esc_attr( get_query_var('cooked_pn','recipes') );
				$current_link = ( isset( $profile_nav[$link_set] ) ? $profile_nav[$link_set] : $profile_nav['recipes'] );
				$cooked_default_set = false;

				do_action( 'cooked_profile_nav_before' );
				foreach( $profile_nav as $link_id => $link ):
					if ( !isset($link['hidden']) || isset($link['hidden']) && $link['hidden'] !== true ):
						echo '<a href="' . esc_url( add_query_arg( 'cooked_pn', $link_id, untrailingslashit( get_permalink() ) . $permalink_user_id ) ) . '" class="cooked-pn-link' . ( isset($link['default']) && $link['default'] && !$cooked_default_set ? ' cooked-pn-link-active' : '' ) . '" data-ptlink-id="' . $link_id . '" id="cooked-pn-link-">' . $link['name'] . '</a>';
						$cooked_default_set = ( !$cooked_default_set && isset($link['default']) && $link['default'] ? true : $cooked_default_set );
					endif;
				endforeach;
				if ( $is_own_profile ):
					echo '<a href="' . wp_logout_url( get_permalink( $post->ID ) ) . '">' . esc_html__( 'Log Out', 'cooked-pro' ) . '</a>';
				endif;
				do_action( 'cooked_profile_nav_after' );

			echo '</span>';
			do_action( 'cooked_profile_banner_end' );
		echo '</div>';
		do_action( 'cooked_profile_banner_after' );

		echo '<div class="cooked-profile-content" id="cooked-pn-content-' . $link_set . '">';
			if ( isset($current_link['link_hook']) && count($current_link['link_hook']) == 2 ):
				$current_link['link_hook'][0]::run( $current_link['link_hook'][1] );
			elseif ( isset($current_link['link_hook']) ):
				$current_link['link_hook'][0];
			endif;
		echo '</div>';

		wp_reset_query();

		return ob_get_clean();

	}

	public function edit_profile_shortcode( $sc_atts, $content = null ){

		global $profile_atts;

		ob_start();

		$atts = shortcode_atts(
			array(
				'title' => true
			), $sc_atts
		);

		$profile_atts = $atts;

		if ( !is_user_logged_in() ):
			echo do_shortcode( '[cooked-login title=' . $profile_atts['title'] . ']' );
		else:
			load_template( COOKED_PRO_DIR . 'templates/front/form-edit-profile.php',false);
		endif;

		return ob_get_clean();

	}

	public function login_shortcode( $sc_atts, $content = null ){

		global $post,$atts,$_cooked_settings;

		$registration = ( isset($_cooked_settings['cooked_registration']) && $_cooked_settings['cooked_registration'] == 'enabled' ? true : false );

		// Shortcode Attributes
		$atts = shortcode_atts(
			array(
				'title' => true,
				'registration' => $registration,
				'redirect' => get_permalink( $post->ID )
			), $sc_atts
		);

		if ( is_user_logged_in() )
			return;

		if ( $atts['registration'] && isset( $_GET['register'] ) && $registration )
			return do_shortcode( '[cooked-register]' );

		ob_start();
		load_template( COOKED_PRO_DIR . 'templates/front/form-login.php',false);
		return ob_get_clean();

	}

	public function register_shortcode( $sc_atts, $content = null ){

		global $post,$atts,$_cooked_settings;

		// Shortcode Attributes
		$atts = shortcode_atts(
			array(
				'redirect' => get_permalink( $post->ID )
			), $sc_atts
		);

		if ( is_user_logged_in() )
			return;

		ob_start();
		load_template( COOKED_PRO_DIR . 'templates/front/form-register.php',false);
		return ob_get_clean();

	}

	function process_edit_profile_form( $data ) {

		if ( wp_verify_nonce( $data['cooked_edit_profile_nonce'], 'cooked-edit-profile-nonce' ) ) {

			if( !is_user_logged_in() ) {
				return;
			}

			if( empty( $_POST['cooked_edit_profile'] ) ) {
				return;
			}

			if ( get_current_user_id() == $data['user_id'] || current_user_can('administrator') ){
				$user = get_user_by( 'id', $data['user_id'] );
				$user_id = $user->ID;
			} else {
				return;
			}

			do_action( 'cooked_pre_process_edit_profile_form' );

			if( empty( $data['cooked_user_login'] ) ) {
				cooked_set_error( 'empty_username', __( 'Invalid display name.', 'cooked-pro' ) );
			}

			if( $user->user_email != $data['cooked_user_email'] && email_exists( $data['cooked_user_email'] ) ) {
				cooked_set_error( 'email_unavailable', __( 'Email address already taken.', 'cooked-pro' ) );
			}

			if( empty( $data['cooked_user_email'] ) || ! is_email( $data['cooked_user_email'] ) ) {
				cooked_set_error( 'email_invalid', __( 'Invalid email address.', 'cooked-pro' ) );
			}

			if( !empty( $data['cooked_user_pass'] ) && $data['cooked_user_pass'] !== $data['cooked_user_pass2'] ) {
				cooked_set_error( 'password_mismatch', __( 'Passwords do not match.', 'cooked-pro' ) );
			}

			$_user_meta = get_user_meta( $user_id, 'cooked_user_meta', true );

			if ( $data['profile_photo'] && $data['profile_photo_url'] && $data['profile_photo'] != 'custom' ):
				if ( isset($_user_meta['profile_photo_id']) && is_numeric($_user_meta['profile_photo_id']) ): wp_delete_attachment( $_user_meta['profile_photo_id'] ); endif;
				$attach_id = self::set_profile_photo( $data['profile_photo'], $user_id );
				if ( !$attach_id ):
					cooked_set_error( 'photo_upload_error', __( 'There was an issue updating your profile photo.', 'cooked-pro' ) );
				else:
					$_user_meta['profile_photo_id'] = $attach_id;
					update_user_meta( $user_id, 'cooked_user_meta', $_user_meta );
				endif;
			elseif ( !$data['profile_photo'] ):
				if ( isset($_user_meta['profile_photo_id']) && is_numeric($_user_meta['profile_photo_id']) ): wp_delete_attachment( $_user_meta['profile_photo_id'] ); endif;
				$_user_meta['profile_photo_id'] = false;
		    	update_user_meta( $user_id, 'cooked_user_meta', $_user_meta );
			endif;

			do_action( 'cooked_process_edit_profile_form' );

			// Check for errors and redirect if none present
			$errors = cooked_get_errors();

			if ( empty( $errors ) ) {

				if( ( !empty( $_POST['cooked_user_pass'] ) && !empty( $_POST['cooked_user_pass2'] ) ) ) {
					wp_update_user( array( 'ID' => $user_id, 'display_name' => $data['cooked_user_login'], 'user_email' => $data['cooked_user_email'], 'user_pass' => $data['cooked_user_pass'] ) );
				} else {
					wp_update_user( array( 'ID' => $user_id, 'display_name' => $data['cooked_user_login'], 'user_email' => $data['cooked_user_email'] ) );
				}

			}

		}

		if ( is_numeric( $data['redirect_id'] ) ):

			$current_user = wp_get_current_user();
			$is_own_profile = ( $user_id == $current_user->ID ? true : false );
			$permalink_user_id = ( $is_own_profile ? '' : '/' . $user_id . '/' );

			$redirect = untrailingslashit( get_permalink( $data['redirect_id'] ) ) . $permalink_user_id;
			$cooked_profile_nav = ( isset($_GET['cooked_pn']) && $_GET['cooked_pn'] ? esc_attr($_GET['cooked_pn']) : false );
			if ( $cooked_profile_nav ):
				$redirect = add_query_arg( 'cooked_pn', $cooked_profile_nav, $redirect );
			endif;

			wp_redirect( $redirect );
			exit;

		endif;

	}

	public function get_by_id_or_email( $id_or_email ){
		$user = false;
		$user_id = false;
	    if ( is_numeric( $id_or_email ) ):
	        $id = (int) $id_or_email;
	        $user = get_user_by( 'id' , $id );
	    elseif ( is_object( $id_or_email ) ):
	        if ( ! empty( $id_or_email->user_id ) ):
	            $id = (int) $id_or_email->user_id;
	            $user = get_user_by( 'id' , $id );
	        endif;
	    else:
	    	$user = get_user_by( 'email', $id_or_email );
	    endif;

	    if ( $user && is_object( $user ) ):
	    	$user_id = $user->data->ID;
	    endif;
	    return $user_id;
	}

	public function custom_avatar_url( $url, $id_or_email, $args ){

		$user_id = self::get_by_id_or_email( $id_or_email );
    	$_user_meta = get_user_meta( $user_id, 'cooked_user_meta', true );
        if ( isset($_user_meta['profile_photo_id']) && $_user_meta['profile_photo_id'] ):
        	$url = wp_get_attachment_image_src( $_user_meta['profile_photo_id'], 'full' );
			$url = ( isset($url[0]) && $url[0] ? $url[0] : $url );
        endif;
	    return $url;

	}

	public function custom_avatar( $avatar, $id_or_email, $size, $default, $alt ){

		$user_id = self::get_by_id_or_email( $id_or_email );
    	$_user_meta = get_user_meta( $user_id, 'cooked_user_meta', true );
        if ( isset($_user_meta['profile_photo_id']) && $_user_meta['profile_photo_id'] ):
        	$src = wp_get_attachment_image_src( $_user_meta['profile_photo_id'], array( $size,$size ) );
			$src = ( isset($src[0]) && $src[0] ? $src[0] : $src );
	        $avatar = "<img alt='{$alt}' src='{$src}' class='avatar avatar-{$size} photo' height='{$size}' width='{$size}' />";
        endif;
	    return $avatar;

	}

	public function set_profile_photo( $image_url = false, $user_id = false ){

		if ( $image_url && $user_id ):

			$_user_meta = get_user_meta( $user_id, 'cooked_user_meta' );

			if ( !is_array($_user_meta) ):
				$_user_meta = array();
			endif;

            $_uploads = wp_upload_dir();
            $recipe_image_dir = trailingslashit( $_uploads['basedir'] ) . 'cooked/images/users';
            $recipe_image_url = trailingslashit( $_uploads['baseurl'] ) . 'cooked/images/users';

            if ( wp_mkdir_p( $recipe_image_dir ) ):
                if ( file_exists($image_url) || file_exists($recipe_image_dir . "/cooked_user_" . $user_id . ".jpg") ):
                    if ( file_exists($recipe_image_dir . "/cooked_user_" . $user_id . ".jpg") ):
                        unlink( $recipe_image_dir . "/cooked_user_" . $user_id . ".jpg" );
                    endif;
                    if ( file_exists($image_url) ):
                        rename( $image_url, $recipe_image_dir . "/cooked_user_" . $user_id . ".jpg" );
                    endif;
                endif;
            endif;

            $_image_file = $recipe_image_dir . "/cooked_user_" . $user_id . ".jpg";
			$_wp_filetype = wp_check_filetype( $_image_file, null );
		    $attachment = array(
		        'post_mime_type' => $_wp_filetype['type'],
		        'post_title' => sanitize_file_name( "cooked_user_" . $user_id . ".jpg" ),
		        'post_content' => '',
		        'post_status' => 'inherit'
		    );

		    $attach_id = wp_insert_attachment( $attachment, $_image_file );

		    if ( defined( 'ABSPATH' ) && file_exists( ABSPATH . 'wp-admin/includes/image.php' ) ):
		    	require_once( ABSPATH . 'wp-admin/includes/image.php' );
    			$attach_data = wp_generate_attachment_metadata( $attach_id, $_image_file );
    			$update_attachment = wp_update_attachment_metadata( $attach_id, $attach_data );
    			$_user_meta['profile_photo_id'] = $attach_id;
		    	return $attach_id;
		    endif;

        endif;

        return false;

	}

	function process_login_form( $data ) {

		if ( wp_verify_nonce( $data['cooked_login_nonce'], 'cooked-login-nonce' ) ) {
			$user_data = get_user_by( 'login', $data['cooked_user_login'] );
			if ( ! $user_data ) {
				$user_data = get_user_by( 'email', $data['cooked_user_login'] );
			}
			if ( $user_data ) {
				$user_ID = $user_data->ID;
				$user_email = $user_data->user_email;

				if ( wp_check_password( $data['cooked_user_pass'], $user_data->user_pass, $user_data->ID ) ) {

					if ( isset( $data['remember'] ) ) {
						$data['remember'] = true;
					} else {
						$data['remember'] = false;
					}

					self::log_user_in( $user_data->ID, $data['cooked_user_login'], $data['cooked_user_pass'], $data['remember'] );
				} else {
					cooked_set_error( 'password_incorrect', __( 'The password you entered is incorrect', 'cooked-pro' ) );
				}
			} else {
				cooked_set_error( 'username_incorrect', __( 'The username you entered does not exist', 'cooked-pro' ) );
			}
			// Check for errors and redirect if none present
			$errors = cooked_get_errors();
			if ( ! $errors ) {
				$redirect = apply_filters( 'cooked_login_redirect', $data['cooked_redirect'], $user_ID );
				wp_redirect( $redirect );
				exit;
			}
		}

	}

	function process_register_form( $data ) {

		if ( wp_verify_nonce( $data['cooked_register_nonce'], 'cooked-register-nonce' ) ) {

			if( is_user_logged_in() ) {
				return;
			}

			if( empty( $_POST['cooked_register_submit'] ) ) {
				return;
			}

			do_action( 'cooked_pre_process_register_form' );

			if( empty( $data['cooked_user_login'] ) ) {
				cooked_set_error( 'empty_username', __( 'Invalid username', 'cooked-pro' ) );
			}

			if( username_exists( $data['cooked_user_login'] ) ) {
				cooked_set_error( 'username_unavailable', __( 'Username already taken', 'cooked-pro' ) );
			}

			if( ! validate_username( $data['cooked_user_login'] ) ) {
				cooked_set_error( 'username_invalid', __( 'Invalid username', 'cooked-pro' ) );
			}

			if( email_exists( $data['cooked_user_email'] ) ) {
				cooked_set_error( 'email_unavailable', __( 'Email address already taken', 'cooked-pro' ) );
			}

			if( empty( $data['cooked_user_email'] ) || ! is_email( $data['cooked_user_email'] ) ) {
				cooked_set_error( 'email_invalid', __( 'Invalid email', 'cooked-pro' ) );
			}

			if( empty( $_POST['cooked_user_pass'] ) ) {
				cooked_set_error( 'empty_password', __( 'Please enter a password', 'cooked-pro' ) );
			}

			if( ( ! empty( $_POST['cooked_user_pass'] ) && empty( $_POST['cooked_user_pass2'] ) ) || ( $_POST['cooked_user_pass'] !== $_POST['cooked_user_pass2'] ) ) {
				cooked_set_error( 'password_mismatch', __( 'Passwords do not match', 'cooked-pro' ) );
			}

			do_action( 'cooked_process_register_form' );

			// Check for errors and redirect if none present
			$errors = cooked_get_errors();

			if ( empty( $errors ) ) {

				$redirect = apply_filters( 'cooked_register_redirect', $data['cooked_redirect'] );

				self::register_and_login_new_user( array(
					'user_login'      => $data['cooked_user_login'],
					'user_pass'       => $data['cooked_user_pass'],
					'user_email'      => $data['cooked_user_email'],
					'user_registered' => date( 'Y-m-d H:i:s' ),
					'role'            => get_option( 'default_role' )
				) );

				wp_redirect( $redirect );
				exit;

			}

		}

	}

	public static function register_and_login_new_user( $user_data = array() ) {

		// Verify the array
		if ( empty( $user_data ) )
			return -1;

		if ( cooked_get_errors() )
			return -1;

		$user_args = apply_filters( 'cooked_insert_user_args', array(
			'user_login'      => isset( $user_data['user_login'] ) ? $user_data['user_login'] : '',
			'user_pass'       => isset( $user_data['user_pass'] )  ? $user_data['user_pass']  : '',
			'user_email'      => isset( $user_data['user_email'] ) ? $user_data['user_email'] : '',
			'first_name'      => isset( $user_data['user_first'] ) ? $user_data['user_first'] : '',
			'last_name'       => isset( $user_data['user_last'] )  ? $user_data['user_last']  : '',
			'user_registered' => date( 'Y-m-d H:i:s' ),
			'role'            => get_option( 'default_role' )
		), $user_data );

		// Insert new user
		$user_id = wp_insert_user( $user_args );

		// Validate inserted user
		if ( is_wp_error( $user_id ) )
			return -1;

		// Allow themes and plugins to filter the user data
		$user_data = apply_filters( 'cooked_insert_user_data', $user_data, $user_args );

		// Allow themes and plugins to hook
		do_action( 'cooked_insert_user', $user_id, $user_data );

		// Login new user
		self::log_user_in( $user_id, $user_data['user_login'], $user_data['user_pass'] );

		// Return user id
		return $user_id;

	}

	public static function log_user_in( $user_id, $user_login, $user_pass, $remember = false ) {

		if ( $user_id < 1 )
			return;

		wp_set_auth_cookie( $user_id, $remember );
		wp_set_current_user( $user_id, $user_login );
		do_action( 'wp_login', $user_login, get_userdata( $user_id ) );
		do_action( 'cooked_log_user_in', $user_id, $user_login, $user_pass );

	}

}

new Cooked_Profiles();

endif;
