<?php
/**
 * Cooked Social Functions
 *
 * @package     Cooked
 * @subpackage  Social Functions
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Social {

	function __construct() {

		// Actions
		add_action( 'wp_enqueue_scripts', array(&$this, 'header_enqueues'), 10, 1 );
		add_action( 'wp_enqueue_scripts', array(&$this, 'css_responsive'), 11 );
		add_action( 'wp_head', array( &$this, 'cooked_social_meta_tags' ), 5 );

		// Actions
		add_action( 'wp_enqueue_scripts', array(&$this, 'enqueues'), 10, 1 );

		// Filters
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'social_settings'),12, 1 );

		// Shortcodes
		add_shortcode('cooked-sharing', array($this, 'cooked_sharing') );

	}

	public function enqueues(){
		$min = ( COOKED_DEV ? '' : '.min' );
		wp_register_style( 'cooked-social-css', COOKED_PRO_URL . 'modules/social/social' . $min . '.css', array(), COOKED_PRO_VERSION );
		wp_register_script( 'cooked-pinterest-js', '//assets.pinterest.com/js/pinit.js', array(), COOKED_PRO_VERSION, true );
	}

	public function css_responsive(){
		if ( !isset($_GET['print']) ):
			$file = COOKED_PRO_DIR . 'modules/social/responsive.php';
			$css = Cooked_Enqueues::get_dynamic_css( $file );
			wp_add_inline_style( 'cooked-social-css', $css );
		endif;
	}

	public function header_enqueues( $hook ) {
		echo self::facebook_js_sdk();
		echo self::bigoven_js();
	}

	public static function cooked_info_comments( $recipe ){
		$comment_counts = wp_count_comments( $recipe['id'] );
		$total_comments = $comment_counts->approved;
		echo '<span class="cooked-comments"><span class="cooked-comments-icon"><i class="cooked-icon cooked-icon-comment cooked-icon-fw"></i></span><span class="cooked-comments-total">' . $total_comments . '</span></span>';
	}

	public function cooked_social_meta_tags(){

		global $_cooked_settings,$post;

		if ( isset( $post->post_type ) && $post->post_type == 'cp_recipe' ):

			ob_start();

			$recipe = get_post( $post->ID );
			$recipe_settings = Cooked_Recipes::get( $post->ID, true );
			$image_url = false;

			if ( has_post_thumbnail($recipe) ) :
	   			$image_url = get_the_post_thumbnail_url( $recipe,'cooked-large' );
			endif;

			if ( isset( $_cooked_settings['social_sharing'] ) && !empty( $_cooked_settings['social_sharing'] ) && in_array( 'facebook', $_cooked_settings['social_sharing'] ) && isset( $_cooked_settings['facebook_app_id'] ) && $_cooked_settings['facebook_app_id'] ):
				?><meta property="fb:app_id" content="<?php echo esc_attr( $_cooked_settings['facebook_app_id'] ); ?>">
			<?php endif;

			if ( isset( $_cooked_settings['social_sharing'] ) && !empty( $_cooked_settings['social_sharing'] ) && in_array( 'twitter', $_cooked_settings['social_sharing'] ) ):
				?><meta name="twitter:card" content="summary_large_image">
				<?php if ( isset( $_cooked_settings['twitter_attribution'] ) && $_cooked_settings['twitter_attribution'] ):
					?><meta name="twitter:site" content="@<?php echo esc_attr( str_replace( '@', '', $_cooked_settings['twitter_attribution'] ) ); ?>">
				<?php endif;
				?><meta name="twitter:title" content="<?php echo esc_attr( $post->post_title ); ?>">
				<meta name="twitter:description" content="<?php echo esc_attr( $recipe_settings['excerpt'] ); ?>">
				<meta name="twitter:image" content="<?php echo esc_attr( $image_url ); ?>">
			<?php endif;

			echo ob_get_clean();

		endif;

	}

	public function cooked_sharing( $atts, $content = null ){

		global $_cooked_settings;

		wp_enqueue_style( 'cooked-social-css' );

		$sharing_buttons = Cooked_Social::sharing_options();
		$active_buttons = ( isset( $_cooked_settings['social_sharing'] ) && !empty( $_cooked_settings['social_sharing'] ) ? $_cooked_settings['social_sharing'] : array() );
		$is_active = ( empty( $active_buttons ) ? false : true );

		if ( in_array( 'facebook', $active_buttons ) && !$_cooked_settings['facebook_app_id'] ):
			if ( ( $key = array_search( 'facebook', $active_buttons ) ) !== false ):
			    unset( $active_buttons[$key] );
			endif;
		endif;

		if ( in_array( 'pinterest', $active_buttons ) ):
			wp_enqueue_script( 'cooked-pinterest-js' );
		endif;

		if ( $is_active ):

			ob_start();

			echo '<div class="cooked-sharing-actions cooked-clearfix' . ( isset( $_cooked_settings['sharing_alignment'] ) && $_cooked_settings['sharing_alignment'] ? ' ' . esc_attr( $_cooked_settings['sharing_alignment'] ) : ' cooked-aligned-center' ) . ( isset( $_cooked_settings['sharing_style'] ) && $_cooked_settings['sharing_style'] ? ' ' . esc_attr( $_cooked_settings['sharing_style'] ) : ' cooked-default-style' ) . '">';
				echo '<div class="cooked-sharing-wrap">';
				foreach( $sharing_buttons as $button ):

					if ( is_array($button) ):
						$class = esc_attr( $button['callback'][0] );
						$method = esc_attr( $button['callback'][1] );
						if ( class_exists($class) ):
							$_class = new $class;
							echo ( method_exists($_class,$method) ? $class::$method( $button['title'] ) : '' );
						endif;
					else:
						$function = esc_attr( $button['callback'][0] );
						echo ( function_exists($function) ? $function( $button['title'] ) : '' );
					endif;

				endforeach;
				echo '</div>';
			echo '</div>';

			return ob_get_clean();

		endif;

	}

	public static function social_settings( $settings_array ){

		$cooked_sharing_options = self::sharing_options();

		$settings_array['sharing'] = array(
			'name' => esc_html__('Socials & Sharing','cooked-pro'),
			'icon' => 'comment',
			'fields' => array(
				'social_sharing' => array(
					'title' => esc_html__('Sharing Options', 'cooked-pro'),
					'desc' => sprintf( esc_html_x('Choose the sharing options you would like to display using the %s shortcode.','[cooked-sharing]','cooked-pro'), '<code>[cooked-sharing]</code>' ),
					'type' => 'checkboxes',
					'default' => array(),
					'options' => $cooked_sharing_options
				),
				'facebook_app_id' => array(
					'title' => esc_html__('Facebook App ID','cooked-pro'),
					'conditional_requirement' => 'facebook',
					'desc' => sprintf( esc_html_x( 'A %s is required for the %s sharing option above.', 'Facebook', 'cooked-pro' ), '<a href="https://developers.facebook.com/apps/" target="_blank">Facebook App ID</a>', 'Facebook' ),
					'type' => 'text',
					'default' => ''
				),
				'twitter_attribution' => array(
					'title' => esc_html__('Twitter Attribution (optional)','cooked-pro'),
					'conditional_requirement' => 'twitter',
					'desc' => sprintf( esc_html_x( 'Enter a %s username to which you would like to attribute all shared recipes from this website.', 'Twitter', 'cooked-pro' ), 'Twitter' ),
					'placeholder' => '@username',
					'type' => 'text',
					'default' => ''
				),
				'sharing_style' => array(
					'title' => esc_html__('Sharing Button Style', 'cooked-pro'),
					'desc' => esc_html__('You can choose between colored and minimal buttons.','cooked-pro'),
					'type' => 'radio',
					'default' => 'cooked-default-style',
					'options' => apply_filters( 'cooked_sharing_style_options', array(
						'cooked-default-style' => esc_html__('Colored','cooked-pro'),
						'cooked-minimal-style' => esc_html__('Minimal','cooked-pro'),
						'cooked-grayscale-style' => esc_html__('Grayscale','cooked-pro')
					))
				),
				'sharing_alignment' => array(
					'title' => esc_html__('Sharing Button Alignment', 'cooked-pro'),
					'desc' => esc_html__('You can choose between centered, left and right aligned buttons.','cooked-pro'),
					'type' => 'radio',
					'default' => 'cooked-aligned-center',
					'options' => apply_filters( 'cooked_sharing_alignment_options', array(
						'cooked-aligned-left' => esc_html__('Left','cooked-pro'),
						'cooked-aligned-center' => esc_html__('Centered','cooked-pro'),
						'cooked-aligned-right' => esc_html__('Right','cooked-pro')
					))
				),
			)
		);

		return $settings_array;

	}

	public static function sharing_options(){

		$sharing_options = apply_filters( 'cooked_sharing_options', array(
			'facebook' => array(
				'conditional_value' => 'facebook',
				'label' => esc_html__('Facebook','cooked-pro'),
				'title' => esc_html__( 'Share','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'facebook_share' )
			),
			'twitter' => array(
				'conditional_value' => 'twitter',
				'label' => esc_html__( 'Twitter','cooked-pro' ),
				'title' => esc_html__( 'Tweet','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'twitter_share' )
			),
			'pinterest' => array(
				'label' => esc_html__( 'Pinterest','cooked-pro' ),
				'title' => esc_html__( 'Save','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'pinterest_share' )
			),
			'google' => array(
				'label' => esc_html__( 'Google','cooked-pro' ),
				'title' => esc_html__( 'Share','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'google_share' )
			),
			'viber' => array(
				'label' => esc_html__( 'Viber','cooked-pro' ),
				'title' => esc_html__( 'Share','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'viber_share' )
			),
			'bigoven' => array(
				'label' => esc_html__( 'BigOven','cooked-pro' ),
				'title' => esc_html__( 'BigOven', 'cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'bigoven_save' )
			),
			'email' => array(
				'label' => esc_html__( 'Email','cooked-pro' ),
				'title' => esc_html__( 'Email','cooked-pro' ),
				'callback' => array( 'Cooked_Social', 'email_share' )
			),
			// 'embed' => array(
			// 	'title' => esc_html__( 'Embed','cooked-pro' ),
			// 	'callback' => array( 'Cooked_Social', 'embed_share' )
			// )
		));

		return $sharing_options;

	}

	public static function bigoven_js(){
		ob_start();
		echo "<script>function doSaveRecipe() { var BO_HOST = 'https://www.bigoven.com'; var x = document.createElement('script'); var parentUrl = document.URL; x.type = 'text/javascript'; x.src = BO_HOST + '/assets/noexpire/js/getrecipe.js?' + (new Date().getTime() / 100000); document.getElementsByTagName('head')[0].appendChild(x); }</script>";
		return ob_get_clean();
	}

	public static function facebook_js_sdk(){

		global $_cooked_settings;

		if ( isset($_cooked_settings['facebook_app_id']) && $_cooked_settings['facebook_app_id'] && isset($_cooked_settings['social_sharing']) && in_array( 'facebook', $_cooked_settings['social_sharing'] ) ):

			ob_start();

			$locale = get_locale();
			$locale = ( $locale ? esc_js( $locale ) : 'en_US' );
			$lp = explode( '_', $locale );
			if ( isset( $lp[0] ) && $lp[0] == 'ar' ):
				$locale = 'ar_AR';
			elseif ( isset( $lp[0] ) && $lp[0] == 'es' ):
				$locale = 'es_LA';
			endif;

			echo "
<script>
	window.fbAsyncInit = function() {
    	FB.init({
      		appId      : '" . $_cooked_settings['facebook_app_id'] . "',
      		xfbml      : true,
      		version    : 'v2.8'
    	});
    	FB.AppEvents.logPageView();
  	};

  	(function(d, s, id){
     	var js, fjs = d.getElementsByTagName(s)[0];
     	if (d.getElementById(id)) {return;}
     	js = d.createElement(s); js.id = id;
     	js.src = \"//connect.facebook.net/" . $locale . "/sdk.js\";
    	fjs.parentNode.insertBefore(js, fjs);
   	}(document, 'script', 'facebook-jssdk'));
</script>

";

			return ob_get_clean();

		endif;

	}

	public static function get_hashtags_list( $recipe ){

		global $recipe_settings;

		$hashtags = array();
		$taxonomies = apply_filters( 'cooked_active_taxonomies', array( 'cp_recipe_category' ) );
		$_tags = ( !empty($taxonomies) && in_array( 'cp_recipe_tags', $taxonomies ) ? wp_get_post_terms( $recipe_settings['id'], 'cp_recipe_tags' ) : array() );

		if ( !empty($_tags) ):
			foreach( $_tags as $tag ):
				$hashtags[] = self::convert_to_hashtag( $tag->name );
			endforeach;
		endif;

		return $hashtags;

	}

	public static function convert_to_hashtag( $string ){
		return strtolower( sanitize_text_field( preg_replace( '/\s+/', '',  $string ), '', '' ) );
	}

	public static function twitter_share( $title = 'Tweet' ){

		global $_cooked_settings,$recipe_settings,$post;

		if ( isset($_cooked_settings['social_sharing']) && in_array( 'twitter', $_cooked_settings['social_sharing'] ) ):

			ob_start();

			$recipe = get_post( $recipe_settings['id'] );
			$recipe_settings['title'] = ( isset( $recipe_settings['title'] ) ? $recipe_settings['title'] : get_the_title( $recipe ) );
			$attribution = ( isset($_cooked_settings['twitter_attribution']) && $_cooked_settings['twitter_attribution'] ? str_replace( '@', '', $_cooked_settings['twitter_attribution'] ) : '' );

			$hashtags = self::get_hashtags_list( $recipe );
			$hashtags = ( !empty($hashtags) ? implode( ',', $hashtags ) : '' );

			$url = ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_public_recipes', $_cooked_settings['advanced'] ) ? get_permalink( $post->ID ) : get_permalink( $recipe_settings['id'] ) );

			echo '<span id="cooked_twitter_share_' . $recipe_settings['id'] . '" class="cooked-twitter-share cooked-button" data-url="' . esc_url( 'https://twitter.com/intent/tweet?text=' . $recipe_settings['title'] ) . ':&url=' . $url . ( $attribution ? '&via=' . $attribution : '' ) . ( $hashtags ? '&hashtags=' . $hashtags : '' ) . '">' . esc_html( $title ) . '</span><script>
  jQuery("#cooked_twitter_share_' . $recipe_settings['id'] . '").click(function(event) {
    var width  = 575,
        height = 400,
        left   = (jQuery(window).width()  - width)  / 2,
        top    = (jQuery(window).height() - height) / 2,
        url    = jQuery(this).data("url"),
        opts   = "status=1" +
                 ",width="  + width  +
                 ",height=" + height +
                 ",top="    + top    +
                 ",left="   + left;

    window.open(url, "twitter", opts);

    return false;
  });
</script>';

			return ob_get_clean();

		endif;

	}

	public static function pinterest_share( $title = 'Save' ){

		global $_cooked_settings,$post;

		if ( isset($_cooked_settings['social_sharing']) && in_array( 'pinterest', $_cooked_settings['social_sharing'] ) ):

			global $recipe,$recipe_settings;
			$recipe_image = ( has_post_thumbnail($recipe) ? get_the_post_thumbnail_url( $recipe,'cooked-large' ) : false );

			$hashtags = self::get_hashtags_list( $recipe );
			foreach( $hashtags as $h => $tag ):
				$hashtags[$h] = $tag;
			endforeach;
			$hashtags = ( !empty($hashtags) ? implode( ' ', $hashtags ) : '' );

			$url = ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_public_recipes', $_cooked_settings['advanced'] ) ? get_permalink( $post->ID ) : get_permalink( $recipe_settings['id'] ) );

			return '<span id="cooked_pinterest_share_' . $recipe_settings['id'] . '" class="cooked-pinterest-share cooked-button" data-pin-log="button_pinit" data-pin-href="' . esc_url( 'https://www.pinterest.com/pin/create/button/?url=' . $url . ( $recipe_image ? '&media=' . $recipe_image : '' ) . '&description=' . esc_attr( $recipe_settings['title'] ) . ( $hashtags ? rawurlencode( ' ' . $hashtags ) : '' ) ) . '">' . esc_html( $title ) . '</span>';

		endif;

	}

	public static function bigoven_save( $title = 'BigOven' ){
		global $_cooked_settings,$post;
		if ( isset($_cooked_settings['social_sharing']) && in_array( 'bigoven', $_cooked_settings['social_sharing'] ) ):
			ob_start();
			echo '<span class="cooked-button cooked-bigoven-save" style="cursor:pointer;" title="' . esc_attr__( 'Save recipe or add to grocery list', 'cooked-pro' ) . '" onclick="javascript:doSaveRecipe();">' . esc_html( $title ) . '</span>';
			return ob_get_clean();
		endif;
	}

	public static function viber_share( $title = 'Share' ){
		global $_cooked_settings,$post;
		if ( isset($_cooked_settings['social_sharing']) && in_array( 'viber', $_cooked_settings['social_sharing'] ) ):
			ob_start();
			echo '<a id="viber_share" class="cooked-button cooked-viber-share">' . esc_attr__( 'Share', 'cooked-pro' ) . '</a>';
			echo '<script>var buttonID = "viber_share"; var text = "' . esc_js( __( 'Share', 'cooked-pro' ) ) . '"; document.getElementById(buttonID).setAttribute(\'href\', "https://3p3x.adj.st/?adjust_t=u783g1_kw9yml&adjust_fallback=https%3A%2F%2Fwww.viber.com%2F%3Futm_source%3DPartner%26utm_medium%3DSharebutton%26utm_campaign%3DDefualt&adjust_campaign=Sharebutton&adjust_deeplink=" + encodeURIComponent("viber://forward?text=" + encodeURIComponent(text + " " + window.location.href)));</script>';
			return ob_get_clean();
		endif;
	}

	public static function google_share( $title = 'Share' ){

		global $_cooked_settings,$post;

		if ( isset($_cooked_settings['social_sharing']) && in_array( 'google', $_cooked_settings['social_sharing'] ) ):

			ob_start();

			global $recipe,$recipe_settings;
			$recipe_image = ( has_post_thumbnail($recipe) ? get_the_post_thumbnail_url( $recipe,'cooked-large' ) : false );
			$url = ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_public_recipes', $_cooked_settings['advanced'] ) ? get_permalink( $post->ID ) : get_permalink( $recipe_settings['id'] ) );

			echo '<span id="cooked_google_share_' . $recipe_settings['id'] . '" class="cooked-button cooked-google-share" data-url="' . esc_url( 'https://plus.google.com/share?url=' . $url ) . '">' . esc_html( $title ) . '</span><script>
  jQuery("#cooked_google_share_' . $recipe_settings['id'] . '").click(function(event) {
    var width  = 400,
        height = 600,
        left   = (jQuery(window).width()  - width)  / 2,
        top    = (jQuery(window).height() - height) / 2,
        url    = jQuery(this).data("url"),
        opts   = "status=1" +
                 ",width="  + width  +
                 ",height=" + height +
                 ",top="    + top    +
                 ",left="   + left;

    window.open(url, "google", opts);

    return false;
  });
</script>';

			return ob_get_clean();

		endif;

	}

	public static function email_share( $title = 'Email' ){

		global $_cooked_settings,$post;

		if ( isset($_cooked_settings['social_sharing']) && in_array( 'email', $_cooked_settings['social_sharing'] ) ):

			global $recipe,$recipe_settings;
			$url = ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_public_recipes', $_cooked_settings['advanced'] ) ? get_permalink( $post->ID ) : get_permalink( $recipe_settings['id'] ) );
			return '<a href="mailto:?subject=' . esc_attr( $recipe_settings['title'] ) . '&body=' . esc_url( $url ) . '" class="cooked-button cooked-email-share">' . esc_html( $title ) . '</a>';

		endif;

	}

	// public static function embed_share( $title = 'Embed' ){

	// 	global $_cooked_settings;

	// 	if ( isset($_cooked_settings['social_sharing']) && in_array( 'embed', $_cooked_settings['social_sharing'] ) ):

	// 		global $recipe,$recipe_settings;
	// 		return '<span data-recipe-id="' . esc_attr( $recipe_settings['id'] ) . '" class="cooked-button cooked-embed-share">'. esc_html( $title ) .'</span>';

	// 	endif;

	// }

	public static function facebook_share( $title = 'Share' ){

		global $_cooked_settings,$recipe_settings,$url;

		if ( isset($_cooked_settings['facebook_app_id']) && $_cooked_settings['facebook_app_id'] && isset($_cooked_settings['social_sharing']) && in_array( 'facebook', $_cooked_settings['social_sharing'] ) ):

			ob_start();

			$url = ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_public_recipes', $_cooked_settings['advanced'] ) ? get_permalink( $post->ID ) : get_permalink( $recipe_settings['id'] ) );

			echo "
<span id=\"cooked_facebook_share_" . $recipe_settings['id'] . "\" class=\"cooked-facebook-share cooked-button\">" . esc_html( $title ) . "</span><script>
	document.getElementById('cooked_facebook_share_" . $recipe_settings['id'] . "').onclick = function() {
		FB.ui({
	    	method: 'share',
	    	display: 'popup',
	    	href: '" . $url . "',
	  	}, function(response){});
	}
</script>";

			return ob_get_clean();

		endif;

	}

}

new Cooked_Social();

