<?php
/**
 * Cooked Extended Taxonomies
 *
 * @package     Cooked
 * @subpackage  Extended Taxonomies
 * @since       1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

require_once 'widgets/init.php';

class Cooked_Taxonomies_Extended {

	function __construct() {

		// Actions
		add_action( 'template_redirect', array( &$this, 'redirect_to_browse_page' ) );
		remove_action( 'cooked_check_recipe_query', array( 'Cooked_Recipes', 'check_recipe_query' ), 10 );
		add_action( 'cooked_check_recipe_query', array( &$this, 'pro_check_recipe_query' ), 10 );
		add_action( 'cooked_after_search_filter_columns', array( &$this, 'search_filter_columns' ), 10 );
		add_action( 'cooked_info_taxonomies_shortcode_after', array( &$this, 'taxonomies_shortcode_after' ), 10, 1 );
		add_action( 'admin_init', array( &$this, 'add_diet_terms' ), 11 );

		// Filters
		add_filter( 'cooked_taxonomy_settings', array(&$this, 'extended_taxonomy_settings'), 10, 1 );
		add_filter( 'cooked_taxonomies', array(&$this, 'extended_taxonomies'), 10, 3 );
		add_filter( 'cooked_settings_tabs_fields', array(&$this, 'settings_tabs_fields'), 10, 1 );
		add_filter( 'cooked_taxonomy_options', array(&$this, 'taxonomy_options'), 10, 1 );
		add_filter( 'cooked_taxonomy_settings_update', array(&$this, 'taxonomy_settings_update'), 10, 1 );
		add_filter( 'cooked_tax_query_filter', array(&$this, 'tax_query_filter'), 10, 2 );
		add_filter( 'cooked_schema_array', array(&$this,'schema_array'), 10, 3 );
		add_filter( 'cooked_browse_shortcode_default_attributes', array(&$this, 'browse_shortcode_default_attributes'), 10, 1 );
		add_filter( 'cooked_active_taxonomies', array(&$this, 'active_taxonomies'), 10, 1 );
		add_filter( 'cooked_term_name', array(&$this, 'tag_name_display'), 10, 3 );
		add_filter( 'get_term', array(&$this, 'get_term_filter'), 10, 2 );
		add_filter( 'term_name', array(&$this, 'term_name_filter'), 10, 2 );

		// Shortcodes
		add_shortcode('cooked-cuisines', array($this, 'cooked_cuisines_shortcode') );
		add_shortcode('cooked-cooking-methods', array($this, 'cooked_cooking_methods_shortcode') );
		add_shortcode('cooked-diets', array($this, 'cooked_diets_shortcode') );
		add_shortcode('cooked-tags', array($this, 'cooked_tags_shortcode') );

		// Widgets
		add_action( 'widgets_init', array(&$this, 'register_widgets'), 10, 1 );
		
		// Taxonomy Titles
		add_action( 'wp_head', array( &$this, 'cooked_meta_tags' ), 5 );
		add_action( 'template_redirect', array( &$this, 'remove_default_title_tag' ) );
		add_filter( 'the_title', array( &$this, 'taxonomy_page_title' ), 10, 2 );
		add_filter( 'pre_wp_nav_menu', array( &$this, 'disable_taxonomy_page_title' ), 10, 2 );
		add_filter( 'wp_nav_menu_items', array( &$this, 'enable_taxonomy_page_title' ), 10, 2 );
		add_filter( 'wp_title', array( &$this, 'taxonomy_meta_title' ), 10 );

	}

	function disable_taxonomy_page_title( $nav_menu, $args ) {
		remove_filter( 'the_title', array( &$this, 'taxonomy_page_title' ), 10, 2 );
		return $nav_menu;
	}
	
	function enable_taxonomy_page_title( $items, $args ) {
		add_filter( 'the_title', array( &$this, 'taxonomy_page_title' ), 10, 2 );
		return $items;
	}

	function taxonomy_page_title( $title = '', $id = 0 ){
		
		if ( is_admin() )
			return $title;
		
		global $wp_query, $post, $_cooked_settings;
		$browse_page_id = $_cooked_settings['browse_page'];
		$taxonomies = self::static_taxonomies();
		
		foreach( $taxonomies as $taxonomy ):
			if ( is_page( $browse_page_id ) && $post->ID == $browse_page_id && isset($wp_query->query[$taxonomy]) && taxonomy_exists($taxonomy) && term_exists( $wp_query->query[$taxonomy], $taxonomy ) ):
				$cooked_term = get_term_by( 'slug', $wp_query->query[$taxonomy], $taxonomy );
				return $cooked_term->name;
			endif;
		endforeach;
		
		return $title;
		
	}
	
	public static function static_taxonomies(){
		return [
			'cp_recipe_cuisine',
			'cp_recipe_cooking_method',
			'cp_recipe_diet',
			'cp_recipe_tags',
		];
	}

	public function remove_default_title_tag(){
		global $wp_query;
		$taxonomies = self::static_taxonomies();
		foreach( $taxonomies as $taxonomy ):
			if ( isset($wp_query->query[$taxonomy]) && taxonomy_exists($taxonomy) && term_exists( $wp_query->query[$taxonomy], $taxonomy ) ):
				remove_action( 'wp_head', '_wp_render_title_tag', 1 );
			endif;
		endforeach;
	}
	
	public static function taxonomy_meta_title( $title = '' ){
		
		global $wp_query, $post, $_cooked_settings;
		$browse_page_id = $_cooked_settings['browse_page'];
		$taxonomies = self::static_taxonomies();
		
		foreach( $taxonomies as $taxonomy ):
			if ( is_page( $browse_page_id ) && $post->ID == $browse_page_id && isset($wp_query->query[$taxonomy]) && taxonomy_exists($taxonomy) && term_exists( $wp_query->query[$taxonomy], $taxonomy ) ):
				$cooked_term = get_term_by( 'slug', $wp_query->query[$taxonomy], $taxonomy );
				return $cooked_term->name;
			endif;
		endforeach;
		
		return $title;
		
	}
	
	public static function cooked_meta_tags(){
	
		global $_cooked_settings,$post,$wp_query;
		
		$taxonomies = self::static_taxonomies();
		
		foreach( $taxonomies as $taxonomy ):
			if ( isset($wp_query->query[$taxonomy]) && taxonomy_exists($taxonomy) && term_exists( $wp_query->query[$taxonomy], $taxonomy ) ):
				$cooked_term = get_term_by( 'slug', $wp_query->query[$taxonomy], $taxonomy );
			endif;
			
			if ( isset($_cooked_settings['advanced']) && !empty($_cooked_settings['advanced']) && in_array( 'disable_meta_tags', $_cooked_settings['advanced'] ) )
				return false;
				
			if ( isset( $cooked_term ) && $cooked_term->name ):
				?><title><?php echo $cooked_term->name . ' / ' . get_bloginfo('name'); ?></title>
				<meta property="og:title" content="<?php echo esc_attr( $cooked_term->name ); ?>">
				<meta property="og:description" content="<?php echo esc_attr( $cooked_term->description ); ?>"><?php
			endif;
		endforeach;

	}

	public function diet_terms(){
		return apply_filters( 'cooked_diet_terms', array(
			'vegetarian' => esc_html__( 'Vegetarian', 'cooked-pro' ),
			'vegan' => esc_html__( 'Vegan', 'cooked-pro' ),
			'gluten-free' => esc_html__( 'Gluten Free', 'cooked-pro' ),
			'diabetic' => esc_html__( 'Diabetic', 'cooked-pro' ),
			'low-calorie' => esc_html__( 'Low Calorie', 'cooked-pro' ),
			'low-fat' => esc_html__( 'Low Fat', 'cooked-pro' ),
			'low-lactose' => esc_html__( 'Low Lactose', 'cooked-pro' ),
			'low-salt' => esc_html__( 'Low Salt', 'cooked-pro' ),
			'halal' => esc_html__( 'Halal', 'cooked-pro' ),
			'hindu' => esc_html__( 'Hindu', 'cooked-pro' ),
			'kosher' => esc_html__( 'Kosher', 'cooked-pro' )
		) );
	}

	public function add_diet_terms(){

		global $_cooked_settings;
		if ( in_array( 'cp_recipe_diet', $_cooked_settings['recipe_taxonomies'] ) ):
			$schema_diets = self::diet_terms();
			foreach( $schema_diets as $schema_key => $diet ):
				if ( !term_exists($schema_key, 'cp_recipe_diet') ):
					wp_insert_term(
						$diet,
						'cp_recipe_diet',
						array(
							'slug' => $schema_key
						)
					);
				endif;
			endforeach;
		endif;

	}

	public function register_widgets() {
		$widgets = apply_filters( 'cooked_pro_widgets', array(
			'Cooked_Widget_Recipe_Cuisines',
			'Cooked_Widget_Recipe_Cooking_Methods',
			'Cooked_Widget_Recipe_Diets',
			'Cooked_Widget_Recipe_Tags'
		));
		if ( !empty($widgets) ):
			foreach( $widgets as $widget ):
				register_widget( $widget );
			endforeach;
		endif;
	}

	public function schema_array( $schema_array, $rpost, $recipe ){

		global $_cooked_settings;

		$cuisine_name = '';
		if (in_array('cp_recipe_cuisine',$_cooked_settings['recipe_taxonomies'])):
			$cuisines = get_the_terms( $rpost->ID, 'cp_recipe_cuisine' );
			if (!empty($cuisines)):
				$cuisine = $cuisines[0];
				$cuisine_name = $cuisine->name;
			endif;
		endif;
		$schema_array['recipeCuisine'] = $cuisine_name;

		$diets = '';
		if (in_array('cp_recipe_diet',$_cooked_settings['recipe_taxonomies'])):
			$diets = get_the_terms( $rpost->ID, 'cp_recipe_diet' );
			if (!empty($diets)):
				if ( count( $diets ) == 1 ):
					$schema_array['suitableForDiet'] = esc_js( str_replace( ' ', '', ucwords( str_replace( '-', ' ', $diets[0]->slug ) ) ) . 'Diet' );
				else:
					foreach( $diets as $diet ):
						$schema_array['suitableForDiet'][] = esc_js( str_replace( ' ', '', ucwords( str_replace( '-', ' ', $diet->slug ) ) ) . 'Diet' );
					endforeach;
				endif;
			endif;
		endif;

		$keywords = array();
		if (in_array('cp_recipe_tags',$_cooked_settings['recipe_taxonomies'])):
			$tags = get_the_terms( $rpost->ID, 'cp_recipe_tags' );
			if (!empty($tags)):
				foreach( $tags as $tag ):
					$keywords[] = str_replace( '#', '', $tag->name );
				endforeach;
				$keywords = implode( ',', $keywords );
			endif;
		endif;
		$schema_array['keywords'] = $keywords;

		return $schema_array;

	}

	public function tag_name_display( $term_name, $term_id, $taxonomy_name ){

		if ( $taxonomy_name != 'cp_recipe_tags' )
			return $term_name;

		return '<span class="cooked-tag">#' . str_replace( '#', '', Cooked_Social::convert_to_hashtag( $term_name ) ) . '</span>';

	}

	public function get_term_filter( $term, $taxonomy ){

		if ( !is_admin() && $taxonomy == 'cp_recipe_tags' ):
	    	$term->name = '#' . str_replace( '#', '', Cooked_Social::convert_to_hashtag( $term->name ) );
	    endif;

	    return $term;

	}

	public function term_name_filter( $pad_tag_name, $tag ){
		if ( $tag && is_numeric($tag) ):
			$term = get_term( $tag );
			if ( isset($term->taxonomy) && $term->taxonomy == 'cp_recipe_tags' ):
				return '#' . str_replace( '#', '', Cooked_Social::convert_to_hashtag( $term->name ) );
			endif;
		endif;
		return $pad_tag_name;
	}

	public static function active_taxonomies( $taxonomies ){

		global $_cooked_settings;
		if ( isset($_cooked_settings['recipe_taxonomies']) && !empty($_cooked_settings['recipe_taxonomies']) ):
			foreach( $_cooked_settings['recipe_taxonomies'] as $taxonomy ):
				if ( !in_array( $taxonomy, $taxonomies ) ):
					$taxonomies[] = $taxonomy;
				endif;
			endforeach;
		endif;

		return $taxonomies;

	}

	public function browse_shortcode_default_attributes( $atts ){
		$atts['cuisine'] = false;
		$atts['cooking-method'] = false;
		$atts['tag'] = false;
		return $atts;
	}

	public function taxonomies_shortcode_after( $recipe_settings ){

		global $_cooked_settings,$recipe_terms_list,$clickable;

		if (in_array('cp_recipe_cuisine',$_cooked_settings['recipe_taxonomies'])):
			if ( $clickable ):
				$recipe_terms_list .= get_the_term_list( $recipe_settings['id'], 'cp_recipe_cuisine', '<span class="cooked-taxonomy cooked-cuisine"><strong class="cooked-meta-title">' . esc_html__('Cuisine','cooked-pro') . '</strong>', ', ', '</span>' );
			else:
				$_recipe_terms_array = array();
				$recipe_terms_list .= '<span class="cooked-taxonomy cooked-cuisine"><strong class="cooked-meta-title">' . esc_html__('Cuisine','cooked-pro') . '</strong>';
					$recipe_terms_array = wp_get_object_terms( $recipe_settings['id'], 'cp_recipe_cuisine' );
					if ( !empty($recipe_terms_array) ):
						if ( ! is_wp_error( $recipe_terms_array ) ):
							foreach( $recipe_terms_array as $recipe_term ):
								$_recipe_terms_array[] = esc_html( $recipe_term->name );
							endforeach;
						endif;
					endif;
					$recipe_terms_list .= implode( ', ', $_recipe_terms_array );
				$recipe_terms_list .= '</span>';
			endif;
		endif;
		if (in_array('cp_recipe_cooking_method',$_cooked_settings['recipe_taxonomies'])):
			if ( $clickable ):
				$recipe_terms_list .= get_the_term_list( $recipe_settings['id'], 'cp_recipe_cooking_method', '<span class="cooked-taxonomy cooked-cooking-method"><strong class="cooked-meta-title">' . esc_html__('Cooking Method','cooked-pro') . '</strong>', ', ', '</span>' );
			else:
				$_recipe_terms_array = array();
				$recipe_terms_list .= '<span class="cooked-taxonomy cooked-cooking-method"><strong class="cooked-meta-title">' . esc_html__('Cooking Method','cooked-pro') . '</strong>';
					$recipe_terms_array = wp_get_object_terms( $recipe_settings['id'], 'cp_recipe_cooking_method' );
					if ( !empty($recipe_terms_array) ):
						if ( ! is_wp_error( $recipe_terms_array ) ):
							foreach( $recipe_terms_array as $recipe_term ):
								$_recipe_terms_array[] = esc_html( $recipe_term->name );
							endforeach;
						endif;
					endif;
					$recipe_terms_list .= implode( ', ', $_recipe_terms_array );
				$recipe_terms_list .= '</span>';
			endif;
		endif;
		if (in_array('cp_recipe_diet',$_cooked_settings['recipe_taxonomies'])):
			if ( $clickable ):
				$recipe_terms_list .= get_the_term_list( $recipe_settings['id'], 'cp_recipe_diet', '<span class="cooked-taxonomy cooked-diet"><strong class="cooked-meta-title">' . esc_html__('Diet','cooked-pro') . '</strong>', ', ', '</span>' );
			else:
				$_recipe_terms_array = array();
				$recipe_terms_list .= '<span class="cooked-taxonomy cooked-cooking-method"><strong class="cooked-meta-title">' . esc_html__('Restricted Diet','cooked-pro') . '</strong>';
					$recipe_terms_array = wp_get_object_terms( $recipe_settings['id'], 'cp_recipe_diet' );
					if ( !empty($recipe_terms_array) ):
						if ( ! is_wp_error( $recipe_terms_array ) ):
							foreach( $recipe_terms_array as $recipe_term ):
								$_recipe_terms_array[] = esc_html( $recipe_term->name );
							endforeach;
						endif;
					endif;
					$recipe_terms_list .= implode( ', ', $_recipe_terms_array );
				$recipe_terms_list .= '</span>';
			endif;
		endif;
		if (in_array('cp_recipe_tags',$_cooked_settings['recipe_taxonomies'])):
			if ( $clickable ):
				$recipe_terms_list .= get_the_term_list( $recipe_settings['id'], 'cp_recipe_tags', '<span class="cooked-tags"><strong class="cooked-meta-title">' . esc_html__('Tags','cooked-pro') . '</strong>', ', ', '</span>' );
			else:
				$_recipe_terms_array = array();
				$recipe_terms_list .= '<span class="cooked-taxonomy cooked-tags"><strong class="cooked-meta-title">' . esc_html__('Tags','cooked-pro') . '</strong>';
					$recipe_terms_array = wp_get_object_terms( $recipe_settings['id'], 'cp_recipe_tags' );
					if ( !empty($recipe_terms_array) ):
						if ( ! is_wp_error( $recipe_terms_array ) ):
							foreach( $recipe_terms_array as $recipe_term ):
								$_recipe_terms_array[] = esc_html( $recipe_term->name );
							endforeach;
						endif;
					endif;
					$recipe_terms_list .= implode( ', ', $_recipe_terms_array );
				$recipe_terms_list .= '</span>';
			endif;
		endif;

	}

	public function search_filter_columns(){

		global $_cooked_settings,$tax_col_count,$active_taxonomy;

		if ( isset($active_taxonomy) ):
			$recipes_page_id = ( $_cooked_settings['browse_page'] ? $_cooked_settings['browse_page'] : get_the_ID() );
			$view_all_recipes_url = get_permalink( $recipes_page_id );
		else:
			$view_all_recipes_url = false;
		endif;

		if ( in_array( 'cp_recipe_cuisine',$_cooked_settings['recipe_taxonomies']) ):
			$terms_array = Cooked_Settings::terms_array( 'cp_recipe_cuisine', false, esc_html__('No cuisines','cooked-pro'), true, true, false );
			if ( !empty($terms_array) ):
				echo '<span class="cooked-tax-column">';
					echo '<span class="cooked-tax-column-title">' . esc_html__('Cuisines','cooked-pro') . '</span>';
					echo '<div class="cooked-tax-scrollable">';
						echo ( $view_all_recipes_url ? '<a href="' . $view_all_recipes_url . '">' . esc_html__( 'All Cuisines','cooked-pro' ) . '</a>' : '' );
						foreach( $terms_array as $key => $val ):
							if ( $key ):
								$term = get_term( $key );
								$term_link = ( !empty($term) ? get_term_link( $term ) : false );
								$term_name = apply_filters( 'cooked_term_name', $term->name, $term->ID, $term->taxonomy );
								echo ( $term_link ? ( isset($active_taxonomy) && $active_taxonomy == $val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $term_link . '">' . $term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $val ? '</strong>' : '' ) : '' );
								$sub_terms_array = Cooked_Settings::terms_array( 'cp_recipe_cuisine', false, false, true, false, $key );
								if ( !empty($sub_terms_array) ):
									foreach( $sub_terms_array as $sub_key => $sub_val ):
										if ( $sub_key ):
											$sub_term = get_term( $sub_key );
											$sub_term_link = ( !empty($sub_term) ? get_term_link( $sub_term ) : false );
											$sub_term_name = apply_filters( 'cooked_term_name', $sub_term->name, $sub_term->ID, $sub_term->taxonomy );
											echo ( $sub_term_link ? '<span class="cooked-tax-sub-item">' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $sub_term_link . '">' . $sub_term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '</strong>' : '' ) . '</span>' : '' );
										endif;
									endforeach;
								endif;
							endif;
						endforeach;
						$tax_col_count++;
					echo '</div>';
				echo '</span>';
			endif;
		endif;

		if ( in_array( 'cp_recipe_cooking_method',$_cooked_settings['recipe_taxonomies']) ):
			$terms_array = Cooked_Settings::terms_array( 'cp_recipe_cooking_method', false, esc_html__('No cooking methods','cooked-pro'), true, true, false );
			if ( !empty($terms_array) ):
				echo '<span class="cooked-tax-column">';
					echo '<span class="cooked-tax-column-title">' . esc_html__('Cooking Methods','cooked-pro') . '</span>';
					echo '<div class="cooked-tax-scrollable">';
						echo ( $view_all_recipes_url ? '<a href="' . $view_all_recipes_url . '">' . esc_html__( 'All Cooking Methods','cooked-pro' ) . '</a>' : '' );
						foreach( $terms_array as $key => $val ):
							if ( $key ):
								$term = get_term( $key );
								$term_link = ( !empty($term) ? get_term_link( $term ) : false );
								$term_name = apply_filters( 'cooked_term_name', $term->name, $term->ID, $term->taxonomy );
								echo ( $term_link ? ( isset($active_taxonomy) && $active_taxonomy == $val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $term_link . '">' . $term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $val ? '</strong>' : '' ) : '' );
								$sub_terms_array = Cooked_Settings::terms_array( 'cp_recipe_cooking_method', false, false, true, false, $key );
								if ( !empty($sub_terms_array) ):
									foreach( $sub_terms_array as $sub_key => $sub_val ):
										if ( $sub_key ):
											$sub_term = get_term( $sub_key );
											$sub_term_link = ( !empty($sub_term) ? get_term_link( $sub_term ) : false );
											$sub_term_name = apply_filters( 'cooked_term_name', $sub_term->name, $sub_term->ID, $sub_term->taxonomy );
											echo ( $sub_term_link ? '<span class="cooked-tax-sub-item">' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $sub_term_link . '">' . $sub_term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '</strong>' : '' ) . '</span>' : '' );
										endif;
									endforeach;
								endif;
							endif;
						endforeach;
						$tax_col_count++;
					echo '</div>';
				echo '</span>';
			endif;
		endif;

		if ( in_array( 'cp_recipe_diet',$_cooked_settings['recipe_taxonomies']) ):
			$terms_array = Cooked_Settings::terms_array( 'cp_recipe_diet', false, esc_html__('No diets','cooked-pro'), true, true, false );
			if ( !empty($terms_array) ):
				echo '<span class="cooked-tax-column">';
					echo '<span class="cooked-tax-column-title">' . esc_html__('Restricted Diets','cooked-pro') . '</span>';
					echo '<div class="cooked-tax-scrollable">';
						echo ( $view_all_recipes_url ? '<a href="' . $view_all_recipes_url . '">' . esc_html__( 'All Diets','cooked-pro' ) . '</a>' : '' );
						foreach( $terms_array as $key => $val ):
							if ( $key ):
								$term = get_term( $key );
								$term_link = ( !empty($term) ? get_term_link( $term ) : false );
								$term_name = apply_filters( 'cooked_term_name', $term->name, $term->ID, $term->taxonomy );
								echo ( $term_link ? ( isset($active_taxonomy) && $active_taxonomy == $val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $term_link . '">' . $term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $val ? '</strong>' : '' ) : '' );
								$sub_terms_array = Cooked_Settings::terms_array( 'cp_recipe_diet', false, false, true, false, $key );
								if ( !empty($sub_terms_array) ):
									foreach( $sub_terms_array as $sub_key => $sub_val ):
										if ( $sub_key ):
											$sub_term = get_term( $sub_key );
											$sub_term_link = ( !empty($sub_term) ? get_term_link( $sub_term ) : false );
											$sub_term_name = apply_filters( 'cooked_term_name', $sub_term->name, $sub_term->ID, $sub_term->taxonomy );
											echo ( $sub_term_link ? '<span class="cooked-tax-sub-item">' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $sub_term_link . '">' . $sub_term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $sub_val ? '</strong>' : '' ) . '</span>' : '' );
										endif;
									endforeach;
								endif;
							endif;
						endforeach;
						$tax_col_count++;
					echo '</div>';
				echo '</span>';
			endif;
		endif;

		if ( in_array( 'cp_recipe_tags',$_cooked_settings['recipe_taxonomies']) ):
			$terms_array = Cooked_Settings::terms_array( 'cp_recipe_tags', false, esc_html__('No tags','cooked-pro'), true, true, false );
			if ( !empty($terms_array) ):
				echo '<span class="cooked-tax-column">';
					echo '<span class="cooked-tax-column-title">' . esc_html__('Tags','cooked-pro') . '</span>';
					echo '<div class="cooked-tax-scrollable">';
						echo ( $view_all_recipes_url ? '<a href="' . $view_all_recipes_url . '">' . esc_html__( 'All Tags','cooked-pro' ) . '</a>' : '' );
						foreach( $terms_array as $key => $val ):
							if ( $key ):
								$term = get_term( $key );
								$term_link = ( !empty($term) ? get_term_link( $term ) : false );
								$term_name = apply_filters( 'cooked_term_name', $term->name, $term->ID, $term->taxonomy );
								echo ( $term_link ? ( isset($active_taxonomy) && $active_taxonomy == $val ? '<strong><i class="cooked-icon cooked-icon-angle-right"></i>&nbsp;&nbsp;' : '' ) . '<a href="' . $term_link . '">' . $term_name . '</a>' . ( isset($active_taxonomy) && $active_taxonomy == $val ? '</strong>' : '' ) : '' );
							endif;
						endforeach;
						$tax_col_count++;
					echo '</div>';
				echo '</span>';
			endif;
		endif;

	}

	public function tax_query_filter( $tax_query, $atts ){

		if ( isset( $atts['cuisine'] ) && $atts['cuisine'] ):
			$tax_query['relation'] = 'AND';
			$tax_query[] = array(
				'taxonomy' 	=> 'cp_recipe_cuisine',
				'field'		=> ( is_numeric($atts['cuisine']) ? 'id' : 'slug' ),
				'terms'		=> array_map( 'trim', explode(',', esc_attr( $atts['cuisine'] ) ) )
			);
		endif;

		if ( isset( $atts['cooking-method'] ) && $atts['cooking-method'] ):
			$tax_query['relation'] = 'AND';
			$tax_query[] = array(
				'taxonomy' 	=> 'cp_recipe_cooking_method',
				'field'		=> ( is_numeric($atts['cooking-method']) ? 'id' : 'slug' ),
				'terms'		=> array_map( 'trim', explode(',', esc_attr( $atts['cooking-method'] ) ) )
			);
		endif;

		if ( isset( $atts['diet'] ) && $atts['diet'] ):
			$tax_query['relation'] = 'AND';
			$tax_query[] = array(
				'taxonomy' 	=> 'cp_recipe_diet',
				'field'		=> ( is_numeric($atts['diet']) ? 'id' : 'slug' ),
				'terms'		=> array_map( 'trim', explode(',', esc_attr( $atts['diet'] ) ) )
			);
		endif;

		if ( isset( $atts['tag'] ) && $atts['tag'] ):
			$tax_query['relation'] = 'AND';
			$tax_query[] = array(
				'taxonomy' 	=> 'cp_recipe_tags',
				'field'		=> ( is_numeric($atts['tag']) ? 'id' : 'slug' ),
				'terms'		=> array_map( 'trim', explode(',', esc_attr( $atts['tag'] ) ) )
			);
		endif;

		return $tax_query;

	}

	/*public function pro_taxonomy_redirects(){

		$_cooked_settings = Cooked_Settings::get();
		$parent_page = ( isset($_cooked_settings['browse_page']) && $_cooked_settings['browse_page'] ? $_cooked_settings['browse_page'] : false );
		$front_page = get_option( 'page_on_front' );

		if ( $parent_page ):
			if ( is_tax('cp_recipe_cuisine') ):
				global $wp_query;
				if ( isset($wp_query->query['cp_recipe_cuisine']) && taxonomy_exists('cp_recipe_cuisine') && term_exists( $wp_query->query['cp_recipe_cuisine'], 'cp_recipe_cuisine' )
				     || isset($wp_query->query['taxonomy']) && $wp_query->query['taxonomy'] == 'cp_recipe_cuisine' && taxonomy_exists('cp_recipe_cuisine') && term_exists( $wp_query->query['term'], 'cp_recipe_cuisine' ) ):
					if ( $parent_page != $front_page && get_option('permalink_structure') ):
						if ( wp_redirect( esc_url_raw( untrailingslashit( get_permalink( $parent_page ) ) . '/' . $_cooked_settings['recipe_cuisine_permalink'] . '/' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query['cp_recipe_cuisine'] ) ) ) ):
							exit;
						endif;
					elseif ( $parent_page == $front_page ):
						if ( wp_redirect( esc_url_raw( get_home_url() . '?cp_recipe_cuisine=' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query['cp_recipe_cuisine'] ) ) ) ):
							exit;
						endif;
					else:
						if ( wp_redirect( esc_url_raw( get_permalink( $parent_page ) . '&cp_recipe_cuisine=' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query['cp_recipe_cuisine'] ) ) ) ):
							exit;
						endif;
					endif;
				endif;
			endif;
		endif;

	}*/

	public function pro_check_recipe_query(){

		global $recipe_query;

		if ( !isset($recipe_query['cp_recipe_category']) ):
			$recipe_query['cp_recipe_category'] = ( isset($_GET['cp_recipe_category']) && $_GET['cp_recipe_category'] ? esc_attr($_GET['cp_recipe_category']) : ( isset($_cooked_settings['browse_default_cp_recipe_category']) && $_cooked_settings['browse_default_cp_recipe_category'] ? $_cooked_settings['browse_default_cp_recipe_category'] : false ) );
		endif;

		if ( !isset($recipe_query['cp_recipe_cuisine']) ):
			$recipe_query['cp_recipe_cuisine'] = ( isset($_GET['cp_recipe_cuisine']) && $_GET['cp_recipe_cuisine'] ? esc_attr($_GET['cp_recipe_cuisine']) : ( isset($_cooked_settings['browse_default_cp_recipe_cuisine']) && $_cooked_settings['browse_default_cp_recipe_cuisine'] ? $_cooked_settings['browse_default_cp_recipe_cuisine'] : false ) );
		endif;

		if ( !isset($recipe_query['cp_recipe_cooking_method']) ):
			$recipe_query['cp_recipe_cooking_method'] = ( isset($_GET['cp_recipe_cooking_method']) && $_GET['cp_recipe_cooking_method'] ? esc_attr($_GET['cp_recipe_cooking_method']) : ( isset($_cooked_settings['browse_default_cp_recipe_cooking_method']) && $_cooked_settings['browse_default_cp_recipe_cooking_method'] ? $_cooked_settings['browse_default_cp_recipe_cooking_method'] : false ) );
		endif;

		if ( !isset($recipe_query['cp_recipe_diet']) ):
			$recipe_query['cp_recipe_diet'] = ( isset($_GET['cp_recipe_diet']) && $_GET['cp_recipe_diet'] ? esc_attr($_GET['cp_recipe_diet']) : ( isset($_cooked_settings['browse_default_cp_recipe_diet']) && $_cooked_settings['browse_default_cp_recipe_diet'] ? $_cooked_settings['browse_default_cp_recipe_diet'] : false ) );
		endif;

		if ( !isset($recipe_query['cp_recipe_tags']) ):
			$recipe_query['cp_recipe_tags'] = ( isset($_GET['cp_recipe_tags']) && $_GET['cp_recipe_tags'] ? esc_attr($_GET['cp_recipe_tags']) : ( isset($_cooked_settings['browse_default_cp_recipe_tags']) && $_cooked_settings['browse_default_cp_recipe_tags'] ? $_cooked_settings['browse_default_cp_recipe_tags'] : false ) );
		endif;

	}

	public function profile_favorites_list_atts( $atts ){

		$atts['cuisine'] = false;
		$atts['cooking-method'] = false;
		$atts['tag'] = false;
		return $atts;

	}

	public function taxonomy_settings_update( $taxonomy_settings ){

		global $_cooked_settings;

		$taxonomy_settings['recipe_cuisine_permalink'] = ( !$_cooked_settings['recipe_cuisine_permalink'] ? 'cuisine' : sanitize_title_with_dashes( $_cooked_settings['recipe_cuisine_permalink'], null, 'save' ) );
		$taxonomy_settings['recipe_cooking_method_permalink'] = ( !$_cooked_settings['recipe_cooking_method_permalink'] ? 'cooking-method' : sanitize_title_with_dashes( $_cooked_settings['recipe_cooking_method_permalink'], null, 'save' ) );
		$taxonomy_settings['recipe_diet_permalink'] = ( !$_cooked_settings['recipe_diet_permalink'] ? 'diet' : sanitize_title_with_dashes( $_cooked_settings['recipe_diet_permalink'], null, 'save' ) );
		$taxonomy_settings['recipe_tag_permalink'] = ( !$_cooked_settings['recipe_tag_permalink'] ? 'recipe-tag' : sanitize_title_with_dashes( $_cooked_settings['recipe_tag_permalink'], null, 'save' ) );

		return $taxonomy_settings;

	}

	public function redirect_to_browse_page(){

		$custom_cooked_taxonomy = false;
		$_cooked_settings = Cooked_Settings::get();
		$front_page = get_option( 'page_on_front' );
		$parent_page = ( isset($_cooked_settings['browse_page']) && $_cooked_settings['browse_page'] ? $_cooked_settings['browse_page'] : false );
		if ( $parent_page ):

			if ( is_tax('cp_recipe_cuisine') ):
				$custom_cooked_taxonomy = 'cp_recipe_cuisine';
				$custom_cooked_tax_setting = 'recipe_cuisine_permalink';
			elseif ( is_tax('cp_recipe_cooking_method') ):
				$custom_cooked_taxonomy = 'cp_recipe_cooking_method';
				$custom_cooked_tax_setting = 'recipe_cooking_method_permalink';
			elseif ( is_tax('cp_recipe_diet') ):
				$custom_cooked_taxonomy = 'cp_recipe_diet';
				$custom_cooked_tax_setting = 'recipe_diet_permalink';
			elseif ( is_tax('cp_recipe_tags') ):
				$custom_cooked_taxonomy = 'cp_recipe_tags';
				$custom_cooked_tax_setting = 'recipe_tag_permalink';
			endif;

			if ( $custom_cooked_taxonomy ):
				global $wp_query;
				if ( isset($wp_query->query[$custom_cooked_taxonomy]) && taxonomy_exists($custom_cooked_taxonomy) && term_exists( $wp_query->query[$custom_cooked_taxonomy], $custom_cooked_taxonomy )
				     || isset($wp_query->query['taxonomy']) && $wp_query->query['taxonomy'] == $custom_cooked_taxonomy && taxonomy_exists($custom_cooked_taxonomy) && term_exists( $wp_query->query['term'], $custom_cooked_taxonomy ) ):
					if ( $parent_page != $front_page && get_option('permalink_structure') ):
						if ( wp_redirect( esc_url_raw( untrailingslashit( get_permalink( $parent_page ) ) . '/' . $_cooked_settings[$custom_cooked_tax_setting] . '/' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query[$custom_cooked_taxonomy] ) ) ) ):
							exit;
						endif;
					elseif ( $parent_page == $front_page ):
						if ( wp_redirect( esc_url_raw( get_home_url() . '?' . $custom_cooked_taxonomy . '=' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query[$custom_cooked_taxonomy] ) ) ) ):
							exit;
						endif;
					else:
						if ( wp_redirect( esc_url_raw( get_permalink( $parent_page ) . '&' . $custom_cooked_taxonomy . '=' . ( isset( $wp_query->query['term'] ) ? $wp_query->query['term'] : $wp_query->query[$custom_cooked_taxonomy] ) ) ) ):
							exit;
						endif;
					endif;
				endif;
			endif;

		endif;

	}

	public function taxonomy_options( $options ){

		$options['cp_recipe_cuisine'] = esc_html__('Cuisines','cooked-pro');
		$options['cp_recipe_cooking_method'] = esc_html__('Cooking Methods','cooked-pro');
		$options['cp_recipe_tags'] = esc_html__('Tags','cooked-pro');
		$options['cp_recipe_diet'] = esc_html__('Restricted Diets','cooked-pro') . '<span class="cooked-checkbox-note"><strong>' . esc_html__('Note:','cooked-pro') . '</strong> ' . sprintf( esc_html__('This taxonomy provides %s support. You cannot add or remove diets.','cooked-pro'), 'Schema.org <em><a href="http://schema.org/RestrictedDiet" target="_blank">RestrictedDiet</a></em>' ) . '</span>';

		return $options;

	}

	public function settings_tabs_fields( $settings ){

		$cuisines_array = Cooked_Settings::terms_array( 'cp_recipe_cuisine', esc_html__('No default','cooked-pro'), esc_html__('No cuisines','cooked-pro') );
		$cooking_methods_array = Cooked_Settings::terms_array( 'cp_recipe_cooking_method', esc_html__('No default','cooked-pro'), esc_html__('No cooking methods','cooked-pro') );
		$diets_array = Cooked_Settings::terms_array( 'cp_recipe_diet', esc_html__('No default','cooked-pro'), esc_html__('No diets','cooked-pro') );
		$tags_array = Cooked_Settings::terms_array( 'cp_recipe_tags', esc_html__('No default','cooked-pro'), esc_html__('No tags','cooked-pro') );

		Cooked_Functions::array_splice_assoc($settings['recipe_settings']['fields'], 'browse_default_cp_recipe_category', 0, array(
			'browse_default_cp_recipe_cuisine' => array(
				'title' => esc_html__('Default Cuisine', 'cooked-pro'),
				'desc' => sprintf( esc_html__('Optionally set the default recipe cuisine for your %s shortcode display.','cooked-pro'), '[cooked-browse]' ),
				'type' => 'select',
				'default' => 0,
				'options' => $cuisines_array
				),
			'browse_default_cp_recipe_cooking_method' => array(
					'title' => esc_html__('Default Cooking Method', 'cooked-pro'),
					'desc' => sprintf( esc_html__('Optionally set the default recipe cooking method for your %s shortcode display.','cooked-pro'), '[cooked-browse]' ),
					'type' => 'select',
					'default' => 0,
					'options' => $cooking_methods_array
				),
			'browse_default_cp_recipe_diet' => array(
					'title' => esc_html__('Default Diet', 'cooked-pro'),
					'desc' => sprintf( esc_html__('Optionally set the default recipe diet for your %s shortcode display.','cooked-pro'), '[cooked-browse]' ),
					'type' => 'select',
					'default' => 0,
					'options' => $diets_array
				),
			'browse_default_cp_recipe_tags' => array(
				'title' => esc_html__('Default Tag', 'cooked-pro'),
				'desc' => sprintf( esc_html__('Optionally set the default recipe tag for your %s shortcode display.','cooked-pro'), '[cooked-browse]' ),
				'type' => 'select',
				'default' => 0,
				'options' => $tags_array
				)
			)
		);

		Cooked_Functions::array_splice_assoc($settings['permalinks']['fields'], 'recipe_category_permalink', 0, array(
			'recipe_cuisine_permalink' => array(
				'title' => esc_html__('Recipe Cuisine Permalink', 'cooked-pro'),
				'desc' => '',
				'type' => 'permalink_field',
				'options' => esc_html__( 'recipe-cuisine-name', 'cooked-pro' ),
				'default' => 'cuisine',
				),
			'recipe_cooking_method_permalink' => array(
				'title' => esc_html__('Recipe Cooking Method Permalink', 'cooked-pro'),
				'desc' => '',
				'type' => 'permalink_field',
				'options' => esc_html__( 'recipe-cooking-method-name', 'cooked-pro' ),
				'default' => 'cooking-method',
				),
			'recipe_diet_permalink' => array(
				'title' => esc_html__('Recipe Diet Permalink', 'cooked-pro'),
				'desc' => '',
				'type' => 'permalink_field',
				'options' => esc_html__( 'recipe-diet-name', 'cooked-pro' ),
				'default' => 'diet',
				),
			'recipe_tag_permalink' => array(
				'title' => esc_html__('Recipe Tag Permalink', 'cooked-pro'),
				'desc' => '',
				'type' => 'permalink_field',
				'options' => esc_html__( 'recipe-tag-name', 'cooked-pro' ),
				'default' => 'recipe-tag',
				)
			)
		);

		return $settings;

	}

	public function extended_taxonomy_settings( $taxonomy_settings ){

		global $_cooked_settings;

		$taxonomy_settings['cp_recipe_cuisine'] = ( isset($_cooked_settings['recipe_cuisine_permalink']) && $_cooked_settings['recipe_cuisine_permalink'] ? $_cooked_settings['recipe_cuisine_permalink'] : 'cuisine' );
		$taxonomy_settings['cp_recipe_cooking_method'] = ( isset($_cooked_settings['recipe_cooking_method_permalink']) && $_cooked_settings['recipe_cooking_method_permalink'] ? $_cooked_settings['recipe_cooking_method_permalink'] : 'cooking-method' );
		$taxonomy_settings['cp_recipe_diet'] = ( isset($_cooked_settings['recipe_diet_permalink']) && $_cooked_settings['recipe_diet_permalink'] ? $_cooked_settings['recipe_diet_permalink'] : 'diet' );
		$taxonomy_settings['cp_recipe_tags'] = ( isset($_cooked_settings['recipe_tag_permalink']) && $_cooked_settings['recipe_tag_permalink'] ? $_cooked_settings['recipe_tag_permalink'] : 'recipe-tag' );

		return $taxonomy_settings;

	}

	public function extended_taxonomies( $taxonomies, $taxonomy_permalinks, $query_var = false ){

		global $_cooked_settings;

		$taxonomies['cp_recipe_cuisine'] = array(
			'hierarchical'        => true,
			'labels'              => array(
				'name'                => esc_html__('Cuisines', 'cooked-pro'),
				'singular_name'       => esc_html__('Cuisine', 'cooked-pro'),
				'search_items'        => esc_html__('Search Cuisines', 'cooked-pro'),
				'all_items'           => esc_html__('All Cuisines', 'cooked-pro'),
				'parent_item'         => esc_html__('Parent Cuisine', 'cooked-pro'),
				'parent_item_colon'   => esc_html__('Parent Cuisine:', 'cooked-pro'),
				'edit_item'           => esc_html__('Edit Cuisine', 'cooked-pro'),
				'update_item'         => esc_html__('Update Cuisine', 'cooked-pro'),
				'add_new_item'        => esc_html__('Add New Cuisine', 'cooked-pro'),
				'new_item_name'       => esc_html__('New Cuisine Name', 'cooked-pro'),
				'menu_name'           => esc_html__('Cuisines', 'cooked-pro'),
				'not_found'           => esc_html__('No Cuisines', 'cooked-pro')
			),
			'show_ui'             => true,
			'show_admin_column'   => true,
			'show_in_menu'		  => false,
			'rest_base'             => 'recipe_cuisine',
    		'rest_controller_class' => 'WP_REST_Terms_Controller',
			'query_var'           => $query_var,
			'rewrite'             => array( 'slug' => $taxonomy_permalinks['cp_recipe_cuisine'] )
		);

		$taxonomies['cp_recipe_cooking_method'] = array(
			'hierarchical'        => true,
			'labels'              => array(
				'name'                => esc_html__('Cooking Methods', 'cooked-pro'),
				'singular_name'       => esc_html__('Cooking Method', 'cooked-pro'),
				'search_items'        => esc_html__('Search Cooking Methods', 'cooked-pro'),
				'all_items'           => esc_html__('All Cooking Methods', 'cooked-pro'),
				'parent_item'         => esc_html__('Parent Cooking Method', 'cooked-pro'),
				'parent_item_colon'   => esc_html__('Parent Cooking Method:', 'cooked-pro'),
				'edit_item'           => esc_html__('Edit Cooking Method', 'cooked-pro'),
				'update_item'         => esc_html__('Update Cooking Method', 'cooked-pro'),
				'add_new_item'        => esc_html__('Add New Cooking Method', 'cooked-pro'),
				'new_item_name'       => esc_html__('New Cooking Method Name', 'cooked-pro'),
				'menu_name'           => esc_html__('Cooking Methods', 'cooked-pro'),
				'not_found'           => esc_html__('No Cooking Methods', 'cooked-pro')
			),
			'show_ui'             => true,
			'show_admin_column'   => true,
			'show_in_menu'		  => false,
			'rest_base'             => 'recipe_cooking_method',
    		'rest_controller_class' => 'WP_REST_Terms_Controller',
			'query_var'           => $query_var,
			'rewrite'             => array( 'slug' => $taxonomy_permalinks['cp_recipe_cooking_method'] )
		);

		$taxonomies['cp_recipe_diet'] = array(
			'hierarchical'        => true,
			'labels'              => array(
				'name'                => esc_html__('Restricted Diets', 'cooked-pro'),
				'singular_name'       => esc_html__('Restricted Diet', 'cooked-pro'),
				'search_items'        => esc_html__('Search Diets', 'cooked-pro'),
				'all_items'           => esc_html__('All Diets', 'cooked-pro'),
				'parent_item'         => esc_html__('Parent Diet', 'cooked-pro'),
				'parent_item_colon'   => esc_html__('Parent Diet:', 'cooked-pro'),
				'edit_item'           => esc_html__('Edit Diet', 'cooked-pro'),
				'update_item'         => esc_html__('Update Diet', 'cooked-pro'),
				'add_new_item'        => esc_html__('Add New Diet', 'cooked-pro'),
				'new_item_name'       => esc_html__('New Diet Name', 'cooked-pro'),
				'menu_name'           => esc_html__('Restricted Diets', 'cooked-pro'),
				'not_found'           => esc_html__('No Diets', 'cooked-pro')
			),
			'show_ui'             => true,
			'show_admin_column'   => true,
			'show_in_menu'		  => false,
			'query_var'           => $query_var,
			'rewrite'             => array( 'slug' => $taxonomy_permalinks['cp_recipe_diet'] )
		);

		$taxonomies['cp_recipe_tags'] = array(
			'hierarchical'        => false,
			'labels'              => array(
				'name'                => esc_html__('Tags', 'cooked-pro'),
				'singular_name'       => esc_html__('Tag', 'cooked-pro'),
				'search_items'        => esc_html__('Search Tags', 'cooked-pro'),
				'all_items'           => esc_html__('All Tags', 'cooked-pro'),
				'parent_item'         => esc_html__('Parent Tag', 'cooked-pro'),
				'parent_item_colon'   => esc_html__('Parent Tag:', 'cooked-pro'),
				'edit_item'           => esc_html__('Edit Tag', 'cooked-pro'),
				'update_item'         => esc_html__('Update Tag', 'cooked-pro'),
				'add_new_item'        => esc_html__('Add New Tag', 'cooked-pro'),
				'new_item_name'       => esc_html__('New Tag Name', 'cooked-pro'),
				'menu_name'           => esc_html__('Tags', 'cooked-pro'),
				'not_found'           => esc_html__('No Tags', 'cooked-pro')
			),
			'show_ui'             => true,
			'show_admin_column'   => true,
			'show_in_menu'		  => false,
			'rest_base'             => 'recipe_tags',
    		'rest_controller_class' => 'WP_REST_Terms_Controller',
			'query_var'           => $query_var,
			'rewrite'             => array( 'slug' => $taxonomy_permalinks['cp_recipe_tags'] )
		);

		if ( !in_array( 'cp_recipe_cuisine', $_cooked_settings['recipe_taxonomies'] ) ): unset( $taxonomies['cp_recipe_cuisine'] ); endif;
		if ( !in_array( 'cp_recipe_cooking_method', $_cooked_settings['recipe_taxonomies'] ) ): unset( $taxonomies['cp_recipe_cooking_method'] ); endif;
		if ( !in_array( 'cp_recipe_diet', $_cooked_settings['recipe_taxonomies'] ) ): unset( $taxonomies['cp_recipe_diet'] ); endif;
		if ( !in_array( 'cp_recipe_tags', $_cooked_settings['recipe_taxonomies'] ) ): unset( $taxonomies['cp_recipe_tags'] ); endif;

		return $taxonomies;

	}

	public function cooked_cuisines_shortcode( $atts, $content = null ){
		if ( is_admin() )
			return false;
		$atts = shortcode_atts( array( 'hide_empty' => true, 'child_of' => false, 'style' => 'block' ), $atts );
		return self::cooked_taxonomy_shortcode_html( 'cp_recipe_cuisine', $atts );
	}

	public function cooked_cooking_methods_shortcode( $atts, $content = null ){
		if ( is_admin() )
			return false;
		$atts = shortcode_atts( array( 'hide_empty' => true, 'child_of' => false, 'style' => 'block' ), $atts );
		return self::cooked_taxonomy_shortcode_html( 'cp_recipe_cooking_method', $atts );
	}

	public function cooked_diets_shortcode( $atts, $content = null ){
		if ( is_admin() )
			return false;
		$atts = shortcode_atts( array( 'hide_empty' => true, 'child_of' => false, 'style' => 'block' ), $atts );
		return self::cooked_taxonomy_shortcode_html( 'cp_recipe_diet', $atts );
	}

	public function cooked_tags_shortcode( $atts, $content = null ){
		if ( is_admin() )
			return false;
		$atts = shortcode_atts( array( 'hide_empty' => true, 'child_of' => false, 'style' => 'block' ), $atts );
		return self::cooked_taxonomy_shortcode_html( 'cp_recipe_tags', $atts );
	}

	public function cooked_taxonomy_shortcode_html( $taxonomy = false, $atts = false ){

		if ( !$taxonomy )
			return false;

		$hide_empty = esc_attr( $atts['hide_empty'] );
		$child_of = esc_attr( $atts['child_of'] );
		$style = esc_attr( $atts['style'] );
		$parents_only = true;

		if ( $child_of ):
			$_term = get_term_by( 'slug', $child_of, $taxonomy );
			$child_of = ( isset( $_term->term_id ) ? $_term->term_id : false );
			$parents_only = ( $child_of ? false : true );
		else:
			$child_of = false;
		endif;

		$categories_array = Cooked_Settings::terms_array( $taxonomy, false, false, $hide_empty, $parents_only, $child_of );

		if ( !empty($categories_array) ):
			ob_start();
			echo ( $style == 'list' ? '<div class="cooked-recipe-term-list">' : '<div class="cooked-recipe-term-grid cooked-clearfix">' );
				foreach( $categories_array as $key => $val ):
					Cooked_Taxonomies::single_taxonomy_block( $key, $style, $taxonomy );
				endforeach;
			echo '</div>';
			return ob_get_clean();
		endif;

		return false;

	}

}

new Cooked_Taxonomies_Extended();


